@@ -1,5 +1,5 @@
 /*
- * $Id: action.h,v 1.4 2004/08/24 08:45:10 janakj Exp $
+ * $Id: action.h,v 1.6.2.1 2007/08/29 17:14:30 andrei Exp $
  *
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -21,8 +21,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
@@ -30,11 +30,34 @@
 #ifndef action_h
 #define action_h
 
-#include "parser/msg_parser.h"
+#define USE_LONGJMP
+
 #include "route_struct.h"
 
-int do_action(struct action* a, struct sip_msg* msg);
-int run_actions(struct action* a, struct sip_msg* msg);
+#include "parser/msg_parser.h"
+
+#ifdef USE_LONGJMP
+#include <setjmp.h>
+#endif
+
+
+struct run_act_ctx{
+	int rec_lev;
+	int run_flags;
+	int last_retcode; /* return from last route */
+#ifdef USE_LONGJMP
+	jmp_buf jmp_env;
+#endif
+};
+
+
+#define init_run_actions_ctx(ph) \
+	do{\
+		(ph)->rec_lev=(ph)->run_flags=(ph)->last_retcode=0; \
+	}while(0)
+
+int do_action(struct run_act_ctx* c, struct action* a, struct sip_msg* msg);
+int run_actions(struct run_act_ctx* c, struct action* a, struct sip_msg* msg);
 
 
 
