@@ -1,5 +1,5 @@
 /*
- * $Id: dset.c,v 1.15.2.2 2006/12/15 09:12:38 andrei Exp $
+ * $Id: dset.c,v 1.19 2006/12/15 09:09:11 andrei Exp $
  *
  * destination set
  *
@@ -37,6 +37,7 @@
 #include "error.h"
 #include "dset.h"
 #include "mem/mem.h"
+#include "ip_addr.h"
 
 #define CONTACT "Contact: "
 #define CONTACT_LEN (sizeof(CONTACT) - 1)
@@ -58,6 +59,7 @@ struct branch
 
 	int q; /* Preference of the contact among
 		* contact within the array */
+	struct socket_info* force_send_socket;
 };
 
 
@@ -93,7 +95,7 @@ void init_branch_iterator(void)
  * array, 0 is returned if there are no
  * more branches
  */
-char* next_branch(int* len, qvalue_t* q, char** dst_uri, int* dst_len)
+char* next_branch(int* len, qvalue_t* q, char** dst_uri, int* dst_len, struct socket_info** force_socket)
 {
 	unsigned int i;
 
@@ -106,6 +108,9 @@ char* next_branch(int* len, qvalue_t* q, char** dst_uri, int* dst_len)
 			*dst_uri = branches[i].dst_uri;
 			*dst_len = branches[i].dst_uri_len;
 		}
+		if (force_socket) {
+			*force_socket = branches[i].force_send_socket;
+		}
 		return branches[i].uri;
 	} else {
 		*len = 0;
@@ -114,6 +119,9 @@ char* next_branch(int* len, qvalue_t* q, char** dst_uri, int* dst_len)
 			*dst_uri = 0;
 			*dst_len = 0;
 		}
+		if (force_socket) {
+			*force_socket = 0;
+		}
 		return 0;
 	}
 }
@@ -132,7 +140,8 @@ void clear_branches(void)
 /* 
  * Add a new branch to current transaction 
  */
-int append_branch(struct sip_msg* msg, char* uri, int uri_len, char* dst_uri, int dst_uri_len, qvalue_t q)
+int append_branch(struct sip_msg* msg, char* uri, int uri_len, char* dst_uri, int dst_uri_len, 
+		  qvalue_t q, struct socket_info* force_socket)
 {
 	     /* if we have already set up the maximum number
 	      * of branches, don't try new ones 
@@ -181,6 +190,8 @@ int append_branch(struct sip_msg* msg, char* uri, int uri_len, char* dst_uri, in
  		branches[nr_branches].dst_uri_len = 0;
 	}
 
+	branches[nr_branches].force_send_socket = force_socket;
+	
 	nr_branches++;
 	return 1;
 }
@@ -211,7 +222,7 @@ char* print_dset(struct sip_msg* msg, int* len)
 	}
 
 	init_branch_iterator();
-	while ((uri.s = next_branch(&uri.len, &q, 0, 0))) {
+	while ((uri.s = next_branch(&uri.len, &q, 0, 0, 0))) {
 		cnt++;
 		*len += uri.len;
 		if (q != Q_UNSPECIFIED) {
@@ -252,7 +263,7 @@ char* print_dset(struct sip_msg* msg, int* len)
 	}
 
 	init_branch_iterator();
-	while ((uri.s = next_branch(&uri.len, &q, 0, 0))) {
+	while ((uri.s = next_branch(&uri.len, &q, 0, 0, 0))) {
 		if (i) {
 			memcpy(p, CONTACT_DELIM, CONTACT_DELIM_LEN);
 			p += CONTACT_DELIM_LEN;
@@ -326,7 +337,7 @@ int rewrite_uri(struct sip_msg* _m, str* _s)
 
         buf = (char*)pkg_malloc(_s->len + 1);
         if (!buf) {
-                LOG(L_ERR, "ERROR: TOI: rewrite_uri: No memory left\n");
+                LOG(L_ERR, "ERROR: rewrite_uri: No memory left\n");
                 return -1;
         }
 
@@ -341,7 +352,7 @@ int rewrite_uri(struct sip_msg* _m, str* _s)
         _m->new_uri.s = buf;
         _m->new_uri.len = _s->len;
 
-        DBG("TOI: rewrite_uri: Rewriting Request-URI with '%.*s'\n", _s->len, 
+        DBG("rewrite_uri: Rewriting Request-URI with '%.*s'\n", _s->len, 
 																		   buf);
         return 1;
 }
