@@ -1,5 +1,5 @@
 /*
- * $Id: forward.c,v 1.98.2.2 2007/10/10 19:40:21 andrei Exp $
+ * $Id: forward.c,v 1.106.2.3 2008/04/17 21:33:50 calrissian Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -46,6 +46,14 @@
  *  2003-10-24  converted to the new socket_info lists (andrei)
  *  2004-10-10  modified check_self to use grep_sock_info (andrei)
  *  2004-11-08  added force_send_socket support in get_send_socket (andrei)
+ *  2005-12-11  onsend_router support; forward_request to no longer
+ *              pkg_malloc'ed (andrei)
+ *  2006-04-12  forward_{request,reply} use now struct dest_info (andrei)
+ *  2006-04-21  basic comp via param support (andrei)
+ *  2006-07-31  forward_request can resolve destination on its own, uses the 
+ *              dns cache and falls back on send error to other ips (andrei)
+ *  2007-10-08  get_send_socket() will ignore force_send_socket if the forced
+ *               socket is multicast (andrei)
  */
 
 
@@ -75,6 +83,14 @@
 #include "resolve.h"
 #include "name_alias.h"
 #include "socket_info.h"
+#include "onsend.h"
+#include "resolve.h"
+#ifdef USE_DNS_FAILOVER
+#include "dns_cache.h"
+#endif
+#ifdef USE_DST_BLACKLIST
+#include "dst_blacklist.h"
+#endif
 
 #ifdef DEBUG_DMALLOC
 #include <dmalloc.h>
@@ -157,11 +173,14 @@ struct socket_info* get_send_socket(struct sip_msg *msg,
 				goto not_forced;
 			}
 		}
-		if (msg->force_send_socket->socket!=-1)
+		if ((msg->force_send_socket->socket!=-1) &&
+					!(msg->force_send_socket->flags & SI_IS_MCAST))
 				return msg->force_send_socket;
-		else
-			LOG(L_WARN, "WARNING: get_send_socket: not listening"
-						 " on the requested socket, no fork mode?\n");
+		else{
+			if (!(msg->force_send_socket->flags & SI_IS_MCAST))
+				LOG(L_WARN, "WARNING: get_send_socket: not listening"
+							 " on the requested socket, no fork mode?\n");
+		}
 	};
 not_forced:
 	if (mhomed && proto==PROTO_UDP){
@@ -255,51 +274,82 @@ found:
 	return 1;
 }
 
+/* checks if the proto:port is one of the ports we listen on;
+ * if proto==0 (PROTO_NONE) the protocol is ignored
+ * returns 1 if true, 0 if false, -1 on error
+ */
+int check_self_port(unsigned short port, unsigned short proto)
+{
+	if (grep_sock_info_by_port(port, proto))
+		/* as aliases do not contain different ports we can skip them */
+		return 1;
+	else
+		return 0;
+}
 
 
-int forward_request( struct sip_msg* msg, struct proxy_l * p, int proto)
+
+/* forwards a request to dst
+ * parameters:
+ *   msg       - sip msg
+ *   dst       - destination name, if non-null it will be resolved and
+ *               send_info updated with the ip/port. Even if dst is non
+ *               null send_info must contain the protocol and if a non
+ *               default port or non srv. lookup is desired, the port must
+ *               be !=0 
+ *   port      - used only if dst!=0 (else the port in send_info->to is used)
+ *   send_info - filled dest_info structure:
+ *               if the send_socket memeber is null, a send_socket will be 
+ *               choosen automatically
+ * WARNING: don't forget to zero-fill all the  unused members (a non-zero 
+ * random id along with proto==PROTO_TCP can have bad consequences, same for
+ *   a bogus send_socket value)
+ */
+int forward_request(struct sip_msg* msg, str* dst, unsigned short port,
+							struct dest_info* send_info)
 {
 	unsigned int len;
 	char* buf;
-	union sockaddr_union* to;
-	struct socket_info* send_sock;
 	char md5[MD5_LEN];
-	int id; /* used as branch for tcp! */
+	struct socket_info* orig_send_sock; /* initial send_sock */
+	int ret;
+	struct ip_addr ip; /* debugging only */
+#ifdef USE_DNS_FAILOVER
+	struct socket_info* prev_send_sock;
+	int err;
+	struct dns_srv_handle dns_srv_h;
+	
+	prev_send_sock=0;
+	err=0;
+#endif
+	
 	
-	to=0;
 	buf=0;
-	id=0;
-	
-	to=(union sockaddr_union*)pkg_malloc(sizeof(union sockaddr_union));
-	if (to==0){
-		ser_error=E_OUT_OF_MEM;
-		LOG(L_ERR, "ERROR: forward_request: out of memory\n");
-		goto error;
-	}
-	
-	
-	/* if error try next ip address if possible */
-	if (p->ok==0){
-		if (p->host.h_addr_list[p->addr_idx+1])
-			p->addr_idx++;
-		else p->addr_idx=0;
-		p->ok=1;
-	}
-	
-	hostent2su(to, &p->host, p->addr_idx, 
-				(p->port)?p->port:SIP_PORT);
-	p->tx++;
-	p->tx_bytes+=len;
-	
-
-	send_sock=get_send_socket(msg, to, proto);
-	if (send_sock==0){
-		LOG(L_ERR, "forward_req: ERROR: cannot forward to af %d, proto %d "
-				"no corresponding listening socket\n", to->s.sa_family, proto);
-		ser_error=E_NO_SOCKET;
-		goto error1;
-	}
+	orig_send_sock=send_info->send_sock;
+	ret=0;
 
+	if(dst){
+#ifdef USE_DNS_FAILOVER
+		if (use_dns_failover){
+			dns_srv_handle_init(&dns_srv_h);
+			err=dns_sip_resolve2su(&dns_srv_h, &send_info->to, dst, port,
+									send_info->proto, dns_flags);
+			if (err!=0){
+				LOG(L_ERR, "ERROR: forward_request: resolving \"%.*s\""
+						" failed: %s [%d]\n", dst->len, ZSW(dst->s),
+						dns_strerror(err), err);
+				ret=E_BAD_ADDRESS;
+				goto error;
+			}
+		}else
+#endif
+		if (sip_hostport2su(&send_info->to, dst, port, send_info->proto)<0){
+			LOG(L_ERR, "ERROR: forward_request: bad host name %.*s,"
+						" dropping packet\n", dst->len, ZSW(dst->s));
+			ret=E_BAD_ADDRESS;
+			goto error;
+		}
+	}/* dst */
 	/* calculate branch for outbound request;  if syn_branch is turned off,
 	   calculate is from transaction key, i.e., as an md5 of From/To/CallID/
 	   CSeq exactly the same way as TM does; good for reboot -- than messages
@@ -315,46 +365,130 @@ int forward_request( struct sip_msg* msg, struct proxy_l * p, int proto)
 	} else {
 		if (!char_msg_val( msg, md5 )) 	{ /* parses transaction key */
 			LOG(L_ERR, "ERROR: forward_request: char_msg_val failed\n");
-			goto error1;
+			ret=E_UNSPEC;
+			goto error;
 		}
 		msg->hash_index=hash( msg->callid->body, get_cseq(msg)->number);
-		if (!branch_builder( msg->hash_index, 0, md5, id /* 0-th branch */,
+		if (!branch_builder( msg->hash_index, 0, md5, 0 /* 0-th branch */,
 					msg->add_to_branch_s, &msg->add_to_branch_len )) {
 			LOG(L_ERR, "ERROR: forward_request: branch_builder failed\n");
-			goto error1;
+			ret=E_UNSPEC;
+			goto error;
 		}
 	}
-
-	buf = build_req_buf_from_sip_req( msg, &len, send_sock,  proto);
-	if (!buf){
-		LOG(L_ERR, "ERROR: forward_request: building failed\n");
-		goto error1;
+	/* try to send the message until success or all the ips are exhausted
+	 *  (if dns lookup is peformed && the dns cache used ) */
+#ifdef USE_DNS_FAILOVER
+	do{
+#endif
+		if (orig_send_sock==0) /* no forced send_sock => find it **/
+			send_info->send_sock=get_send_socket(msg, &send_info->to,
+												send_info->proto);
+		if (send_info->send_sock==0){
+			LOG(L_ERR, "forward_req: ERROR: cannot forward to af %d, proto %d "
+						"no corresponding listening socket\n",
+						send_info->to.s.sa_family, send_info->proto);
+			ret=ser_error=E_NO_SOCKET;
+#ifdef USE_DNS_FAILOVER
+			/* continue, maybe we find a socket for some other ip */
+			continue;
+#else
+			goto error;
+#endif
+		}
+	
+#ifdef USE_DNS_FAILOVER
+		if (prev_send_sock!=send_info->send_sock){
+			/* rebuild the message only if the send_sock changed */
+			prev_send_sock=send_info->send_sock;
+#endif
+			if (buf) pkg_free(buf);
+			buf = build_req_buf_from_sip_req(msg, &len, send_info);
+			if (!buf){
+				LOG(L_ERR, "ERROR: forward_request: building failed\n");
+				ret=E_OUT_OF_MEM; /* most probable */
+				goto error;
+			}
+#ifdef USE_DNS_FAILOVER
+		}
+#endif
+		 /* send it! */
+		DBG("Sending:\n%.*s.\n", (int)len, buf);
+		DBG("orig. len=%d, new_len=%d, proto=%d\n",
+				msg->len, len, send_info->proto );
+	
+		if (run_onsend(msg, send_info, buf, len)==0){
+			su2ip_addr(&ip, &send_info->to);
+			LOG(L_INFO, "forward_request: request to %s:%d(%d) dropped"
+					" (onsend_route)\n", ip_addr2a(&ip),
+						su_getport(&send_info->to), send_info->proto);
+			ser_error=E_OK; /* no error */
+			ret=E_ADM_PROHIBITED;
+#ifdef USE_DNS_FAILOVER
+			continue; /* try another ip */
+#else
+			goto error; /* error ? */
+#endif
+		}
+#ifdef USE_DST_BLACKLIST
+		if (use_dst_blacklist){
+			if (dst_is_blacklisted(send_info)){
+				su2ip_addr(&ip, &send_info->to);
+				LOG(L_ERR, "ERROR: blacklisted destination:%s:%d (%d)\n",
+							ip_addr2a(&ip), su_getport(&send_info->to),
+							send_info->proto);
+				ret=ser_error=E_SEND;
+#ifdef USE_DNS_FAILOVER
+				continue; /* try another ip */
+#else
+				goto error;
+#endif
+			}
+		}
+#endif
+		if (msg_send(send_info, buf, len)<0){
+			ret=ser_error=E_SEND;
+#ifdef USE_DST_BLACKLIST
+			if (use_dst_blacklist)
+				dst_blacklist_add(BLST_ERR_SEND, send_info);
+#endif
+#ifdef USE_DNS_FAILOVER
+			continue; /* try another ip */
+#else
+			goto error;
+#endif
+		}else{
+			ret=ser_error=E_OK;
+			/* sent requests stats */
+			STATS_TX_REQUEST(  msg->first_line.u.request.method_value );
+			/* exit succcesfully */
+			goto end;
+		}
+#ifdef USE_DNS_FAILOVER
+	}while(dst && use_dns_failover && dns_srv_handle_next(&dns_srv_h, err) && 
+			((err=dns_sip_resolve2su(&dns_srv_h, &send_info->to, dst, port,
+								  send_info->proto, dns_flags))==0));
+	if ((err!=0) && (err!=-E_DNS_EOR)){
+		LOG(L_ERR, "ERROR:  resolving %.*s host name in uri"
+							" failed: %s [%d] (dropping packet)\n",
+									dst->len, ZSW(dst->s),
+									dns_strerror(err), err);
+		ret=ser_error=E_BAD_ADDRESS;
+		goto error;
 	}
-	 /* send it! */
-	DBG("Sending:\n%.*s.\n", (int)len, buf);
-	DBG("orig. len=%d, new_len=%d, proto=%d\n", msg->len, len, proto );
+#endif
 	
-	if (msg_send(send_sock, proto, to, 0, buf, len)<0){
-		ser_error=E_SEND;
-		p->errors++;
-		p->ok=0;
-		STATS_TX_DROPS;
-		goto error1;
-	}
-	
-	/* sent requests stats */
-	STATS_TX_REQUEST(  msg->first_line.u.request.method_value );
-	
-	pkg_free(buf);
-	pkg_free(to);
-	/* received_buf & line_buf will be freed in receive_msg by free_lump_list*/
-	return 0;
-
-error1:
-	pkg_free(to);
 error:
+	STATS_TX_DROPS;
+end:
+#ifdef USE_DNS_FAILOVER
+	if (dst && use_dns_failover){
+				dns_srv_handle_put(&dns_srv_h);
+	}
+#endif
 	if (buf) pkg_free(buf);
-	return -1;
+	/* received_buf & line_buf will be freed in receive_msg by free_lump_list*/
+	return ret;
 }
 
 
@@ -428,18 +562,14 @@ int update_sock_struct_from_via( union sockaddr_union* to,
 int forward_reply(struct sip_msg* msg)
 {
 	char* new_buf;
-	union sockaddr_union* to;
+	struct dest_info dst;
 	unsigned int new_len;
 	struct sr_module *mod;
-	int proto;
-	unsigned int id; /* used only by tcp*/
 #ifdef USE_TCP
 	char* s;
 	int len;
 #endif
-	
-	to=0;
-	id=0;
+	init_dest_info(&dst);
 	new_buf=0;
 	/*check if first via host = us */
 	if (check_via){
@@ -463,7 +593,7 @@ int forward_reply(struct sip_msg* msg)
 	}
 
 	/* we have to forward the reply stateless, so we need second via -bogdan*/
-	if (parse_headers( msg, HDR_VIA2, 0 )==-1 
+	if (parse_headers( msg, HDR_VIA2_F, 0 )==-1 
 		|| (msg->via2==0) || (msg->via2->error!=PARSE_OK))
 	{
 		/* no second via => error */
@@ -471,44 +601,39 @@ int forward_reply(struct sip_msg* msg)
 		goto error;
 	}
 
-	to=(union sockaddr_union*)pkg_malloc(sizeof(union sockaddr_union));
-	if (to==0){
-		LOG(L_ERR, "ERROR: forward_reply: out of memory\n");
-		goto error;
-	}
-
 	new_buf = build_res_buf_from_sip_res( msg, &new_len);
 	if (!new_buf){
 		LOG(L_ERR, "ERROR: forward_reply: building failed\n");
 		goto error;
 	}
 
-	proto=msg->via2->proto;
-	if (update_sock_struct_from_via( to, msg, msg->via2 )==-1) goto error;
-
+	dst.proto=msg->via2->proto;
+	if (update_sock_struct_from_via( &dst.to, msg, msg->via2 )==-1) goto error;
+#ifdef USE_COMP
+	dst.comp=msg->via2->comp_no;
+#endif
 
 #ifdef USE_TCP
-	if (proto==PROTO_TCP
+	if (dst.proto==PROTO_TCP
 #ifdef USE_TLS
-			|| proto==PROTO_TLS
+			|| dst.proto==PROTO_TLS
 #endif
 			){
 		/* find id in i param if it exists */
-		if (msg->via1->i&&msg->via1->i->value.s){
+		if (msg->via1->i && msg->via1->i->value.s){
 			s=msg->via1->i->value.s;
 			len=msg->via1->i->value.len;
 			DBG("forward_reply: i=%.*s\n",len, ZSW(s));
-			if (reverse_hex2int(s, len, &id)<0){
+			if (reverse_hex2int(s, len, (unsigned int*)&dst.id)<0){
 				LOG(L_ERR, "ERROR: forward_reply: bad via i param \"%.*s\"\n",
 						len, ZSW(s));
-					id=0;
+					dst.id=0;
 			}
-			DBG("forward_reply: id= %x\n", id);
 		}		
 				
 	} 
 #endif
-	if (msg_send(0, proto, to, (int)id, new_buf, new_len)<0) goto error;
+	if (msg_send(&dst, new_buf, new_len)<0) goto error;
 #ifdef STATS
 	STATS_TX_RESPONSE(  (msg->first_line.u.reply.statuscode/100) );
 #endif
@@ -518,11 +643,9 @@ int forward_reply(struct sip_msg* msg)
 			(unsigned short) msg->via2->port);
 
 	pkg_free(new_buf);
-	pkg_free(to);
 skip:
 	return 0;
 error:
 	if (new_buf) pkg_free(new_buf);
-	if (to) pkg_free(to);
 	return -1;
 }
