@@ -1,5 +1,5 @@
 /*
- * $Id: globals.h,v 1.52.2.1 2006/10/25 19:33:32 andrei Exp $
+ * $Id: globals.h,v 1.67.2.4 2008/04/15 16:14:55 andrei Exp $
  *
  * global variables
  *
@@ -36,6 +36,7 @@
 #include "types.h"
 #include "ip_addr.h"
 #include "str.h"
+#include "poll_types.h"
 
 #define NO_DNS     0
 #define DO_DNS     1
@@ -48,12 +49,12 @@ extern int config_check;
 extern char *stat_file;
 extern unsigned short port_no;
 
+extern pid_t creator_pid;  /* pid of first process before daemonization */
 extern int uid;
 extern int gid;
-char* pid_file;
-char* pgid_file;
-extern int own_pgid; /* whether or not we have our own pgid (and it's ok
->--->--->--->--->--->--->--->--->--->--->--- to use kill(0, sig) */
+extern char* pid_file;
+extern char* pgid_file;
+extern int own_pgid; /* whether or not we have our own pgid (and it's ok to use kill(0, sig) */
 
 extern struct socket_info* bind_address; /* pointer to the crt. proc.
 											listening address */
@@ -75,21 +76,29 @@ extern struct socket_info* sendipv6_tls; /* same as above for ipv6 */
 extern unsigned int maxbuffer;
 extern int children_no;
 #ifdef USE_TCP
+extern int tcp_main_pid;
 extern int tcp_children_no;
 extern int tcp_disable;
 extern int tcp_accept_aliases;
 extern int tcp_connect_timeout;
 extern int tcp_send_timeout;
+extern int tcp_con_lifetime; /* connection lifetime */
+extern enum poll_types tcp_poll_method;
+extern int tcp_max_fd_no;
+extern int tcp_max_connections;
 #endif
 #ifdef USE_TLS
 extern int tls_disable;
 extern unsigned short tls_port_no;
 #endif
 extern int dont_fork;
+extern int dont_daemonize;
 extern int check_via;
+extern int phone2tel;
 extern int received_dns;
 extern int syn_branch;
 /* extern int process_no; */
+extern int child_rank;
 extern int sip_warning;
 extern int server_signature;
 extern char* user;
@@ -107,6 +116,15 @@ extern int mcast_loopback;
 extern int mcast_ttl;
 #endif /* USE_MCAST */
 
+#ifdef USE_STUN
+extern unsigned int stun_refresh_interval;
+extern int stun_allow_stun;
+extern int stun_allow_fp;
+#endif
+
+extern int tos;
+extern int pmtu_discovery;
+
 /*
  * debug & log_stderr moved to dprint.h*/
 
@@ -119,17 +137,6 @@ extern unsigned int msg_no;
 
 extern unsigned long shm_mem_size;
 
-/* FIFO server config */
-char extern *fifo; /* FIFO name */
-extern int fifo_mode;
-char extern *fifo_dir; /* dir. where  reply fifos are allowed */
-extern char *fifo_db_url;  /* db url used by db_fifo interface */
-
-/* UNIX domain socket configuration */
-extern char *unixsock_name;   /* The name of the socket */
-extern int unixsock_children; /* The number of listening children */
-extern int unixsock_tx_timeout; /* Timeout (in ms) used when sending data */
-
 /* AVP configuration */
 extern char *avp_db_url;  /* db url used by user preferences (AVPs) */
 
@@ -141,9 +148,15 @@ extern int process_no;
 extern int reply_to_via;
 
 extern int is_main;
+extern int fixup_complete;
 
 /* debugging level for dumping memory status */
 extern int memlog;
+/* debugging level for malloc debugging messages */
+extern int memdbg;
+
+/* debugging level for timer debugging (see -DTIMER_DEBUG) */
+extern int timerlog;
 
 /* looking up outbound interface ? */
 extern int mhomed;
@@ -157,8 +170,37 @@ extern str default_global_address;
 /* pre-ser ports */
 extern str default_global_port;
 
+/* how much time to allow for shutdown, before killing everything */
+extern int ser_kill_timeout;
+
 /* core dump and file limits */
 extern int disable_core_dump;
 extern int open_files_limit;
 
+/* resolver */
+extern int dns_retr_time;
+extern int dns_retr_no;
+extern int dns_servers_no;
+extern int dns_search_list;
+#ifdef USE_DNS_CACHE
+extern int use_dns_cache; /* 1 if the cache is enabled, 0 otherwise */
+extern int use_dns_failover; /* 1 if failover is enabled, 0 otherwise */
+extern unsigned int dns_cache_max_mem; /* maximum memory used for the 
+										  cached entries*/
+extern unsigned int dns_neg_cache_ttl; /* neg. cache ttl */
+extern unsigned int dns_cache_max_ttl; /* maximum ttl */
+extern unsigned int dns_cache_min_ttl; /* minimum ttl */
+extern unsigned int dns_timer_interval; /* gc timer interval in s */
+extern int dns_flags; /* default flags used for the  dns_*resolvehost 
+                    (compatibility wrappers) */
+#endif
+#ifdef USE_DST_BLACKLIST
+extern int use_dst_blacklist; /* 1 if the blacklist is enabled */
+extern unsigned int  blst_max_mem; /* maximum memory used for the blacklist
+									  entries*/
+extern unsigned int blst_timeout; /* blacklist entry ttl */
+extern unsigned int blst_timer_interval; /* blacklist gc timer interval 
+											(in s)*/
+#endif
+
 #endif
