@@ -1,4 +1,4 @@
-/* $Id: f_malloc.c,v 1.17.2.2 2006/02/03 21:08:08 andrei Exp $
+/* $Id: f_malloc.c,v 1.25 2006/04/07 14:55:27 andrei Exp $
  *
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -33,9 +33,11 @@
  *               memory blocks (64 bits machine & size >=2^32) 
  *              GET_HASH s/</<=/ (avoids waste of 1 hash cell)   (andrei)
  *  2004-11-10  support for > 4Gb mem., switched to long (andrei)
+ *  2005-03-02  added fm_info() (andrei)
  *  2005-12-12  fixed realloc shrink real_used accounting (andrei)
  *              fixed initial size (andrei)
  *  2006-02-03  fixed realloc out of mem. free bug (andrei)
+ *  2006-04-07  s/DBG/MDBG (andrei)
  */
 
 
@@ -47,6 +49,7 @@
 #include "f_malloc.h"
 #include "../dprint.h"
 #include "../globals.h"
+#include "memdbg.h"
 
 
 /*useful macros*/
@@ -164,8 +167,10 @@ void fm_split_frag(struct fm_block* qm, struct fm_frag* frag,
 		n=FRAG_NEXT(frag);
 		n->size=rest-FRAG_OVERHEAD;
 		FRAG_CLEAR_USED(n); /* never used */
-#ifdef DBG_F_MALLOC
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 		qm->real_used+=FRAG_OVERHEAD;
+#endif
+#ifdef DBG_F_MALLOC
 		/* frag created by malloc, mark it*/
 		n->file=file;
 		n->func="frag. from fm_malloc";
@@ -214,7 +219,7 @@ struct fm_block* fm_malloc_init(char* address, unsigned long size)
 	qm=(struct fm_block*)start;
 	memset(qm, 0, sizeof(struct fm_block));
 	qm->size=size;
-#ifdef DBG_F_MALLOC
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 	qm->real_used=init_overhead;
 	qm->max_real_used=qm->real_used;
 #endif
@@ -253,7 +258,7 @@ void* fm_malloc(struct fm_block* qm, unsigned long size)
 	int hash;
 	
 #ifdef DBG_F_MALLOC
-	DBG("fm_malloc(%p, %lu) called from %s: %s(%d)\n", qm, size, file, func,
+	MDBG("fm_malloc(%p, %lu) called from %s: %s(%d)\n", qm, size, file, func,
 			line);
 #endif
 	/*size must be a multiple of 8*/
@@ -284,20 +289,21 @@ found:
 	
 #ifdef DBG_F_MALLOC
 	fm_split_frag(qm, frag, size, file, func, line);
-	qm->real_used+=frag->size;
-	qm->used+=frag->size;
-
-	if (qm->max_real_used<qm->real_used)
-		qm->max_real_used=qm->real_used;
 
 	frag->file=file;
 	frag->func=func;
 	frag->line=line;
 	frag->check=ST_CHECK_PATTERN;
-	DBG("fm_malloc(%p, %lu) returns address %p \n", qm, size,
+	MDBG("fm_malloc(%p, %lu) returns address %p \n", qm, size,
 		(char*)frag+sizeof(struct fm_frag));
 #else
 	fm_split_frag(qm, frag, size);
+#endif
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
+	qm->real_used+=frag->size;
+	qm->used+=frag->size;
+	if (qm->max_real_used<qm->real_used)
+		qm->max_real_used=qm->real_used;
 #endif
 	FRAG_MARK_USED(frag); /* mark it as used */
 	return (char*)frag+sizeof(struct fm_frag);
@@ -316,7 +322,7 @@ void fm_free(struct fm_block* qm, void* p)
 	unsigned long size;
 
 #ifdef DBG_F_MALLOC
-	DBG("fm_free(%p, %p), called from %s: %s(%d)\n", qm, p, file, func, line);
+	MDBG("fm_free(%p, %p), called from %s: %s(%d)\n", qm, p, file, func, line);
 	if (p>(void*)qm->last_frag || p<(void*)qm->first_frag){
 		LOG(L_CRIT, "BUG: fm_free: bad pointer %p (out of memory block!) - "
 				"aborting\n", p);
@@ -329,14 +335,15 @@ void fm_free(struct fm_block* qm, void* p)
 	}
 	f=(struct fm_frag*) ((char*)p-sizeof(struct fm_frag));
 #ifdef DBG_F_MALLOC
-	DBG("fm_free: freeing block alloc'ed from %s: %s(%ld)\n", f->file, f->func,
-			f->line);
+	MDBG("fm_free: freeing block alloc'ed from %s: %s(%ld)\n",
+			f->file, f->func, f->line);
 #endif
 	size=f->size;
-
-#ifdef DBG_F_MALLOC
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 	qm->used-=size;
 	qm->real_used-=size;
+#endif
+#ifdef DBG_F_MALLOC
 	f->file=file;
 	f->func=func;
 	f->line=line;
@@ -361,7 +368,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 	int hash;
 	
 #ifdef DBG_F_MALLOC
-	DBG("fm_realloc(%p, %p, %lu) called from %s: %s(%d)\n", qm, p, size,
+	MDBG("fm_realloc(%p, %p, %lu) called from %s: %s(%d)\n", qm, p, size,
 			file, func, line);
 	if ((p)&&(p>(void*)qm->last_frag || p<(void*)qm->first_frag)){
 		LOG(L_CRIT, "BUG: fm_free: bad pointer %p (out of memory block!) - "
@@ -386,7 +393,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 #endif
 	f=(struct fm_frag*) ((char*)p-sizeof(struct fm_frag));
 #ifdef DBG_F_MALLOC
-	DBG("fm_realloc: realloc'ing frag %p alloc'ed from %s: %s(%ld)\n",
+	MDBG("fm_realloc: realloc'ing frag %p alloc'ed from %s: %s(%ld)\n",
 			f, f->file, f->func, f->line);
 #endif
 	size=ROUNDUP(size);
@@ -394,17 +401,19 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 	if (f->size > size){
 		/* shrink */
 #ifdef DBG_F_MALLOC
-		DBG("fm_realloc: shrinking from %lu to %lu\n", f->size, size);
+		MDBG("fm_realloc: shrinking from %lu to %lu\n", f->size, size);
 		fm_split_frag(qm, f, size, file, "frag. from fm_realloc", line);
-		qm->real_used-=(orig_size-f->size-FRAG_OVERHEAD);
-		qm->used-=(orig_size-f->size);
 #else
 		fm_split_frag(qm, f, size);
+#endif
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
+		qm->real_used-=(orig_size-f->size-FRAG_OVERHEAD);
+		qm->used-=(orig_size-f->size);
 #endif
 	}else if (f->size<size){
 		/* grow */
 #ifdef DBG_F_MALLOC
-		DBG("fm_realloc: growing from %lu to %lu\n", f->size, size);
+		MDBG("fm_realloc: growing from %lu to %lu\n", f->size, size);
 #endif
 		diff=size-f->size;
 		n=FRAG_NEXT(f);
@@ -427,7 +436,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 			qm->free_hash[hash].no--;
 			/* join */
 			f->size+=n->size+FRAG_OVERHEAD;
-		#ifdef DBG_F_MALLOC
+		#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 			qm->real_used-=FRAG_OVERHEAD;
 		#endif
 			/* split it if necessary */
@@ -439,7 +448,7 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 				fm_split_frag(qm, f, size);
 		#endif
 			}
-		#ifdef DBG_F_MALLOC
+		#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 			qm->real_used+=(f->size-orig_size);
 			qm->used+=(f->size-orig_size);
 		#endif
@@ -464,12 +473,12 @@ void* fm_realloc(struct fm_block* qm, void* p, unsigned long size)
 	}else{
 		/* do nothing */
 #ifdef DBG_F_MALLOC
-		DBG("fm_realloc: doing nothing, same size: %lu - %lu\n", 
+		MDBG("fm_realloc: doing nothing, same size: %lu - %lu\n", 
 				f->size, size);
 #endif
 	}
 #ifdef DBG_F_MALLOC
-	DBG("fm_realloc: returning %p\n", p);
+	MDBG("fm_realloc: returning %p\n", p);
 #endif
 	return p;
 }
@@ -488,7 +497,7 @@ void fm_status(struct fm_block* qm)
 	if (!qm) return;
 
 	LOG(memlog, " heap size= %ld\n", qm->size);
-#ifdef DBG_F_MALLOC
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
 	LOG(memlog, " used= %lu, used+overhead=%lu, free=%lu\n",
 			qm->used, qm->real_used, qm->size-qm->real_used);
 	LOG(memlog, " max used (+overhead)= %lu\n", qm->max_real_used);
@@ -550,5 +559,46 @@ void fm_status(struct fm_block* qm)
 
 
 
+/* fills a malloc info structure with info about the block
+ * if a parameter is not supported, it will be filled with 0 */
+void fm_info(struct fm_block* qm, struct mem_info* info)
+{
+	int r;
+	long total_frags;
+#if !defined(DBG_F_MALLOC) && !defined(MALLOC_STATS)
+	struct fm_frag* f;
+#endif
+	
+	memset(info,0, sizeof(*info));
+	total_frags=0;
+	info->total_size=qm->size;
+	info->min_frag=MIN_FRAG_SIZE;
+#if defined(DBG_F_MALLOC) || defined(MALLOC_STATS)
+	info->free=qm->size-qm->real_used;
+	info->used=qm->used;
+	info->real_used=qm->real_used;
+	info->max_used=qm->max_real_used;
+	for(r=0;r<F_HASH_SIZE; r++){
+		total_frags+=qm->free_hash[r].no;
+	}
+#else
+	/* we'll have to compute it all */
+	for (r=0; r<=F_MALLOC_OPTIMIZE/ROUNDTO; r++){
+		info->free+=qm->free_hash[r].no*UN_HASH(r);
+		total_frags+=qm->free_hash[r].no;
+	}
+	for(;r<F_HASH_SIZE; r++){
+		total_frags+=qm->free_hash[r].no;
+		for(f=qm->free_hash[r].first;f;f=f->u.nxt_free){
+			info->free+=f->size;
+		}
+	}
+	info->real_used=info->total_size-info->free;
+	info->used=0; /* we don't really now */
+	info->max_used=0; /* we don't really now */
+#endif
+	info->total_frags=total_frags;
+}
+
 
 #endif
