@@ -1,4 +1,4 @@
-/* $Id: q_malloc.c,v 1.20.2.2 2006/02/03 21:08:08 andrei Exp $
+/* $Id: q_malloc.c,v 1.25 2006/04/07 14:55:27 andrei Exp $
  *
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -34,9 +34,11 @@
  *               memory blocks (64 bits machine & size>=2^32) (andrei)
  *              GET_HASH s/</<=/ (avoids waste of 1 hash cell) (andrei)
  *  2004-11-10  support for > 4Gb mem., switched to long (andrei)
+ *  2005-03-02  added qm_info() (andrei)
  *  2005-12-12  fixed realloc shrink real_used & used accounting;
  *              fixed initial size (andrei)
  *  2006-02-03  fixed realloc out of mem. free bug (andrei)
+ *  2006-04-07  s/DBG/MDBG (andrei)
  */
 
 
@@ -49,6 +51,7 @@
 #include "q_malloc.h"
 #include "../dprint.h"
 #include "../globals.h"
+#include "memdbg.h"
 
 
 /*useful macros*/
@@ -363,7 +366,7 @@ void* qm_malloc(struct qm_block* qm, unsigned long size)
 	unsigned int list_cntr;
 
 	list_cntr = 0;
-	DBG("qm_malloc(%p, %lu) called from %s: %s(%d)\n", qm, size, file, func,
+	MDBG("qm_malloc(%p, %lu) called from %s: %s(%d)\n", qm, size, file, func,
 			line);
 #endif
 	/*size must be a multiple of 8*/
@@ -402,7 +405,7 @@ void* qm_malloc(struct qm_block* qm, unsigned long size)
 		f->check=ST_CHECK_PATTERN;
 		/*  FRAG_END(f)->check1=END_CHECK_PATTERN1;
 			FRAG_END(f)->check2=END_CHECK_PATTERN2;*/
-		DBG("qm_malloc(%p, %lu) returns address %p frag. %p (size=%lu) on %d"
+		MDBG("qm_malloc(%p, %lu) returns address %p frag. %p (size=%lu) on %d"
 				" -th hit\n",
 			 qm, size, (char*)f+sizeof(struct qm_frag), f, f->size, list_cntr );
 #endif
@@ -426,7 +429,7 @@ void qm_free(struct qm_block* qm, void* p)
 	unsigned long size;
 
 #ifdef DBG_QM_MALLOC
-	DBG("qm_free(%p, %p), called from %s: %s(%d)\n", qm, p, file, func, line);
+	MDBG("qm_free(%p, %p), called from %s: %s(%d)\n", qm, p, file, func, line);
 	if (p>(void*)qm->last_frag_end || p<(void*)qm->first_frag){
 		LOG(L_CRIT, "BUG: qm_free: bad pointer %p (out of memory block!) - "
 				"aborting\n", p);
@@ -447,7 +450,7 @@ void qm_free(struct qm_block* qm, void* p)
 				f->file, f->func, f->line);
 		abort();
 	}
-	DBG("qm_free: freeing frag. %p alloc'ed from %s: %s(%ld)\n",
+	MDBG("qm_free: freeing frag. %p alloc'ed from %s: %s(%ld)\n",
 			f, f->file, f->func, f->line);
 #endif
 	size=f->size;
@@ -512,7 +515,7 @@ void* qm_realloc(struct qm_block* qm, void* p, unsigned long size)
 	
 	
 #ifdef DBG_QM_MALLOC
-	DBG("qm_realloc(%p, %p, %lu) called from %s: %s(%d)\n", qm, p, size,
+	MDBG("qm_realloc(%p, %p, %lu) called from %s: %s(%d)\n", qm, p, size,
 			file, func, line);
 	if ((p)&&(p>(void*)qm->last_frag_end || p<(void*)qm->first_frag)){
 		LOG(L_CRIT, "BUG: qm_free: bad pointer %p (out of memory block!) - "
@@ -539,7 +542,7 @@ void* qm_realloc(struct qm_block* qm, void* p, unsigned long size)
 	f=(struct qm_frag*) ((char*)p-sizeof(struct qm_frag));
 #ifdef DBG_QM_MALLOC
 	qm_debug_frag(qm, f);
-	DBG("qm_realloc: realloc'ing frag %p alloc'ed from %s: %s(%ld)\n",
+	MDBG("qm_realloc: realloc'ing frag %p alloc'ed from %s: %s(%ld)\n",
 			f, f->file, f->func, f->line);
 	if (f->u.is_free){
 		LOG(L_CRIT, "BUG:qm_realloc: trying to realloc an already freed "
@@ -553,9 +556,9 @@ void* qm_realloc(struct qm_block* qm, void* p, unsigned long size)
 		orig_size=f->size;
 		/* shrink */
 #ifdef DBG_QM_MALLOC
-		DBG("qm_realloc: shrinking from %lu to %lu\n", f->size, size);
+		MDBG("qm_realloc: shrinking from %lu to %lu\n", f->size, size);
 		if(split_frag(qm, f, size, file, "fragm. from qm_realloc", line)!=0){
-		DBG("qm_realloc : shrinked successful\n");
+		MDBG("qm_realloc : shrinked successful\n");
 #else
 		if(split_frag(qm, f, size)!=0){
 #endif
@@ -567,7 +570,7 @@ void* qm_realloc(struct qm_block* qm, void* p, unsigned long size)
 	}else if (f->size < size){
 		/* grow */
 #ifdef DBG_QM_MALLOC
-		DBG("qm_realloc: growing from %lu to %lu\n", f->size, size);
+		MDBG("qm_realloc: growing from %lu to %lu\n", f->size, size);
 #endif
 			orig_size=f->size;
 			diff=size-f->size;
@@ -613,12 +616,12 @@ void* qm_realloc(struct qm_block* qm, void* p, unsigned long size)
 	}else{
 		/* do nothing */
 #ifdef DBG_QM_MALLOC
-		DBG("qm_realloc: doing nothing, same size: %lu - %lu\n",
+		MDBG("qm_realloc: doing nothing, same size: %lu - %lu\n",
 				f->size, size);
 #endif
 	}
 #ifdef DBG_QM_MALLOC
-	DBG("qm_realloc: returning %p\n", p);
+	MDBG("qm_realloc: returning %p\n", p);
 #endif
 	return p;
 }
@@ -689,6 +692,27 @@ void qm_status(struct qm_block* qm)
 }
 
 
+/* fills a malloc info structure with info about the block
+ * if a parameter is not supported, it will be filled with 0 */
+void qm_info(struct qm_block* qm, struct mem_info* info)
+{
+	int r;
+	long total_frags;
+	
+	total_frags=0;
+	memset(info,0, sizeof(*info));
+	info->total_size=qm->size;
+	info->min_frag=MIN_FRAG_SIZE;
+	info->free=qm->size-qm->real_used;
+	info->used=qm->used;
+	info->real_used=qm->real_used;
+	info->max_used=qm->max_real_used;
+	for(r=0;r<QM_HASH_SIZE; r++){
+		total_frags+=qm->free_hash[r].no;
+	}
+	info->total_frags=total_frags;
+}
+
 
 
 #endif
