@@ -1,5 +1,5 @@
 /*
- * $Id: authrad_mod.h,v 1.6.2.4 2005/07/20 17:11:51 andrei Exp $
+ * $Id: authrad_mod.h,v 1.11 2006/12/09 00:37:08 janakj Exp $
  *
  * Digest Authentication - Radius support
  *
@@ -36,13 +36,14 @@
 #define AUTHRAD_MOD_H
 
 #include "../auth/api.h"
-#include "../../modules/acc/dict.h" /* attr & val */
+#include "../../rad_dict.h"
 
 extern struct attr attrs[];
 extern struct val vals[];
 extern void *rh;
 
-extern pre_auth_f pre_auth_func;
-extern post_auth_f post_auth_func;
+extern int use_did;
+
+extern auth_api_t auth_api;
 
 #endif /* AUTHRAD_MOD_H */
