@@ -1,5 +1,5 @@
 /*
- * $Id: avpops.c,v 1.6.2.1 2005/01/20 10:01:37 ramona Exp $
+ * $Id: avpops.c,v 1.15.2.1 2007/11/15 02:28:32 sobomax Exp $
  *
  * Copyright (C) 2004 Voice Sistem SRL
  *
@@ -29,6 +29,8 @@
  *  2004-10-04  first version (ramona)
  *  2004-11-15  added support for db schemes for avp_db_load (ramona)
  *  2004-11-17  aligned to new AVP core global aliases (ramona)
+ *  2005-01-30  "fm" (fast match) operator added (ramona)
+ *  2005-01-30  avp_copy (copy/move operation) added (ramona)
  */
 
 
@@ -40,6 +42,7 @@
 
 #include "../../mem/shm_mem.h"
 #include "../../mem/mem.h"
+#include "../../parser/parse_hname2.h"
 #include "../../sr_module.h"
 #include "../../str.h"
 #include "../../dprint.h"
@@ -55,7 +58,7 @@ MODULE_VERSION
 static char *DB_URL        = 0;  /* database url */
 static char *DB_TABLE      = 0;  /* table */
 static int  use_domain     = 0;  /* if domain should be use for avp matching */
-static char *db_columns[6] = {"uuid","attribute","value",
+static char *db_columns[6] = {"uid","name","value",
                               "type","username","domain"};
 
 
@@ -70,6 +73,8 @@ static int fixup_write_avp(void** param, int param_no);
 static int fixup_delete_avp(void** param, int param_no);
 static int fixup_pushto_avp(void** param, int param_no);
 static int fixup_check_avp(void** param, int param_no);
+static int fixup_copy_avp(void** param, int param_no);
+static int fixup_is_avp_set(void** param, int param_no);
 
 static int w_dbload_avps(struct sip_msg* msg, char* source, char* param);
 static int w_dbstore_avps(struct sip_msg* msg, char* source, char* param);
@@ -78,7 +83,9 @@ static int w_write_avps(struct sip_msg* msg, char* source, char* param);
 static int w_delete_avps(struct sip_msg* msg, char* param, char *foo);
 static int w_pushto_avps(struct sip_msg* msg, char* destination, char *param);
 static int w_check_avps(struct sip_msg* msg, char* param, char *check);
+static int w_copy_avps(struct sip_msg* msg, char* param, char *check);
 static int w_print_avps(struct sip_msg* msg, char* foo, char *bar);
+static int w_is_avp_set(struct sip_msg* msg, char* param, char *foo);
 
 
 
@@ -100,8 +107,12 @@ static cmd_export_t cmds[] = {
 									REQUEST_ROUTE|FAILURE_ROUTE},
 	{"avp_check", w_check_avps, 2, fixup_check_avp,
 									REQUEST_ROUTE|FAILURE_ROUTE},
+	{"avp_copy",  w_copy_avps, 2,  fixup_copy_avp,
+									REQUEST_ROUTE|FAILURE_ROUTE},
 	{"avp_print", w_print_avps, 0, 0,
 									REQUEST_ROUTE|FAILURE_ROUTE},
+	{"is_avp_set", w_is_avp_set, 1, fixup_is_avp_set,
+									REQUEST_ROUTE|FAILURE_ROUTE},
 	{0, 0, 0, 0, 0}
 };
 
@@ -110,17 +121,17 @@ static cmd_export_t cmds[] = {
  * Exported parameters
  */
 static param_export_t params[] = {
-	{"avp_url",           STR_PARAM, &DB_URL         },
-	{"avp_table",         STR_PARAM, &DB_TABLE       },
-	{"avp_aliases",       STR_PARAM|USE_FUNC_PARAM, (void*)register_galiases },
-	{"use_domain",        INT_PARAM, &use_domain     },
-	{"uuid_column",       STR_PARAM, &db_columns[0]  },
-	{"attribute_column",  STR_PARAM, &db_columns[1]  },
-	{"value_column",      STR_PARAM, &db_columns[2]  },
-	{"type_column",       STR_PARAM, &db_columns[3]  },
-	{"username_column",   STR_PARAM, &db_columns[4]  },
-	{"domain_column",     STR_PARAM, &db_columns[5]  },
-	{"db_scheme",         STR_PARAM|USE_FUNC_PARAM, (void*)avp_add_db_scheme },
+	{"avp_url",           PARAM_STRING, &DB_URL         },
+	{"avp_table",         PARAM_STRING, &DB_TABLE       },
+	{"avp_aliases",       PARAM_STRING|PARAM_USE_FUNC, (void*)register_galiases },
+	{"use_domain",        PARAM_INT, &use_domain     },
+	{"uid_column",        PARAM_STRING, &db_columns[0]  },
+	{"name_column",       PARAM_STRING, &db_columns[1]  },
+	{"value_column",      PARAM_STRING, &db_columns[2]  },
+	{"type_column",       PARAM_STRING, &db_columns[3]  },
+	{"username_column",   PARAM_STRING, &db_columns[4]  },
+	{"domain_column",     PARAM_STRING, &db_columns[5]  },
+	{"db_scheme",         PARAM_STRING|PARAM_USE_FUNC, (void*)avp_add_db_scheme },
 	{0, 0, 0}
 };
 
@@ -128,6 +139,7 @@ static param_export_t params[] = {
 struct module_exports exports = {
 	"avpops",
 	cmds,     /* Exported functions */
+	0,        /* RPC methods */
 	params,   /* Exported parameters */
 	avpops_init, /* Module initialization function */
 	(response_function) 0,
@@ -140,7 +152,7 @@ struct module_exports exports = {
 
 static int register_galiases( modparam_t type, void* val)
 {
-	
+
 	if (val!=0 && ((char*)val)[0]!=0)
 	{
 		if ( add_avp_galias_str((char*)val)!=0 )
@@ -265,23 +277,22 @@ static int fixup_db_avp(void** param, int param_no, int allow_scheme)
 
 		if (*s!='$')
 		{
-			/* is a constant string -> use it as uuid*/
+			/* is a constant string -> use it as uid*/
 			sp->flags = AVPOPS_VAL_STR;
-			sp->val.s = (str*)pkg_malloc(strlen(s)+1+sizeof(str));
-			if (sp->val.s==0) {
+			sp->val.s.s = pkg_malloc(strlen(s)+1);
+			if (sp->val.s.s==0) {
 				LOG(L_ERR,"ERROR:avpops:fixup_db_avp: no more pkg mem\n");
 				return E_OUT_OF_MEM;
 			}
-			sp->val.s->s = ((char*)sp->val.s) + sizeof(str);
-			sp->val.s->len = strlen(s);
-			strcpy(sp->val.s->s,s);
+			sp->val.s.len = strlen(s);
+			strcpy(sp->val.s.s,s);
 		} else {
 			/* is a variable $xxxxx */
 			s++;
 			if ( (p=strchr(s,'/'))!=0)
 				*(p++) = 0;
-			if ( (!strcasecmp( "from", s) && (flags|=AVPOPS_USE_FROM)) 
-			|| (!strcasecmp( "to", s) && (flags|=AVPOPS_USE_TO)) 
+			if ( (!strcasecmp( "from", s) && (flags|=AVPOPS_USE_FROM))
+			|| (!strcasecmp( "to", s) && (flags|=AVPOPS_USE_TO))
 			|| (!strcasecmp( "ruri", s) && (flags|=AVPOPS_USE_RURI)) )
 			{
 				/* check for extra flags/params */
@@ -351,15 +362,17 @@ static int fixup_db_store_avp(void** param, int param_no)
 
 static int fixup_write_avp(void** param, int param_no)
 {
+	struct hdr_field hdr;
 	struct fis_param *ap;
 	int  flags;
+	int  len;
 	char *s;
 	char *p;
 
 	flags=0;
 	s = (char*)*param;
 	ap = 0 ;
-	
+
 	if (param_no==1)
 	{
 		if ( *s=='$' )
@@ -368,7 +381,7 @@ static int fixup_write_avp(void** param, int param_no)
 			if ((++s)==0)
 			{
 				LOG(L_ERR,"ERROR:avops:fixup_write_avp: bad param 1; "
-					"expected : $[from|to|ruri] or int/str value\n");
+					"expected : $[from|to|ruri|hdr] or int/str value\n");
 				return E_UNSPEC;
 			}
 			if ( (p=strchr(s,'/'))!=0)
@@ -376,7 +389,9 @@ static int fixup_write_avp(void** param, int param_no)
 			if ( (!strcasecmp( "from", s) && (flags|=AVPOPS_USE_FROM))
 				|| (!strcasecmp( "to", s) && (flags|=AVPOPS_USE_TO))
 				|| (!strcasecmp( "ruri", s) && (flags|=AVPOPS_USE_RURI))
-				|| (!strcasecmp( "src_ip", s) && (flags|=AVPOPS_USE_SRC_IP)) )
+				|| (!strcasecmp( "src_ip", s) && (flags|=AVPOPS_USE_SRC_IP))
+				|| (!strcasecmp( "dst_ip", s) && (flags|=AVPOPS_USE_DST_IP))
+				|| (!strncasecmp( "hdr", s, 3) && (flags|=AVPOPS_USE_HDRREQ)) )
 			{
 				ap = (struct fis_param*)pkg_malloc(sizeof(struct fis_param));
 				if (ap==0)
@@ -387,14 +402,51 @@ static int fixup_write_avp(void** param, int param_no)
 				}
 				memset( ap, 0, sizeof(struct fis_param));
 				/* any falgs ? */
-				if ( p && !( (flags&AVPOPS_USE_SRC_IP)==0 && (
-				(!strcasecmp("username",p) && (flags|=AVPOPS_FLAG_USER)) ||
+				if(p && !((flags&
+				(AVPOPS_USE_SRC_IP|AVPOPS_USE_DST_IP|AVPOPS_USE_HDRREQ))==0
+				&& ((!strcasecmp("username",p) && (flags|=AVPOPS_FLAG_USER)) ||
 				(!strcasecmp("domain", p) && (flags|=AVPOPS_FLAG_DOMAIN)))) )
 				{
 					LOG(L_ERR,"ERROR:avpops:fixup_write_avp: flag \"%s\""
 						" unknown!\n", p);
 					return E_UNSPEC;
 				}
+				if (flags&AVPOPS_USE_HDRREQ)
+				{
+					len = strlen(s);
+					if (len<6 || s[3]!='[' || s[len-1]!=']')
+					{
+						LOG(L_ERR,"ERROR:avpops:fixup_write_avp: invalid hdr "
+							"specificatoin \"%s\"\n",s);
+						return E_UNSPEC;
+					}
+					s[len-1] = ':';
+					/* parse header name */
+					if (parse_hname2( s+4, s+len, &hdr)==0) {
+						LOG(L_ERR,"BUG:avpops:fixup_write_avp: parse header "
+							"failed\n");
+						return E_UNSPEC;
+					}
+					if (hdr.type==HDR_OTHER_T) {
+						/* duplicate hdr name */
+						len -= 5; /*hdr[]*/
+						ap->val.s.s = pkg_malloc(len+1);
+						if (ap->val.s.s==0)
+						{
+							LOG(L_ERR,"ERROR:avpops:fixup_write_avp: no more "
+								"pkg mem\n");
+							return E_OUT_OF_MEM;
+						}
+						ap->val.s.len = len;
+						memcpy( ap->val.s.s, s+4, len);
+						ap->val.s.s[len] = 0;
+						DBG("DEBUF:avpops:fixup_write_avp: hdr=<%s>\n",
+							ap->val.s.s);
+					} else {
+						ap->val.n = hdr.type;
+						flags |= AVPOPS_VAL_INT;
+					}
+				}
 				ap->flags = flags|AVPOPS_VAL_NONE;
 			} else {
 				LOG(L_ERR,"ERROR:avpops:fixup_write_avp: source \"%s\""
@@ -531,15 +583,14 @@ static int fixup_pushto_avp(void** param, int param_no)
 				}
 			}
 			/* copy header name */
-			ap->val.s = (str*)pkg_malloc( sizeof(str) + strlen(s) + 1 );
-			if (ap->val.s==0)
+			ap->val.s.s = pkg_malloc(strlen(s) + 1 );
+			if (ap->val.s.s==0)
 			{
 				LOG(L_ERR,"ERROR:avpops:fixup_pushto_avp: no more pkg mem\n");
 				return E_OUT_OF_MEM;
 			}
-			ap->val.s->s = ((char*)ap->val.s) + sizeof(str);
-			ap->val.s->len = strlen(s);
-			strcpy( ap->val.s->s, s);
+			ap->val.s.len = strlen(s);
+			strcpy( ap->val.s.s, s);
 		}
 	} else if (param_no==2) {
 		/* attribute name / alias */
@@ -617,7 +668,7 @@ static int fixup_check_avp(void** param, int param_no)
 		{
 			if ( (ap->flags&AVPOPS_VAL_STR)==0 )
 			{
-				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: regexp operation"
+				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: regexp operation "
 					"requires string value\n");
 				return E_UNSPEC;
 			}
@@ -628,17 +679,26 @@ static int fixup_check_avp(void** param, int param_no)
 				return E_OUT_OF_MEM;
 			}
 			DBG("DEBUG:avpops:fixup_check_avp: compiling regexp <%s>\n",
-				ap->val.s->s);
-			if (regcomp(re, ap->val.s->s, REG_EXTENDED|REG_ICASE|REG_NEWLINE))
+				ap->val.s.s);
+			if (regcomp(re, ap->val.s.s, REG_EXTENDED|REG_ICASE|REG_NEWLINE))
 			{
 				pkg_free(re);
 				LOG(L_ERR,"ERROR:avpops:fixip_check_avp: bad re <%s>\n",
-					ap->val.s->s);
+					ap->val.s.s);
 				return E_BAD_RE;
 			}
 			/* free the string and link the regexp */
-			pkg_free(ap->val.s);
-			ap->val.s = (str*)re;
+			pkg_free(ap->val.s.s);
+			ap->val.s.s = (char*)re;
+		} else if (ap->flags&AVPOPS_OP_FM) {
+			if ( !( ap->flags&AVPOPS_VAL_AVP ||
+			(!(ap->flags&AVPOPS_VAL_AVP) && ap->flags&AVPOPS_VAL_STR) ) )
+			{
+				LOG(L_ERR,"ERROR:avpops:fixup_check_avp: fast_match operation "
+					"requires string value or avp name/alias (%d)\n",
+					ap->flags);
+				return E_UNSPEC;
+			}
 		}
 	}
 
@@ -648,6 +708,87 @@ static int fixup_check_avp(void** param, int param_no)
 }
 
 
+static int fixup_copy_avp(void** param, int param_no)
+{
+	struct fis_param *ap;
+	char *s;
+	char *p;
+
+	s = (char*)*param;
+	ap = 0;
+	p = 0;
+
+	if (param_no==2)
+	{
+		/* avp / flags */
+		if ( (p=strchr(s,'/'))!=0 )
+			*(p++)=0;
+	}
+
+	if ( (ap=get_attr_or_alias(s))==0 )
+	{
+		LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: bad attribute name"
+			"/alias <%s>\n", (char*)*param);
+		return E_UNSPEC;
+	}
+	/* attr name is mandatory */
+	if (ap->flags&AVPOPS_VAL_NONE)
+	{
+		LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: you must specify "
+			"a name for the AVP\n");
+		return E_UNSPEC;
+	}
+
+	if (param_no==2)
+	{
+		/* flags */
+		for( ; p&&*p ; p++ )
+		{
+			switch (*p) {
+				case 'g':
+				case 'G':
+					ap->flags|=AVPOPS_FLAG_ALL;
+					break;
+				case 'd':
+				case 'D':
+					ap->flags|=AVPOPS_FLAG_DELETE;
+					break;
+				default:
+					LOG(L_ERR,"ERROR:avpops:fixup_copy_avp: bad flag "
+						"<%c>\n",*p);
+					return E_UNSPEC;
+			}
+		}
+	}
+
+	pkg_free(*param);
+	*param=(void*)ap;
+	return 0;
+}
+
+
+static int fixup_is_avp_set(void** param, int param_no)
+{
+	struct fis_param *ap;
+
+	if (param_no==1) {
+		/* attribute name / alias */
+		if ( ((ap=get_attr_or_alias((char*)*param))==0)
+				|| (ap->flags&AVPOPS_VAL_NONE))
+		{
+			LOG(L_ERR,"ERROR:avpops:fixup_is_avp_set: bad attribute name"
+				"/alias <%s>\n", (char*)*param);
+			return E_UNSPEC;
+		}
+
+		pkg_free(*param);
+		*param=(void*)ap;
+	}
+
+	return 0;
+}
+
+
 static int w_dbload_avps(struct sip_msg* msg, char* source, char* param)
 {
 	return ops_dbload_avps ( msg, (struct fis_param*)source,
@@ -671,7 +812,7 @@ static int w_dbdelete_avps(struct sip_msg* msg, char* source, char* param)
 
 static int w_write_avps(struct sip_msg* msg, char* source, char* param)
 {
-	return ops_write_avp ( msg, (struct fis_param*)source, 
+	return ops_write_avp ( msg, (struct fis_param*)source,
 								(struct fis_param*)param);
 }
 
@@ -695,9 +836,18 @@ static int w_check_avps(struct sip_msg* msg, char* param, char *check)
 								(struct fis_param*)check);
 }
 
+static int w_copy_avps(struct sip_msg* msg, char* name1, char *name2)
+{
+	return ops_copy_avp ( msg, (struct fis_param*)name1,
+								(struct fis_param*)name2);
+}
+
 static int w_print_avps(struct sip_msg* msg, char* foo, char *bar)
 {
 	return ops_print_avp();
 }
 
-
+static int w_is_avp_set(struct sip_msg* msg, char* param, char *op)
+{
+	return ops_is_avp_set(msg, (struct fis_param*)param);
+}
