@@ -1,5 +1,5 @@
 /*
- * $Id: avpops_impl.c,v 1.5 2004/11/15 10:19:39 ramona Exp $
+ * $Id: avpops_impl.c,v 1.17.2.2 2007/11/15 02:28:32 sobomax Exp $
  *
  * Copyright (C) 2004 Voice Sistem SRL
  *
@@ -27,12 +27,15 @@
  * History:
  * ---------
  *  2004-10-04  first version (ramona)
+ *  2005-01-30  "fm" (fast match) operator added (ramona)
+ *  2005-01-30  avp_copy (copy/move operation) added (ramona)
  */
 
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
+#include <fnmatch.h>
 
 #include "../../ut.h"
 #include "../../dprint.h"
@@ -52,7 +55,7 @@
 
 static db_key_t  store_keys[6];
 static db_val_t  store_vals[6];
-static str      empty={"",0};
+static str      empty=STR_STATIC_INIT("");
 
 
 void init_store_avps( char **db_columns)
@@ -120,7 +123,7 @@ static int dbrow2avp(struct db_row *row, int flags, int_str attr,
 		uint = (unsigned int)row->values[2].val.int_val;
 		db_flags = ((uint&AVPOPS_DB_NAME_INT)?0:AVP_NAME_STR) |
 			((uint&AVPOPS_DB_VAL_INT)?0:AVP_VAL_STR);
-	
+
 		DBG("db_flags=%d, flags=%d\n",db_flags,flags);
 		/* does the avp type match ? */
 		if(!((flags&(AVPOPS_VAL_INT|AVPOPS_VAL_STR))==0 ||
@@ -129,7 +132,7 @@ static int dbrow2avp(struct db_row *row, int flags, int_str attr,
 			return -2;
 	} else {
 		/* check the validity of value column */
-		if (row->values[0].nul || 
+		if (row->values[0].nul ||
 		(row->values[0].type!=DB_STRING && row->values[0].type!=DB_STR) )
 		{
 			LOG(L_ERR,"ERROR:avpops:dbrow2avp: empty or wrong type for"
@@ -156,7 +159,7 @@ static int dbrow2avp(struct db_row *row, int flags, int_str attr,
 		if (db_flags&AVP_NAME_STR)
 		{
 			/* name is string */
-			avp_attr.s = &atmp;
+			avp_attr.s = atmp;
 		} else {
 			/* name is ID */
 			if (str2int( &atmp, &uint)==-1)
@@ -180,7 +183,7 @@ static int dbrow2avp(struct db_row *row, int flags, int_str attr,
 	if (db_flags&AVP_VAL_STR)
 	{
 		/* value is string */
-		avp_val.s = &vtmp;
+		avp_val.s = vtmp;
 	} else {
 		/* name is ID */
 		if (str2int(&vtmp, &uint)==-1)
@@ -212,7 +215,7 @@ inline static str* get_source_uri(struct sip_msg* msg,int source)
 		return &(get_from(msg)->uri);
 	} else if (source&AVPOPS_USE_TO)
 	{  /* to */
-		if (parse_headers( msg, HDR_TO, 0)<0)
+		if (parse_headers( msg, HDR_TO_F, 0)<0)
 		{
 			LOG(L_ERR,"ERROR:avpops:get_source_uri: failed "
 				"to parse to\n");
@@ -270,7 +273,7 @@ static inline void int_str2db_val( int_str is_val, str *val, int is_s)
 	if (is_s)
 	{
 		/* val is string */
-		*val = *is_val.s;
+		*val = is_val.s;
 	} else {
 		/* val is integer */
 		val->s =
@@ -286,7 +289,7 @@ static int get_avp_as_str(struct fis_param *ap ,str *val)
 	int_str         avp_val;
 
 	name_type = ((ap->flags&AVPOPS_VAL_INT)?0:AVP_NAME_STR);
-	avp = search_first_avp( name_type, ap->val, &avp_val);
+	avp = search_first_avp( name_type, ap->val, &avp_val, 0);
 	if (avp==0)
 	{
 		DBG("DEBUG:avpops:get_val_as_str: no avp found\n");
@@ -329,7 +332,7 @@ int ops_dbload_avps (struct sip_msg* msg, struct fis_param *sp,
 	} else  if (sp->flags&AVPOPS_VAL_STR) {
 		/* use the STR val as uuid */
 		/* do DB query */
-		res = db_load_avp( sp->val.s, 0, 0, dbp->sa.s, dbp->table, dbp->scheme);
+		res = db_load_avp( &sp->val.s, 0, 0, dbp->sa.s, dbp->table, dbp->scheme);
 	} else {
 		LOG(L_CRIT,"BUG:avpops:load_avps: invalid flag combination (%d)\n",
 			sp->flags);
@@ -366,16 +369,17 @@ error:
 int ops_dbstore_avps (struct sip_msg* msg, struct fis_param *sp,
 									struct db_param *dbp, int use_domain)
 {
+	struct search_state st;
 	struct sip_uri   uri;
-	struct usr_avp   **avp_list;
-	struct usr_avp   *avp;
+	avp_list_t avp_list;
+	avp_t* avp;
 	unsigned short   name_type;
 	int_str          i_s;
-	str              uuid;
+	str              uuid, *t;
 	int              keys_off;
 	int              keys_nr;
 	int              n;
-	
+
 
 	if (sp->flags&AVPOPS_VAL_NONE)
 	{
@@ -412,7 +416,7 @@ int ops_dbstore_avps (struct sip_msg* msg, struct fis_param *sp,
 		/* set values for keys  */
 		keys_off = 0;
 		keys_nr = 4;
-		store_vals[0].val.str_val = *sp->val.s;
+		store_vals[0].val.str_val = sp->val.s;
 	} else {
 		LOG(L_CRIT,"BUG:avpops:store_avps: invalid flag combination (%d)\n",
 			sp->flags);
@@ -427,8 +431,8 @@ int ops_dbstore_avps (struct sip_msg* msg, struct fis_param *sp,
 		/* avp name is known ->set it and its type */
 		name_type = (((dbp->a.flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
 		store_vals[1].val.str_val = dbp->sa; /*attr name*/
-		avp = search_first_avp( name_type, dbp->a.val, &i_s);
-		for( ; avp; avp=search_next_avp(avp,&i_s))
+		avp = search_first_avp( name_type, dbp->a.val, &i_s, &st);
+		for( ; avp; avp=search_next_avp(&st,&i_s))
 		{
 			/* don't insert avps which were loaded */
 			if (avp->flags&AVP_IS_IN_DB)
@@ -450,8 +454,8 @@ int ops_dbstore_avps (struct sip_msg* msg, struct fis_param *sp,
 		}
 	} else {
 		/* avp name is unknown -> go through all list */
-		avp_list = get_avp_list();
-		avp = *avp_list;
+		avp_list = get_avp_list(AVP_TRACK_FROM | AVP_CLASS_USER);
+		avp = avp_list;
 
 		for ( ; avp ; avp=avp->next )
 		{
@@ -465,8 +469,12 @@ int ops_dbstore_avps (struct sip_msg* msg, struct fis_param *sp,
 				continue;
 
 			/* set attribute name and type */
-			if ( (i_s.s=get_avp_name(avp))==0 )
+			t = get_avp_name(avp);
+			if (!t) {
 				i_s.n = avp->id;
+			} else {
+				i_s.s = *t;
+			}
 			int_str2db_val( i_s, &store_vals[1].val.str_val,
 				avp->flags&AVP_NAME_STR);
 			store_vals[3].val.int_val =
@@ -526,7 +534,7 @@ int ops_dbdelete_avps (struct sip_msg* msg, struct fis_param *sp,
 	} else if (sp->flags&AVPOPS_VAL_STR) {
 		/* use the STR value as uuid */
 		/* do DB delete */
-		res = db_delete_avp( sp->val.s, 0, 0, dbp->sa.s, dbp->table );
+		res = db_delete_avp( &sp->val.s, 0, 0, dbp->sa.s, dbp->table );
 	} else {
 		LOG(L_CRIT,"BUG:avpops:dbdelete_avps: invalid flag combination (%d)\n",
 			sp->flags);
@@ -546,6 +554,34 @@ error:
 }
 
 
+static inline str *search_hdr(struct sip_msg* msg, struct fis_param *hdr_def)
+{
+	static str body;
+	struct hdr_field *hdr;
+
+	/* parse all HDRs */
+	if (parse_headers( msg, HDR_EOH_F, 0)!=0) {
+		LOG(L_ERR,"ERROR:tm:append2buf: parsing hdrs failed\n");
+		return 0;
+	}
+	/* search the HDR */
+	if (hdr_def->flags&AVPOPS_VAL_INT) {
+		for(hdr=msg->headers;hdr;hdr=hdr->next)
+			if (hdr_def->val.n==hdr->type)
+				goto found;
+	} else {
+		for(hdr=msg->headers;hdr;hdr=hdr->next)
+			if (hdr_def->val.s.len==hdr->name.len &&
+			strncasecmp( hdr_def->val.s.s, hdr->name.s, hdr->name.len)==0)
+				goto found;
+	}
+	return 0;
+found:
+	/* trim the body */
+	trim_len( body.len, body.s, hdr->body );
+	return &body;
+}
+
 
 int ops_write_avp(struct sip_msg* msg, struct fis_param *src,
 													struct fis_param *ap)
@@ -553,7 +589,7 @@ int ops_write_avp(struct sip_msg* msg, struct fis_param *src,
 	struct sip_uri uri;
 	int_str avp_val;
 	unsigned short flags;
-	str s_ip;
+	str s_ip, *t, *t2;
 
 	if (src->flags&AVPOPS_VAL_NONE)
 	{
@@ -566,7 +602,25 @@ int ops_write_avp(struct sip_msg* msg, struct fis_param *src,
 				goto error;
 			}
 			s_ip.len = strlen(s_ip.s);
-			avp_val.s = &s_ip;
+			avp_val.s = s_ip;
+		} else if (src->flags&AVPOPS_USE_DST_IP) {
+			/* get data from dst ip */
+			if ( (s_ip.s=ip_addr2a( &msg->rcv.dst_ip ))==0)
+			{
+				LOG(L_ERR,"ERROR:avpops:write_avp: cannot get dst_ip\n");
+				goto error;
+			}
+			s_ip.len = strlen(s_ip.s);
+			avp_val.s = s_ip;
+		} else if (src->flags&(AVPOPS_USE_HDRREQ|AVPOPS_USE_HDRRPL)) {
+			/* get data from hdr */
+			t = search_hdr(msg,src);
+			if (t == 0) {
+				DBG("DEBUG:avpops:write_avp: hdr not found\n");
+				goto error;
+			} else {
+				avp_val.s = *t;
+			}
 		} else {
 			/* get data from uri (from,to,ruri) */
 			if (src->flags&(AVPOPS_FLAG_USER|AVPOPS_FLAG_DOMAIN))
@@ -577,15 +631,17 @@ int ops_write_avp(struct sip_msg* msg, struct fis_param *src,
 					goto error;
 				}
 				if (src->flags&AVPOPS_FLAG_DOMAIN)
-					avp_val.s = &uri.host;
+					avp_val.s = uri.host;
 				else
-					avp_val.s = &uri.user;
+					avp_val.s = uri.user;
 			} else {
 				/* get whole uri */
-				if ( (avp_val.s=get_source_uri(msg,src->flags))==0 )
-				{
+				t2 = get_source_uri(msg,src->flags);
+				if (t2 == 0) {
 					LOG(L_ERR,"ERROR:avpops:write_avp: cannot get uri\n");
 					goto error;
+				} else {
+					avp_val.s = *t2;
 				}
 			}
 		}
@@ -611,9 +667,9 @@ error:
 
 int ops_delete_avp(struct sip_msg* msg, struct fis_param *ap)
 {
-	struct usr_avp **avp_list;
-	struct usr_avp *avp;
-	struct usr_avp *avp_next;
+	avp_list_t avp_list;
+	avp_t* avp;
+	avp_t* avp_next;
 	unsigned short name_type;
 	int n;
 
@@ -623,7 +679,7 @@ int ops_delete_avp(struct sip_msg* msg, struct fis_param *ap)
 	{
 		/* avp name is known ->search by name */
 		name_type = (((ap->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
-		while ( (avp=search_first_avp( name_type, ap->val, 0))!=0 )
+		while ( (avp=search_first_avp( name_type, ap->val, 0, 0))!=0 )
 		{
 			destroy_avp( avp );
 			n++;
@@ -633,8 +689,8 @@ int ops_delete_avp(struct sip_msg* msg, struct fis_param *ap)
 	} else {
 		/* avp name is not given - we have just flags */
 		/* -> go through all list */
-		avp_list = get_avp_list();
-		avp = *avp_list;
+		avp_list = get_avp_list(AVP_TRACK_FROM | AVP_CLASS_USER);
+		avp = avp_list;
 
 		for ( ; avp ; avp=avp_next )
 		{
@@ -718,6 +774,7 @@ inline static int append_0(str *in, str *out)
 int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 													struct fis_param* ap)
 {
+	struct search_state st;
 	struct lump    *anchor;
 	struct action  act;
 	struct usr_avp *avp;
@@ -726,10 +783,11 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 	str            val;
 	int            act_type;
 	int            n;
+	struct run_act_ctx ra_ctx;
 
 	/* search for the avp */
 	name_type = (((ap->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
-	avp = search_first_avp( name_type, ap->val, &avp_val);
+	avp = search_first_avp( name_type, ap->val, &avp_val, &st);
 	if (avp==0)
 	{
 		DBG("DEBUG:avpops:pushto_avp: no avp found\n");
@@ -741,7 +799,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 	{
 		/* the avp val will be used all the time as str */
 		if (avp->flags&AVP_VAL_STR) {
-			val = *(avp_val.s);
+			val = avp_val.s;
 		} else {
 			val.s = int2str((unsigned long)avp_val.n, &val.len);
 		}
@@ -761,7 +819,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 				goto error;
 			}
 		} else if (dst->flags&(AVPOPS_USE_HDRREQ|AVPOPS_USE_HDRRPL)) {
-			if ( compose_hdr( dst->val.s, &val, &val,
+			if ( compose_hdr( &dst->val.s, &val, &val,
 				dst->flags&AVPOPS_USE_HDRREQ)<0 )
 			{
 				LOG(L_ERR,"ERROR:avpops:pushto_avp: failed to build hdr\n");
@@ -772,7 +830,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 				dst->flags);
 			goto error;
 		}
-	
+
 		if ( act_type )
 		{
 			/* rewrite part of ruri */
@@ -780,7 +838,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 			{
 				/* if is not the first modification, push the current uri as
 				 * branch */
-				if (append_branch( msg, 0, 0, 0, 0, 0)!=1 )
+				if (append_branch( msg, 0, 0, 0, 0, Q_UNSPECIFIED, 0)!=1 )
 				{
 					LOG(L_ERR,"ERROR:avpops:pushto_avp: append_branch action"
 						" failed\n");
@@ -788,10 +846,11 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 				}
 			}
 			memset(&act, 0, sizeof(act));
-			act.p1_type = STRING_ST;
-			act.p1.string = val.s;
+			act.val[0].type = STRING_ST;
+			act.val[0].u.string = val.s;
 			act.type = act_type;
-			if (do_action(&act, msg)<0)
+			init_run_actions_ctx(&ra_ctx);
+			if (do_action(&ra_ctx, &act, msg)<0)
 			{
 				LOG(L_ERR,"ERROR:avpops:pushto_avp: SET_XXXX_T action"
 					" failed\n");
@@ -806,7 +865,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 			}
 		} else {
 			/* set a header for request */
-			if (parse_headers(msg, HDR_EOH, 0)==-1)
+			if (parse_headers(msg, HDR_EOH_F, 0)==-1)
 			{
 				LOG(L_ERR, "ERROR:avpops:pushto_avp: message parse failed\n");
 				goto error;
@@ -827,7 +886,7 @@ int ops_pushto_avp ( struct sip_msg* msg, struct fis_param* dst,
 		n++;
 		if ( !(ap->flags&AVPOPS_FLAG_ALL) )
 			break;
-		avp = search_next_avp( avp, &avp_val);
+		avp = search_next_avp(&st, &avp_val);
 	} /* end while */
 
 	DBG("DEBUG:avpops:pushto_avps: %d avps were processed\n",n);
@@ -840,19 +899,20 @@ error:
 int ops_check_avp( struct sip_msg* msg, struct fis_param* ap,
 													struct fis_param* val)
 {
+	struct search_state st1, st2;
 	unsigned short    name_type;
 	struct usr_avp    *avp1;
 	struct usr_avp    *avp2;
 	regmatch_t        pmatch;
 	int_str           avp_val;
 	int_str           ck_val;
-	str               s_ip;
+	str               s_ip, *t;
 	int               ck_flg;
 	int               n;
 
 	/* look if the required avp(s) is/are present */
 	name_type = (((ap->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
-	avp1 = search_first_avp( name_type, ap->val, &avp_val);
+	avp1 = search_first_avp( name_type, ap->val, &avp_val, &st1);
 	if (avp1==0)
 	{
 		DBG("DEBUG:avpops:check_avp: no avp found to check\n");
@@ -864,7 +924,12 @@ cycle1:
 	{
 		/* the 2nd operator is an avp name -> get avp val */
 		name_type = (((val->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
-		avp2 = search_first_avp( name_type, val->val, &ck_val);
+		avp2 = search_first_avp( name_type, val->val, &ck_val, &st2);
+		if (avp2==0)
+		{
+			DBG("DEBUG:avpops:check_avp: no avp2 found to check\n");
+			goto error;
+		}
 		ck_flg = avp2->flags&AVP_VAL_STR?AVPOPS_VAL_STR:AVPOPS_VAL_INT;
 	} else {
 		ck_val = val->val;
@@ -897,44 +962,58 @@ cycle2:
 					goto error;
 				}
 				s_ip.len = strlen(s_ip.s);
-				ck_val.s = &s_ip;
+				ck_val.s = s_ip;
+			} else if (val->flags&AVPOPS_USE_DST_IP) {
+				/* get value from dst ip */
+				if ( (s_ip.s=ip_addr2a( &msg->rcv.dst_ip ))==0)
+				{
+					LOG(L_ERR,"ERROR:avpops:check_avp: cannot get dst_ip\n");
+					goto error;
+				}
+				s_ip.len = strlen(s_ip.s);
+				ck_val.s = s_ip;
 			} else {
 				/* get value from uri */
-				if ( (ck_val.s=get_source_uri(msg,val->flags))==0 )
-				{
+				t = get_source_uri(msg,val->flags);
+				if (t == 0) {
 					LOG(L_ERR,"ERROR:avpops:check_avp: cannot get uri\n");
 					goto next;
+				} else {
+					ck_val.s = *t;
 				}
 			}
 		}
 		DBG("DEBUG:avpops:check_avp: check <%.*s> against <%.*s> as str\n",
-			avp_val.s->len,avp_val.s->s,
-			(val->flags&AVPOPS_OP_RE)?6:ck_val.s->len,
-			(val->flags&AVPOPS_OP_RE)?"REGEXP":ck_val.s->s);
+			avp_val.s.len,avp_val.s.s,
+			(val->flags&AVPOPS_OP_RE)?6:ck_val.s.len,
+			(val->flags&AVPOPS_OP_RE)?"REGEXP":ck_val.s.s);
 		/* do check */
 		if (val->flags&AVPOPS_OP_EQ)
 		{
-			if (avp_val.s->len==ck_val.s->len)
+			if (avp_val.s.len==ck_val.s.len)
 			{
 				if (val->flags&AVPOPS_FLAG_CI)
 				{
-					if (strncasecmp(avp_val.s->s,ck_val.s->s,ck_val.s->len)==0)
+					if (strncasecmp(avp_val.s.s,ck_val.s.s,ck_val.s.len)==0)
 						return 1;
 				} else {
-					if (strncmp(avp_val.s->s,ck_val.s->s,ck_val.s->len)==0 )
+					if (strncmp(avp_val.s.s,ck_val.s.s,ck_val.s.len)==0 )
 						return 1;
 				}
 			}
 		} else if (val->flags&AVPOPS_OP_LT) {
-			n = (avp_val.s->len>=ck_val.s->len)?avp_val.s->len:ck_val.s->len;
-			if (strncasecmp(avp_val.s->s,ck_val.s->s,n)==-1)
+			n = (avp_val.s.len>=ck_val.s.len)?avp_val.s.len:ck_val.s.len;
+			if (strncasecmp(avp_val.s.s,ck_val.s.s,n)==-1)
 				return 1;
 		} else if (val->flags&AVPOPS_OP_GT) {
-			n = (avp_val.s->len>=ck_val.s->len)?avp_val.s->len:ck_val.s->len;
-			if (strncasecmp(avp_val.s->s,ck_val.s->s,n)==1)
+			n = (avp_val.s.len>=ck_val.s.len)?avp_val.s.len:ck_val.s.len;
+			if (strncasecmp(avp_val.s.s,ck_val.s.s,n)==1)
 				return 1;
 		} else if (val->flags&AVPOPS_OP_RE) {
-			if (regexec((regex_t*)ck_val.s, avp_val.s->s, 1, &pmatch, 0)==0)
+			if (regexec((regex_t*)ck_val.s.s, avp_val.s.s, 1, &pmatch, 0)==0)
+				return 1;
+		} else if (val->flags&AVPOPS_OP_FM){
+			if (fnmatch( ck_val.s.s, avp_val.s.s, 0)==0)
 				return 1;
 		} else {
 			LOG(L_CRIT,"BUG:avpops:check_avp: unknown operation "
@@ -962,13 +1041,13 @@ cycle2:
 
 next:
 	/* cycle for the second value (only if avp can have multiple vals) */
-	if ((val->flags&AVPOPS_VAL_AVP)&&(avp2=search_next_avp(avp2,&ck_val))!=0)
+	if ((val->flags&AVPOPS_VAL_AVP)&&(avp2=search_next_avp(&st2,&ck_val))!=0)
 	{
 		ck_flg = avp2->flags&AVP_VAL_STR?AVPOPS_VAL_STR:AVPOPS_VAL_INT;
 		goto cycle2;
 	/* cycle for the first value -> next avp */
 	} else {
-		avp1=(val->flags&AVPOPS_FLAG_ALL)?search_next_avp(avp1,&avp_val):0;
+		avp1=(val->flags&AVPOPS_FLAG_ALL)?search_next_avp(&st1,&avp_val):0;
 		if (avp1)
 			goto cycle1;
 	}
@@ -980,17 +1059,70 @@ error:
 }
 
 
+/*
+ * FIXME: janakj: I need to check this function to make sure it works
+ * properly with domain and global attributes
+ */
+int ops_copy_avp( struct sip_msg* msg, struct fis_param* name1,
+		  struct fis_param* name2)
+{
+	struct search_state st;
+	struct usr_avp *avp;
+	struct usr_avp *prev_avp;
+	int_str         avp_val;
+	unsigned short name_type1;
+	unsigned short name_type2;
+	int n;
+
+	n = 0;
+	prev_avp = 0;
+
+	/* avp name is known ->search by name */
+	name_type1 = (((name1->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
+	name_type2 = (((name2->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR);
+
+	avp = search_first_avp( name_type1, name1->val, &avp_val, &st);
+	while ( avp )
+	{
+		/* build a new avp with new name, but old value */
+		if ( add_avp( name_type2|(avp->flags&AVP_VAL_STR)|AVP_CLASS_USER, name2->val,
+		avp_val)==-1 ) {
+			LOG(L_ERR,"ERROR:avpops:copy_avp: failed to create new avp\n");
+			goto error;
+		}
+		n++;
+		/* copy all avps? */
+		if ( !(name2->flags&AVPOPS_FLAG_ALL) ) {
+			/* delete the old one? */
+			if (name2->flags&AVPOPS_FLAG_DELETE)
+				destroy_avp( avp );
+			break;
+		} else {
+			prev_avp = avp;
+			avp = search_next_avp(&st, &avp_val);
+			/* delete the old one? */
+			if (name2->flags&AVPOPS_FLAG_DELETE)
+				destroy_avp( prev_avp );
+		}
+	}
+
+	return n?1:-1;
+error:
+	return -1;
+}
+
+
 
 int ops_print_avp()
 {
-	struct usr_avp **avp_list;
-	struct usr_avp *avp;
+	avp_list_t avp_list;
+	avp_t* avp;
 	int_str         val;
 	str            *name;
 
 	/* go through all list */
-	avp_list = get_avp_list();
-	avp = *avp_list;
+	avp_list = get_avp_list(AVP_TRACK_FROM | AVP_CLASS_USER);
+	avp = avp_list;
 
 	for ( ; avp ; avp=avp->next)
 	{
@@ -1005,13 +1137,22 @@ int ops_print_avp()
 		get_avp_val( avp, &val);
 		if (avp->flags&AVP_VAL_STR)
 		{
-			DBG("DEBUG:\t\t\tval_str=<%.*s>\n",val.s->len,val.s->s);
+			DBG("DEBUG:\t\t\tval_str=<%.*s>\n",val.s.len,val.s.s);
 		} else {
 			DBG("DEBUG:\t\t\tval_int=<%d>\n",val.n);
 		}
 	}
 
-	
+
 	return 1;
 }
 
+
+int ops_is_avp_set(struct sip_msg* msg, struct fis_param *ap)
+{
+	if(search_first_avp((((ap->flags&AVPOPS_VAL_INT))?0:AVP_NAME_STR),
+				ap->val, NULL, 0)!=0)
+		return 1;
+	
+	return -1;
+}
