@@ -1,5 +1,5 @@
 /*
- * $Id: cpl.c,v 1.51.2.3 2005/06/21 17:52:13 andrei Exp $
+ * $Id: cpl.c,v 1.62 2006/03/01 16:01:43 janakj Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -20,8 +20,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * History:
@@ -33,7 +33,7 @@
  * 2004-06-14: all global variables merged into cpl_env and cpl_fct;
  *             case_sensitive and realm_prefix added for building AORs - see
  *             build_userhost (bogdan)
- * 2004-10-09: added process_register_norpl to allow register processing 
+ * 2004-10-09: added process_register_norpl to allow register processing
  *             without sending the reply(bogdan) - based on a patch sent by
  *             Christopher Crawford
  */
@@ -54,7 +54,6 @@
 #include "../../ut.h"
 #include "../../dprint.h"
 #include "../../data_lump_rpl.h"
-#include "../../fifo_server.h"
 #include "../../usr_avp.h"
 #include "../../parser/parse_uri.h"
 #include "../../parser/parse_from.h"
@@ -67,6 +66,7 @@
 #include "cpl_loader.h"
 #include "cpl_parser.h"
 #include "cpl_nonsig.h"
+#include "cpl_rpc.h"
 #include "loc_set.h"
 
 
@@ -129,27 +129,28 @@ static cmd_export_t cmds[] = {
  * Exported parameters
  */
 static param_export_t params[] = {
-	{"cpl_db",         STR_PARAM, &DB_URL      },
-	{"cpl_table",      STR_PARAM, &DB_TABLE    },
-	{"cpl_dtd_file",   STR_PARAM, &dtd_file    },
-	{"proxy_recurse",  INT_PARAM, &cpl_env.proxy_recurse  },
-	{"proxy_route",    INT_PARAM, &cpl_env.proxy_route    },
-	{"nat_flag",       INT_PARAM, &cpl_env.nat_flag       },
-	{"log_dir",        STR_PARAM, &cpl_env.log_dir        },
-	{"case_sensitive", INT_PARAM, &cpl_env.case_sensitive },
-	{"realm_prefix",   STR_PARAM, &cpl_env.realm_prefix.s },
-	{"lookup_domain",  STR_PARAM, &lookup_domain          },
-	{"lookup_append_branches", INT_PARAM, &cpl_env.lu_append_branches},
-	{"timer_avp",      STR_PARAM, &timer_avp   },
+	{"cpl_db",         PARAM_STRING, &DB_URL      },
+	{"cpl_table",      PARAM_STRING, &DB_TABLE    },
+	{"cpl_dtd_file",   PARAM_STRING, &dtd_file    },
+	{"proxy_recurse",  PARAM_INT,    &cpl_env.proxy_recurse  },
+	{"proxy_route",    PARAM_INT,    &cpl_env.proxy_route    },
+	{"nat_flag",       PARAM_INT,    &cpl_env.nat_flag       },
+	{"log_dir",        PARAM_STRING, &cpl_env.log_dir        },
+	{"case_sensitive", PARAM_INT,    &cpl_env.case_sensitive },
+	{"realm_prefix",   PARAM_STR,    &cpl_env.realm_prefix   },
+	{"lookup_domain",  PARAM_STRING, &lookup_domain          },
+	{"lookup_append_branches", PARAM_INT, &cpl_env.lu_append_branches},
+	{"timer_avp",      PARAM_STRING, &timer_avp   },
 	{0, 0, 0}
 };
 
 
 struct module_exports exports = {
 	"cpl-c",
-	cmds,     /* Exported functions */
-	params,   /* Exported parameters */
-	cpl_init, /* Module initialization function */
+	cmds,             /* Exported functions */
+	cpl_rpc_methods,  /* RPC methods */
+	params,           /* Exported parameters */
+	cpl_init,         /* Module initialization function */
 	(response_function) 0,
 	(destroy_function) cpl_exit,
 	0,
@@ -198,6 +199,7 @@ static int fixup_cpl_run_script(void** param, int param_no)
 static int cpl_init(void)
 {
 	bind_usrloc_t bind_usrloc;
+	bind_sl_t     bind_sl;
 	load_tm_f     load_tm;
 	struct stat   stat_t;
 	char *ptr;
@@ -230,17 +232,13 @@ static int cpl_init(void)
 	if (timer_avp) {
 		foo.s = timer_avp;
 		foo.len = strlen(foo.s);
-		if (parse_avp_spec(&foo,&cpl_env.timer_avp_type,&cpl_env.timer_avp)<0){
+		if (parse_avp_spec(&foo,&cpl_env.timer_avp_type,&cpl_env.timer_avp,0)<0){
 			LOG(L_CRIT,"ERROR:cpl_init: invalid timer AVP specs \"%s\"\n",
 				timer_avp);
 			goto error;
 		}
-		if (cpl_env.timer_avp_type&AVP_NAME_STR && cpl_env.timer_avp.s==&foo) {
-			if ( (cpl_env.timer_avp.s=(str*)pkg_malloc(sizeof(str)))==0 ) {
-				LOG(L_ERR, "ERROR:cpl_init: no more pkg mem\n");
-				goto error;
-			}
-			*(cpl_env.timer_avp.s) = foo;
+		if (cpl_env.timer_avp_type&AVP_NAME_STR && cpl_env.timer_avp.s.s==foo.s) {
+			cpl_env.timer_avp.s = foo;
 		}
 	}
 
@@ -307,12 +305,12 @@ static int cpl_init(void)
 	if (load_tm( &(cpl_fct.tmb) )==-1)
 		goto error;
 
-	/* load the send_reply function from sl module */
-	if ((cpl_fct.sl_reply=find_export("sl_send_reply", 2, 0))==0) {
-		LOG(L_ERR, "ERROR:cpl_c:cpl_init: cannot import sl_send_reply; maybe "
-			"you forgot to load the sl module\n");
-		goto error;
+        bind_sl = (bind_sl_t)find_export("bind_sl", 0, 0);
+	if (!bind_sl) {
+		ERR("This module requires sl module\n");
+		return -1;
 	}
+	if (bind_sl(&cpl_fct.sl) < 0) return -1;
 
 	/* bind to usrloc module if requested */
 	if (lookup_domain) {
@@ -338,21 +336,6 @@ static int cpl_init(void)
 			" lookup node\n");
 	}
 
-
-	/* register the fifo commands */
-	if (register_fifo_cmd( cpl_load, "LOAD_CPL", 0)!=1) {
-		LOG(L_CRIT,"ERROR:cpl_init: cannot register LOAD_CPL fifo cmd!\n");
-		goto error;
-	}
-	if (register_fifo_cmd( cpl_remove, "REMOVE_CPL", 0)!=1) {
-		LOG(L_CRIT,"ERROR:cpl_init: cannot register REMOVE_CPL fifo cmd!\n");
-		goto error;
-	}
-	if (register_fifo_cmd( cpl_get, "GET_CPL", 0)!=1) {
-		LOG(L_CRIT,"ERROR:cpl_init: cannot register GET_CPL fifo cmd!\n");
-		goto error;
-	}
-
 	/* build a pipe for sending commands to aux process */
 	if ( pipe( cpl_env.cmd_pipe )==-1 ) {
 		LOG(L_CRIT,"ERROR:cpl_init: cannot create command pipe: %s!\n",
@@ -390,7 +373,6 @@ static int cpl_init(void)
 
 	/* convert realm_prefix from string null terminated to str */
 	if (cpl_env.realm_prefix.s) {
-		cpl_env.realm_prefix.len = strlen(cpl_env.realm_prefix.s);
 		/* convert the realm_prefix to lower cases */
 		strlower( &cpl_env.realm_prefix );
 	}
@@ -549,9 +531,10 @@ static inline int get_dest_user(struct sip_msg *msg, str *uh, int flg)
 		msg->first_line.u.request.uri.len ,&uri)==-1 || !uri.user.len )
 		{
 			DBG("DEBUG:cpl-c:get_dest_user: trying to get user from To\n");
-			if ( (!msg->to&&( (parse_headers(msg,HDR_TO,0)==-1) || !msg->to))||
-			parse_uri( get_to(msg)->uri.s, get_to(msg)->uri.len, &uri)==-1
-			|| !uri.user.len)
+			if ( (!msg->to&&( (parse_headers(msg,HDR_TO_F,0)==-1) ||
+					!msg->to)) ||
+				parse_uri( get_to(msg)->uri.s, get_to(msg)->uri.len, &uri)==-1
+				|| !uri.user.len)
 			{
 				LOG(L_ERR,"ERROR:cpl-c:get_dest_user: unable to extract user"
 					" name from RURI or To header!\n");
@@ -568,7 +551,7 @@ static inline int get_orig_user(struct sip_msg *msg, str *uh, int flg)
 {
 	struct to_body *from;
 	struct sip_uri uri;
-	
+
 	/* if it's outgoing -> get the user_name from From */
 	/* parsing from header */
 	DBG("DEBUG:cpl-c:get_orig_user: trying to get user from From\n");
@@ -589,9 +572,9 @@ static inline int get_orig_user(struct sip_msg *msg, str *uh, int flg)
 
 
 
-/* Params: 
- *   str1 - as unsigned int - can be CPL_RUN_INCOMING or CPL_RUN_OUTGOING 
- *   str2 - as unsigned int - flags regarding state(less)|(ful) 
+/* Params:
+ *   str1 - as unsigned int - can be CPL_RUN_INCOMING or CPL_RUN_OUTGOING
+ *   str2 - as unsigned int - flags regarding state(less)|(ful)
  */
 static int cpl_invoke_script(struct sip_msg* msg, char* str1, char* str2)
 {
@@ -703,14 +686,16 @@ static struct cpl_error *cpl_err = &bad_req;
 
 static inline int do_script_action(struct sip_msg *msg, int action)
 {
-	str  body = {0,0};
-	str  user = {0,0};
-	str  bin  = {0,0};
-	str  log  = {0,0};
+	str  body = STR_NULL;
+	str  user = STR_NULL;
+	str  bin  = STR_NULL;
+	str  log  = STR_NULL;
 
 	/* content-length (if present) */
-	if ( !msg->content_length && ((parse_headers(msg,HDR_CONTENTLENGTH,0)==-1)
-	|| !msg->content_length) ) {
+	if ( !msg->content_length &&
+			((parse_headers(msg, HDR_CONTENTLENGTH_F, 0)==-1)
+			 || !msg->content_length) )
+	{
 		LOG(L_ERR,"ERROR:cpl-c:do_script_action: no Content-Length "
 			"hdr found!\n");
 		goto error;
@@ -777,8 +762,8 @@ error:
 
 static inline int do_script_download(struct sip_msg *msg)
 {
-	str  user  = {0,0};
-	str script = {0,0};
+	str  user  = STR_NULL;
+	str script = STR_NULL;
 
 	/* get the destination user name */
 	if (get_dest_user( msg, &user, 0)==-1)
@@ -907,7 +892,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 			goto resume_script;
 
 		/* send a 200 OK reply back */
-		cpl_fct.sl_reply( msg, (char*)200, "OK");
+		cpl_fct.sl.reply( msg, 200, "OK");
 		/* I send the reply and I don't want to return to script execution, so
 		 * I return 0 to do break */
 		goto stop_script;
@@ -943,7 +928,7 @@ static int cpl_process_register(struct sip_msg* msg, int no_rpl)
 		goto resume_script;
 
 	/* send a 200 OK reply back */
-	cpl_fct.sl_reply( msg, (char*)200, "OK");
+	cpl_fct.sl.reply( msg, 200, "OK");
 
 stop_script:
 	return 0;
@@ -951,7 +936,7 @@ resume_script:
 	return 1;
 error:
 	/* send a error reply back */
-	cpl_fct.sl_reply( msg, (char*)(long)cpl_err->err_code, cpl_err->err_msg);
+	cpl_fct.sl.reply( msg, cpl_err->err_code, cpl_err->err_msg);
 	/* I don't want to return to script execution, so I return 0 to do break */
 	return 0;
 }
