@@ -1,5 +1,5 @@
 /*
- * $Id: cpl_loader.h,v 1.4 2004/08/24 08:58:26 janakj Exp $
+ * $Id: cpl_loader.h,v 1.6 2006/01/28 13:37:41 janakj Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -33,10 +33,9 @@
 #ifndef _CPL_LOADER_H
 #define _CPL_LOADER_H
 
+#include "../../str.h"
 
-int cpl_load( FILE *fifo_stream, char *response_file );
-int cpl_remove( FILE *fifo_stream, char *response_file );
-int cpl_get( FILE *fifo_stream, char *response_file );
+int load_file( char *filename, str *xml);
 
 #endif
 
