@@ -1,5 +1,5 @@
 /*
- * $Id: cpl_run.c,v 1.37.2.2 2005/07/20 17:11:51 andrei Exp $
+ * $Id: cpl_run.c,v 1.41 2006/03/01 16:01:43 janakj Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -287,7 +287,7 @@ static inline char *run_lookup( struct cpl_interpreter *intr )
 		} else {
 			contact = r->contacts;
 			/* skip expired contacts */
-			while (contact && contact->expires<=tc)
+			while ( contact && contact->expires <= tc)
 				contact = contact->next;
 			/* any contacts left? */
 			if (contact) {
@@ -613,7 +613,7 @@ static inline char *run_reject( struct cpl_interpreter *intr )
 		i = cpl_fct.tmb.t_reply(intr->msg, (int)status, reason_s );
 	} else {
 		/* reply statelessly */
-		i = cpl_fct.sl_reply(intr->msg, (char*)(long)status, reason_s );
+		i = cpl_fct.sl.reply(intr->msg, status, reason_s );
 	}
 
 	if ( i!=1 ) {
@@ -740,9 +740,9 @@ static inline char *run_redirect( struct cpl_interpreter *intr )
 	} else {
 		/* reply statelessly */
 		if (permanent)
-			i = cpl_fct.sl_reply( intr->msg, (char*)301, "Moved permanently");
+			i = cpl_fct.sl.reply( intr->msg, 301, "Moved permanently");
 		else
-			i = cpl_fct.sl_reply( intr->msg, (char*)302, "Moved temporarily");
+			i = cpl_fct.sl.reply( intr->msg, 302, "Moved temporarily");
 	}
 
 	/* msg which I'm working on can be in private memory or is a clone into
@@ -773,8 +773,8 @@ static inline char *run_log( struct cpl_interpreter *intr )
 	char  *p;
 	unsigned short attr_name;
 	unsigned short n;
-	str name    = {0,0};
-	str comment = {0,0};
+	str name    = STR_NULL;
+	str comment = STR_NULL;
 	str user;
 	int i;
 
@@ -856,9 +856,9 @@ static inline char *run_mail( struct cpl_interpreter *intr )
 	unsigned short attr_name;
 	unsigned short n;
 	char  *p;
-	str subject = {0,0};
-	str body    = {0,0};
-	str to      = {0,0};
+	str subject = STR_NULL;
+	str body    = STR_NULL;
+	str to      = STR_NULL;
 	int i;
 
 	/* sanity check */
