@@ -1,5 +1,5 @@
 /*
- * $Id: cpl_switches.h,v 1.13.2.1 2005/06/03 00:56:22 andrei Exp $
+ * $Id: cpl_switches.h,v 1.16 2005/11/17 03:50:07 sobomax Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -39,7 +39,7 @@
  */
 static inline char *run_address_switch( struct cpl_interpreter *intr )
 {
-	static str def_port_str = {"5060",4};
+	static str def_port_str = STR_STATIC_INIT("5060");
 	unsigned short field, subfield;
 	char  *p;
 	char  *kid;
@@ -147,7 +147,7 @@ static inline char *run_address_switch( struct cpl_interpreter *intr )
 							if (!intr->to) {
 								/* get and parse the header */
 								if (!intr->msg->to &&
-								(parse_headers(intr->msg,HDR_TO,0)==-1 ||
+								(parse_headers(intr->msg,HDR_TO_F,0)==-1 ||
 								!intr->msg->to)) {
 									LOG(L_ERR,"ERROR:run_address_switch: bad "
 										"msg or missing TO header\n");
@@ -354,7 +354,7 @@ static inline char *run_string_switch( struct cpl_interpreter *intr )
 								/* get the subject header */
 								if (!intr->msg->subject) {
 									if (parse_headers(intr->msg,
-									HDR_SUBJECT,0)==-1) {
+									HDR_SUBJECT_F,0)==-1) {
 										LOG(L_ERR,"ERROR:run_string_switch: "
 										"bad SUBJECT header\n");
 										goto runtime_error;
@@ -377,7 +377,7 @@ static inline char *run_string_switch( struct cpl_interpreter *intr )
 								/* get the organization header */
 								if (!intr->msg->organization) {
 									if (parse_headers(intr->msg,
-									HDR_ORGANIZATION,0)==-1) {
+									HDR_ORGANIZATION_F,0)==-1) {
 										LOG(L_ERR,"ERROR:run_string_switch: "
 										"bad ORGANIZATION hdr\n");
 										goto runtime_error;
@@ -400,7 +400,7 @@ static inline char *run_string_switch( struct cpl_interpreter *intr )
 								/* get the  header */
 								if (!intr->msg->user_agent) {
 									if (parse_headers(intr->msg,
-									HDR_USERAGENT,0)==-1) {
+									HDR_USERAGENT_F,0)==-1) {
 										LOG(L_ERR,"ERROR:run_string_switch: "
 										"bad USERAGENT hdr\n");
 										goto runtime_error;
@@ -479,7 +479,7 @@ script_error:
  */
 static inline char *run_priority_switch( struct cpl_interpreter *intr )
 {
-	static str default_val={"normal",6};
+	static str default_val=STR_STATIC_INIT("normal");
 	unsigned short n;
 	char *p;
 	char *kid;
@@ -489,8 +489,8 @@ static inline char *run_priority_switch( struct cpl_interpreter *intr )
 	unsigned short msg_attr_val;
 	unsigned short msg_prio;
 	int i;
-	str cpl_val = {0,0};
-	str msg_val = {0,0};
+	str cpl_val = STR_NULL;
+	str msg_val = STR_NULL;
 
 	not_present_node = 0;
 	msg_attr_val = NORMAL_VAL;
@@ -558,7 +558,7 @@ static inline char *run_priority_switch( struct cpl_interpreter *intr )
 					if (!intr->priority) {
 						/* get the PRIORITY header from message */
 						if (!intr->msg->priority) {
-							if (parse_headers(intr->msg,HDR_PRIORITY,0)==-1) {
+							if (parse_headers(intr->msg,HDR_PRIORITY_F,0)==-1){
 								LOG(L_ERR,"ERROR:run_priority_switch: bad "
 									"sip msg or PRIORITY header !\n");
 								goto runtime_error;
@@ -704,7 +704,7 @@ static inline char *run_time_switch( struct cpl_interpreter *intr )
 	unsigned char  flags = 0;
 	int nr_attrs;
 	int i,j;
-	str user_tz = {0,0};
+	str user_tz = STR_NULL;
 	ac_tm_t att;
 	tmrec_t trt;
 
@@ -893,8 +893,8 @@ inline static int is_lang_tag_matching(str *range,str *cpl_tag,str *cpl_subtag)
 {
 	char *c;
 	char *end;
-	str tag = {0,0};
-	str subtag = {0,0};
+	str tag = STR_NULL;
+	str subtag = STR_NULL;
 
 	c = range->s;
 	end = range->s + range->len;
@@ -984,10 +984,10 @@ static inline char *run_language_switch( struct cpl_interpreter *intr )
 	unsigned short attr_name;
 	int nr_attr;
 	int i,j;
-	str attr = {0,0};
-	str msg_val = {0,0};
-	str lang_tag = {0,0};
-	str lang_subtag = {0,0};
+	str attr = STR_NULL;
+	str msg_val = STR_NULL;
+	str lang_tag = STR_NULL;
+	str lang_subtag = STR_NULL;
 
 	not_present_node = 0;
 
@@ -1050,7 +1050,7 @@ static inline char *run_language_switch( struct cpl_interpreter *intr )
 						/* get the accept_language header */
 						if (!intr->msg->accept_language) {
 							if (parse_headers(intr->msg,
-							HDR_ACCEPTLANGUAGE,0)==-1) {
+							HDR_ACCEPTLANGUAGE_F,0)==-1) {
 								LOG(L_ERR,"ERROR:run_language_switch: "
 									"bad ACCEPT_LANGUAGE header\n");
 								goto runtime_error;
