@@ -1,5 +1,5 @@
 /**
- * $Id: dispatch.h,v 1.2 2004/08/24 08:58:27 janakj Exp $
+ * $Id: dispatch.h,v 1.5 2006/05/30 21:21:51 calrissian Exp $
  *
  * dispatcher module
  * 
@@ -40,9 +40,16 @@
 
 #include "../../parser/msg_parser.h"
 
+
+#define DS_HASH_USER_ONLY 1  /* use only the uri user part for hashing */
+#define DS_HASH_USER_OR_HOST 2  /* use user part of uri for hashing with
+                                   fallback to host */
+extern int ds_flags; 
+
 int ds_load_list(char *lfile);
 int ds_destroy_list();
 int ds_select_dst(struct sip_msg *msg, char *set, char *alg);
+int ds_select_new(struct sip_msg *msg, char *set, char *alg);
 
 #endif
 
