@@ -1,5 +1,5 @@
 /*
- * $Id: xjab_worker.c,v 1.36.2.1 2005/06/16 10:30:08 andrei Exp $
+ * $Id: xjab_worker.c,v 1.38 2006/01/30 15:59:51 kubartv Exp $
  *
  * eXtended JABber module - worker implementation
  *
@@ -93,7 +93,7 @@ int main_loop = 1;
 
 /** **/
 extern char *registrar;
-static str jab_gw_name = {"jabber_gateway@127.0.0.1", 24};
+static str jab_gw_name = STR_STATIC_INIT("jabber_gateway@127.0.0.1");
 
 /**
  * address correction
@@ -352,7 +352,7 @@ int xj_worker_process(xj_wlist jwl, char* jaddress, int jport, int rank,
 		if(!FD_ISSET(pipe, &mset))
 			goto step_y;
 		
-		if(read(pipe, &jsmsg, sizeof(jsmsg)) < (int)sizeof(jsmsg))
+		if(read(pipe, &jsmsg, sizeof(jsmsg)) < sizeof(jsmsg))
 		{
 			DBG("XJAB:xj_worker:%d: BROKEN PIPE - exiting\n", _xj_pid);
 			break;
@@ -1162,7 +1162,7 @@ void xj_sig_handler(int s)
  */
 int xj_send_sip_msg(str *proxy, str *to, str *from, str *msg, int *cbp)
 {
-	str  msg_type = { "MESSAGE", 7};
+	str  msg_type = STR_STATIC_INIT("MESSAGE");
 	char buf[512];
 	str  tfrom;
 	str  str_hdr;
@@ -1200,11 +1200,11 @@ int xj_send_sip_msg(str *proxy, str *to, str *from, str *msg, int *cbp)
 		DBG("XJAB:xj_send_sip_msg: uac callback parameter [%p==%d]\n", 
 				cbp, *cbp);
 #endif
-		return tmb.t_request(&msg_type, 0, to, &tfrom, &str_hdr, msg, 
+		return tmb.t_request(&msg_type, 0, to, &tfrom, &str_hdr, msg, 0, 
 					     xj_tuac_callback, (void*)cbp);
 	}
 	else
-		return tmb.t_request(&msg_type, 0, to, &tfrom, &str_hdr, msg, 0, 0);
+		return tmb.t_request(&msg_type, 0, to, &tfrom, &str_hdr, msg, 0, 0, 0);
 }
 
 /**
