@@ -1,4 +1,4 @@
-/* $Id: mediaproxy.c,v 1.23.2.4 2005/07/20 05:45:10 danp Exp $
+/* $Id: mediaproxy.c,v 1.32.2.2 2008/02/02 14:32:22 janakj Exp $
  *
  * Copyright (C) 2004 Dan Pascu
  *
@@ -44,6 +44,7 @@
 #include "../../parser/parse_to.h"
 #include "../../parser/parse_uri.h"
 #include "../../msg_translator.h"
+#include "../../id.h"
 #include "../registrar/sip_msg.h"
 #include "../usrloc/usrloc.h"
 
@@ -138,7 +139,6 @@ static int UseMediaProxy(struct sip_msg *msg, char *str1, char *str2);
 static int EndMediaSession(struct sip_msg *msg, char *str1, char *str2);
 
 static int mod_init(void);
-static int fixstring2int(void **param, int param_count);
 
 static Bool testPrivateContact(struct sip_msg* msg);
 static Bool testSourceAddress(struct sip_msg* msg);
@@ -169,9 +169,7 @@ static AsymmetricClients rtpAsymmetrics = {
     0
 };
 
-static CheckLocalPartyProc isFromLocal;
 //static CheckLocalPartyProc isToLocal;
-static CheckLocalPartyProc isDestinationLocal;
 
 NetInfo rfc1918nets[] = {
     {"10.0.0.0",    0x0a000000UL, 0xff000000UL},
@@ -191,21 +189,22 @@ static cmd_export_t commands[] = {
     {"fix_contact",       FixContact,      0, 0,             REQUEST_ROUTE | ONREPLY_ROUTE },
     {"use_media_proxy",   UseMediaProxy,   0, 0,             REQUEST_ROUTE | ONREPLY_ROUTE },
     {"end_media_session", EndMediaSession, 0, 0,             REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE },
-    {"client_nat_test",   ClientNatTest,   1, fixstring2int, REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE },
+    {"client_nat_test",   ClientNatTest,   1, fixup_int_1, REQUEST_ROUTE | ONREPLY_ROUTE | FAILURE_ROUTE },
     {0, 0, 0, 0, 0}
 };
 
 static param_export_t parameters[] = {
-    {"mediaproxy_socket", STR_PARAM, &mediaproxySocket},
-    {"sip_asymmetrics",   STR_PARAM, &(sipAsymmetrics.file)},
-    {"rtp_asymmetrics",   STR_PARAM, &(rtpAsymmetrics.file)},
-    {"natping_interval",  INT_PARAM, &natpingInterval},
+    {"mediaproxy_socket", PARAM_STRING, &mediaproxySocket},
+    {"sip_asymmetrics",   PARAM_STRING, &(sipAsymmetrics.file)},
+    {"rtp_asymmetrics",   PARAM_STRING, &(rtpAsymmetrics.file)},
+    {"natping_interval",  PARAM_INT,    &natpingInterval},
     {0, 0, 0}
 };
 
 struct module_exports exports = {
     "mediaproxy", // module name
     commands,     // module exported functions
+    0,            // RPC methods
     parameters,   // module exported parameters
     mod_init,     // module init (before any kid is created. kids will inherit)
     NULL,         // reply processing
@@ -442,7 +441,7 @@ static Bool
 getCallId(struct sip_msg* msg, str *cid)
 {
     if (msg->callid == NULL) {
-        if (parse_headers(msg, HDR_CALLID, 0) == -1) {
+        if (parse_headers(msg, HDR_CALLID_F, 0) == -1) {
             return False;
         }
         if (msg->callid == NULL) {
@@ -460,12 +459,19 @@ getCallId(struct sip_msg* msg, str *cid)
 
 /* Get caller domain */
 static str
-getFromDomain(struct sip_msg* msg)
+getFromDomain(char** type, struct sip_msg* msg)
 {
     static char buf[16] = "unknown"; // buf is here for a reason. don't
     static str notfound = {buf, 7};  // use the constant string directly!
     static struct sip_uri puri;
-    str uri;
+    str uri, did;
+
+    if (get_from_did(&did, msg) == 1) {
+		*type = "local";
+		return did;
+    }
+	
+    *type = "remote";
 
     if (parse_from_header(msg) < 0) {
         LOG(L_ERR, "error: mediaproxy/getFromDomain(): error parsing `From' header\n");
@@ -486,12 +492,19 @@ getFromDomain(struct sip_msg* msg)
 
 /* Get called domain */
 static str
-getToDomain(struct sip_msg* msg)
+getToDomain(char** type, struct sip_msg* msg)
 {
     static char buf[16] = "unknown"; // buf is here for a reason. don't
     static str notfound = {buf, 7};  // use the constant string directly!
     static struct sip_uri puri;
-    str uri;
+    str uri, did;
+
+    if (get_to_did(&did, msg) == 0) {
+	*type = "local";
+	return did;
+    }
+
+    *type = "remote";
 
     uri = get_to(msg)->uri;
 
@@ -510,10 +523,18 @@ getToDomain(struct sip_msg* msg)
 // This function only works when called for a request although it's more
 // reliable than the getToDomain function.
 static str
-getDestinationDomain(struct sip_msg* msg)
+getDestinationDomain(char** type, struct sip_msg* msg)
 {
     static char buf[16] = "unknown"; // buf is here for a reason. don't
     static str notfound = {buf, 7};  // use the constant string directly!
+    str did;
+
+    if (get_to_did(&did, msg) == 0) {
+	*type = "local";
+	return did;
+    }
+
+    *type = "remote";
 
     if (parse_sip_msg_uri(msg) < 0) {
         LOG(L_ERR, "error: mediaproxy/getDestinationDomain(): error parsing destination URI\n");
@@ -643,7 +664,7 @@ getUserAgent(struct sip_msg* msg)
     str block, server;
     char *ptr;
 
-    if ((parse_headers(msg, HDR_USERAGENT, 0)!=-1) && msg->user_agent &&
+    if ((parse_headers(msg, HDR_USERAGENT_F, 0)!=-1) && msg->user_agent &&
         msg->user_agent->body.len>0) {
         return msg->user_agent->body;
     }
@@ -676,7 +697,7 @@ static Bool
 getContactURI(struct sip_msg* msg, struct sip_uri *uri, contact_t** _c)
 {
 
-    if ((parse_headers(msg, HDR_CONTACT, 0) == -1) || !msg->contact)
+    if ((parse_headers(msg, HDR_CONTACT_F, 0) == -1) || !msg->contact)
         return False;
 
     if (!msg->contact->parsed && parse_contact(msg->contact) < 0) {
@@ -1303,7 +1324,7 @@ ClientNatTest(struct sip_msg* msg, char* str1, char* str2)
 {
     int tests, i;
 
-    tests = (int)(long)str1;
+	if (get_int_fparam(&tests, msg, (fparam_t*) str1) < 0) return -1;
 
     for (i=0; natTests[i].test!=NTNone; i++) {
         if ((tests & natTests[i].test)!=0 && natTests[i].proc(msg)) {
@@ -1408,19 +1429,16 @@ UseMediaProxy(struct sip_msg* msg, char* str1, char* str2)
         return 1;
     }
 
-    fromDomain = getFromDomain(msg);
-    fromType   = (isFromLocal(msg, NULL, NULL)>0) ? "local" : "remote";
+    fromDomain = getFromDomain(&fromType, msg);
     fromAddr   = getFromAddress(msg);
     toAddr     = getToAddress(msg);
     fromTag    = getFromTag(msg);
     toTag      = getToTag(msg);
     userAgent  = getUserAgent(msg);
     if (request) {
-        toDomain = getDestinationDomain(msg); // call only for requests
-        toType = (isDestinationLocal(msg, NULL, NULL)>0) ? "local" : "remote";
+        toDomain = getDestinationDomain(&toType, msg); // call only for requests
     } else {
-        toDomain = getToDomain(msg);
-        toType = "unknown"; //there's no function to check if To domain is local
+        toDomain = getToDomain(&toType, msg);
     }
 
     clientIP = ip_addr2a(&msg->rcv.src_ip);
@@ -1558,14 +1576,6 @@ mod_init(void)
 {
     bind_usrloc_t ul_bind_usrloc;
 
-    isFromLocal = (CheckLocalPartyProc)find_export("is_from_local", 0, 0);
-    isDestinationLocal = (CheckLocalPartyProc)find_export("is_uri_host_local", 0, 0);
-    if (!isFromLocal || !isDestinationLocal) {
-        LOG(L_ERR, "error: mediaproxy/mod_init(): can't find is_from_local "
-            "and/or is_uri_host_local functions. Check if domain.so is loaded\n");
-        return -1;
-    }
-
     if (natpingInterval > 0) {
         ul_bind_usrloc = (bind_usrloc_t)find_export("ul_bind_usrloc", 1, 0);
         if (!ul_bind_usrloc) {
@@ -1590,29 +1600,3 @@ mod_init(void)
 
     return 0;
 }
-
-
-// convert string parameter to integer for functions that expect an integer
-static int
-fixstring2int(void **param, int param_count)
-{
-    unsigned long number;
-    int err;
-
-    if (param_count == 1) {
-        number = str2s(*param, strlen(*param), &err);
-        if (err == 0) {
-            pkg_free(*param);
-            *param = (void*)number;
-            return 0;
-        } else {
-            LOG(L_ERR, "error: mediaproxy/fixstring2int(): bad number `%s'\n",
-                (char*)(*param));
-            return E_CFG;
-        }
-    }
-
-    return 0;
-}
-
-
