@@ -1,5 +1,5 @@
 /**
- * $Id: msilo.c,v 1.42 2004/12/03 19:09:32 andrei Exp $
+ * $Id: msilo.c,v 1.53.2.2 2007/05/16 09:13:10 kubartv Exp $
  *
  * MSILO module
  *
@@ -22,8 +22,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
@@ -64,6 +64,7 @@
 #include "../../parser/parse_content.h"
 #include "../../parser/contact/parse_contact.h"
 #include "../../resolve.h"
+#include "../../id.h"
 
 #include "../tm/tm_load.h"
 
@@ -77,14 +78,14 @@
 #include "ms_msg_list.h"
 #include "msfuncs.h"
 
-#define MAX_DEL_KEYS	1	
-#define NR_KEYS			9
+#define MAX_DEL_KEYS	1
+#define NR_KEYS		9
 
 char *sc_mid      = "mid";       /* 0 */
-char *sc_from     = "src_addr";  /* 1 */
-char *sc_to       = "dst_addr";  /* 2 */
-char *sc_uri_user = "username";  /* 3 */
-char *sc_uri_host = "domain";    /* 4 */
+char *sc_from     = "from_hdr";  /* 1 */
+char *sc_to       = "to_hdr";    /* 2 */
+char *sc_ruri     = "ruri";      /* 3 */
+char *sc_uid      = "uid";       /* 4 */
 char *sc_body     = "body";      /* 5 */
 char *sc_ctype    = "ctype";     /* 6 */
 char *sc_exp_time = "exp_time";  /* 7 */
@@ -115,7 +116,7 @@ char *sc_inc_time = "inc_time";  /* 8 */
 
 MODULE_VERSION
 
-#define S_TABLE_VERSION 3
+#define S_TABLE_VERSION 4
 
 /** database connection */
 static db_con_t *db_con = NULL;
@@ -137,7 +138,7 @@ int  ms_check_time=30;
 int  ms_clean_period=5;
 int  ms_use_contact=1;
 
-str msg_type = { "MESSAGE", 7 };
+str msg_type = STR_STATIC_INIT("MESSAGE");
 
 str reg_addr;
 
@@ -156,29 +157,31 @@ void m_clean_silo(unsigned int ticks, void *);
 static void m_tm_callback( struct cell *t, int type, struct tmcb_params *ps);
 
 static cmd_export_t cmds[]={
+	{"m_store",  m_store, 2, 0, REQUEST_ROUTE | FAILURE_ROUTE},
 	{"m_store",  m_store, 1, 0, REQUEST_ROUTE | FAILURE_ROUTE},
+	{"m_dump",   m_dump,  1, 0, REQUEST_ROUTE},
 	{"m_dump",   m_dump,  0, 0, REQUEST_ROUTE},
 	{0,0,0,0,0}
 };
 
 
 static param_export_t params[]={
-	{"db_url",       STR_PARAM, &ms_db_url},
-	{"db_table",     STR_PARAM, &ms_db_table},
-	{"registrar",    STR_PARAM, &ms_registrar},
-	{"expire_time",  INT_PARAM, &ms_expire_time},
-	{"check_time",   INT_PARAM, &ms_check_time},
-	{"clean_period", INT_PARAM, &ms_clean_period},
-	{"use_contact",  INT_PARAM, &ms_use_contact},
-	{"sc_mid",       STR_PARAM, &sc_mid},
-	{"sc_from",      STR_PARAM, &sc_from},
-	{"sc_to",        STR_PARAM, &sc_to},
-	{"sc_uri_user",  STR_PARAM, &sc_uri_user},
-	{"sc_uri_host",  STR_PARAM, &sc_uri_host},
-	{"sc_body",      STR_PARAM, &sc_body},
-	{"sc_ctype",     STR_PARAM, &sc_ctype},
-	{"sc_exp_time",  STR_PARAM, &sc_exp_time},
-	{"sc_inc_time",  STR_PARAM, &sc_inc_time},
+	{"db_url",       PARAM_STRING, &ms_db_url},
+	{"db_table",     PARAM_STRING, &ms_db_table},
+	{"registrar",    PARAM_STRING, &ms_registrar},
+	{"expire_time",  PARAM_INT,    &ms_expire_time},
+	{"check_time",   PARAM_INT,    &ms_check_time},
+	{"clean_period", PARAM_INT,    &ms_clean_period},
+	{"use_contact",  PARAM_INT,    &ms_use_contact},
+	{"sc_mid",       PARAM_STRING, &sc_mid},
+	{"sc_from",      PARAM_STRING, &sc_from},
+	{"sc_to",        PARAM_STRING, &sc_to},
+	{"sc_ruri",      PARAM_STRING, &sc_ruri},
+	{"sc_uid",       PARAM_STRING, &sc_uid},
+	{"sc_body",      PARAM_STRING, &sc_body},
+	{"sc_ctype",     PARAM_STRING, &sc_ctype},
+	{"sc_exp_time",  PARAM_STRING, &sc_exp_time},
+	{"sc_inc_time",  PARAM_STRING, &sc_inc_time},
 	{0,0,0}
 };
 
@@ -187,8 +190,9 @@ static param_export_t params[]={
 struct module_exports exports= {
 	"msilo",    /* module id */
 	cmds,       /* module's exported functions */
+	0,          /* RPC methods */
 	params,     /* module's exported parameters */
-	
+
 	mod_init,   /* module initialization function */
 	(response_function) 0,       /* response handler */
 	(destroy_function) destroy,  /* module destroy function */
@@ -229,7 +233,7 @@ static int mod_init(void)
 	_s.s = ms_db_table;
 	_s.len = strlen(ms_db_table);
 	ver =  table_version(&msilo_dbf, db_con, &_s);
-	if(ver!=3)
+	if(ver!=S_TABLE_VERSION)
 	{
 		LOG(L_ERR,"MSILO:mod_init: Wrong version v%d for table <%s>,"
 				" need v%d\n", ver, ms_db_table, S_TABLE_VERSION);
@@ -267,6 +271,9 @@ static int mod_init(void)
  */
 static int child_init(int rank)
 {
+
+	if (rank==PROC_MAIN || rank==PROC_TCP_MAIN)
+		return 0; /* do nothing for the main or tcp_main processes */
 	DBG("MSILO: init_child #%d / pid <%d>\n", rank, getpid());
 	if (msilo_dbf.init==0)
 	{
@@ -285,234 +292,139 @@ static int child_init(int rank)
 			LOG(L_ERR, "MSILO: child %d: Error in use_table\n", rank);
 			return -1;
 		}
-		
+
 		DBG("MSILO: child %d: Database connection opened successfully\n", rank);
 	}
 	return 0;
 }
 
+
 /**
  * store message
  * mode = "0" -- look for outgoing URI starting with new_uri
  * 		= "1" -- look for outgoing URI starting with r-uri
  * 		= "2" -- look for outgoing URI only at to header
+ * next_hop = parameter specifying next hop for outgoing messages (like outbound proxy)
  */
-static int m_store(struct sip_msg* msg, char* mode, char* str2)
+static int m_store(struct sip_msg* msg, char* str1, char* str2)
 {
-	str body, str_hdr, ctaddr;
-	struct to_body to, *pto, *pfrom;
-	struct sip_uri puri;
-
+	str body, str_hdr, ctaddr, uri, uid;
+	struct to_body* to, *from;
 	db_key_t db_keys[NR_KEYS-1];
 	db_val_t db_vals[NR_KEYS-1];
-	
+
 	int nr_keys = 0, val, lexpire;
 	t_content_type ctype;
 	static char buf[512];
 	static char buf1[1024];
-	int mime;
+	int mime, mode;
+	str next_hop = STR_NULL;
 
 	DBG("MSILO: m_store: ------------ start ------------\n");
 
+	if (!str1) {
+		LOG(L_ERR, "MSILO:m_store: Invalid parameter value\n");
+		goto error;
+	}
+	mode = str1[0] - '0';
+	if (str2) {
+		next_hop.s = str2;
+		next_hop.len = strlen(str2);
+	}
+
+	if (get_to_uid(&uid, msg) < 0) {
+		LOG(L_ERR, "MSILO:m_store: Unable to find out identity of user\n");
+	        goto error;
+	}
+
 	/* get message body - after that whole SIP MESSAGE is parsed */
 	body.s = get_body( msg );
-	if (body.s==0) 
-	{
+	if (body.s==0) {
 		LOG(L_ERR,"MSILO:m_store: ERROR cannot extract body from msg\n");
 		goto error;
 	}
-	
+
 	/* content-length (if present) must be already parsed */
-	if (!msg->content_length) 
-	{
+	if (!msg->content_length) {
 		LOG(L_ERR,"MSILO:m_store: ERROR no Content-Length header found!\n");
 		goto error;
 	}
-	body.len = get_content_length( msg );
+	body.len = get_content_length(msg);
 
 	/* check if the body of message contains something */
-	if(body.len <= 0)
-	{
+	if(body.len <= 0) {
 		DBG("MSILO:m_store: body of the message is empty!\n");
 		goto error;
 	}
-	
-	/* get TO URI */
-	if(!msg->to || !msg->to->body.s)
-	{
-		DBG("MSILO:m_store: cannot find 'to' header!\n");
+
+	to = get_to(msg);
+	if (!to) {
+		LOG(L_ERR, "MSILO:m_store: Cannot get To header\n");
 		goto error;
 	}
 
-	if(msg->to->parsed != NULL)
-	{
-		pto = (struct to_body*)msg->to->parsed;
-		DBG("MSILO:m_store: 'To' header ALREADY PARSED: <%.*s>\n",
-			pto->uri.len, pto->uri.s );	
+	if (parse_from_header(msg) < 0) {
+		LOG(L_ERR, "MSILO:m_store: Error while Parsing From header\n");
+		goto error;
 	}
-	else
-	{
-		DBG("MSILO:m_store: 'To' header NOT PARSED ->parsing ...\n");
-		memset( &to , 0, sizeof(to) );
-		parse_to(msg->to->body.s, msg->to->body.s+msg->to->body.len+1, &to);
-		if(to.uri.len > 0) /* && to.error == PARSE_OK) */
-		{
-			DBG("MSILO:m_store: 'To' parsed OK <%.*s>.\n", 
-				to.uri.len, to.uri.s);
-			pto = &to;
-		}
-		else
-		{
-			DBG("MSILO:m_store: ERROR 'To' cannot be parsed\n");
-			goto error;
-		}
-	}
-	
-	if(pto->uri.len == reg_addr.len && 
-			!strncasecmp(pto->uri.s, reg_addr.s, reg_addr.len))
-	{
-		DBG("MSILO:m_store: message to MSILO REGISTRAR!\n");
+	from = get_from(msg);
+	if (!from) {
+		LOG(L_ERR, "MSILO:m_store: Cannot find From header\n");
 		goto error;
 	}
 
 	db_keys[nr_keys] = sc_to;
-	
 	db_vals[nr_keys].type = DB_STR;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.str_val.s = pto->uri.s;
-	db_vals[nr_keys].val.str_val.len = pto->uri.len;
-
+	db_vals[nr_keys].val.str_val = to->uri;
 	nr_keys++;
 
-	/* check FROM URI */
-	if(!msg->from || !msg->from->body.s)
-	{
-		DBG("MSILO:m_store: ERROR cannot find 'from' header!\n");
-		goto error;
-	}
-
-	if(msg->from->parsed == NULL)
-	{
-		DBG("MSILO:m_store: 'From' header not parsed\n");
-		/* parsing from header */
-		if ( parse_from_header( msg )==-1 ) 
-		{
-			DBG("MSILO:m_store: ERROR cannot parse From header\n");
-			goto error;
-		}
-	}
-	pfrom = (struct to_body*)msg->from->parsed;
-	DBG("MSILO:m_store: 'From' header: <%.*s>\n", pfrom->uri.len,
-			pfrom->uri.s);	
-	
-	if(reg_addr.s && pfrom->uri.len == reg_addr.len && 
-			!strncasecmp(pfrom->uri.s, reg_addr.s, reg_addr.len))
-	{
-		DBG("MSILO:m_store: message from MSILO REGISTRAR!\n");
-		goto error;
-	}
-
 	db_keys[nr_keys] = sc_from;
-	
 	db_vals[nr_keys].type = DB_STR;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.str_val.s = pfrom->uri.s;
-	db_vals[nr_keys].val.str_val.len = pfrom->uri.len;
-
+	db_vals[nr_keys].val.str_val = from->uri;
 	nr_keys++;
 
-	/* get the R-URI */
-	puri.user.len = 0;
-	if(mode && mode[0]=='0' && msg->new_uri.len > 0)
-	{
-		DBG("MSILO:m_store: NEW R-URI found - check if is AoR!\n");
-		if(parse_uri(msg->new_uri.s, msg->new_uri.len, &puri)!=0)
-		{
-			LOG(L_ERR, "MSILO:m_store: bad new R-URI!\n");
-			goto error;
-		}
-		if(puri.user.len>0 && puri.user.s!=NULL
-				&& puri.host.len>0 && puri.host.s!=NULL)
-		{
-			if(str2ip(&puri.host)!=NULL || str2ip6(&puri.host)!=NULL)
-			{ /* it is a IPv4 or IPv6 address */
-				puri.user.len = 0;
-			}
-		}
-		else
-			puri.user.len = 0;
-	}
-	
-	if(mode && mode[0]<='1' && puri.user.len == 0 
-			&& msg->first_line.u.request.uri.len > 0 )
-	{
-		DBG("MSILO:m_store: R-URI found - check if is AoR!\n");
-		if(parse_uri(msg->first_line.u.request.uri.s,
-			msg->first_line.u.request.uri.len, &puri)!=0)
-		{
-			LOG(L_ERR, "MSILO:m_store: bad R-URI!\n");
-			goto error;
-		}
-		if(puri.user.len>0 && puri.user.s!=NULL
-				&& puri.host.len>0 && puri.host.s!=NULL)
-		{
-			if(str2ip(&puri.host)!=NULL || str2ip6(&puri.host)!=NULL)
-			{ /* it is a IPv4 or IPv6 address */
-				puri.user.len = 0;
-			}
-		}
-		else
-			puri.user.len = 0;
-	}
-	
-	if (puri.user.len == 0)
-	{
-		DBG("MSILO:m_store: TO used as R-URI\n");
-		if(parse_uri(pto->uri.s, pto->uri.len, &puri)!=0)
-		{
-			LOG(L_ERR, "MSILO:m_store: bad R-URI!\n");
-			goto error;
-		}
-		if(puri.user.len<=0 || puri.user.s==NULL
-				|| puri.host.len<=0 || puri.host.s==NULL)
-		{
-			LOG(L_ERR, "MSILO:m_store: bad URI in To header!\n");
-			goto error;
-		}
-	}
-
-	db_keys[nr_keys] = sc_uri_user;
-	
+	db_keys[nr_keys] = sc_ruri;
 	db_vals[nr_keys].type = DB_STR;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.str_val.s = puri.user.s;
-	db_vals[nr_keys].val.str_val.len = puri.user.len;
 
+	switch(mode) {
+	case 0:
+		uri = *GET_RURI(msg);
+		     /* new_ruri, orig_ruri*/
+		break;
+	case 1:
+		     /* orig_ruri */
+		uri = msg->first_line.u.request.uri;
+		break;
+	case 2:
+	        uri = to->uri;
+		break;
+	default:
+		LOG(L_ERR, "MSILO:m_store: Unrecognized parameter value: %s\n", str1);
+		goto error;
+	}
+	db_vals[nr_keys].val.str_val = uri;
 	nr_keys++;
 
-	db_keys[nr_keys] = sc_uri_host;
-	
+	db_keys[nr_keys] = sc_uid;
 	db_vals[nr_keys].type = DB_STR;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.str_val.s = puri.host.s;
-	db_vals[nr_keys].val.str_val.len = puri.host.len;
-
+	db_vals[nr_keys].val.str_val = uid;
 	nr_keys++;
 
 	/* add the message's body in SQL query */
-	
+
 	db_keys[nr_keys] = sc_body;
-	
 	db_vals[nr_keys].type = DB_BLOB;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.blob_val.s = body.s;
-	db_vals[nr_keys].val.blob_val.len = body.len;
-
+	db_vals[nr_keys].val.blob_val = body;
 	nr_keys++;
-	
+
 	lexpire = ms_expire_time;
 	/* add 'content-type' -- parse the content-type header */
-	if ((mime=parse_content_type_hdr(msg))<1 ) 
+	if ((mime=parse_content_type_hdr(msg))<1 )
 	{
 		LOG(L_ERR,"MSILO:m_store: ERROR cannot parse Content-Type header\n");
 		goto error;
@@ -523,19 +435,19 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 	db_vals[nr_keys].nul  = 0;
 	db_vals[nr_keys].val.str_val.s   = "text/plain";
 	db_vals[nr_keys].val.str_val.len = 10;
-	
+
 	/** check the content-type value */
 	if( mime!=(TYPE_TEXT<<16)+SUBTYPE_PLAIN
 		&& mime!=(TYPE_MESSAGE<<16)+SUBTYPE_CPIM )
 	{
-		if(m_extract_content_type(msg->content_type->body.s, 
+		if(m_extract_content_type(msg->content_type->body.s,
 				msg->content_type->body.len, &ctype, CT_TYPE) != -1)
 		{
 			DBG("MSILO:m_store: 'content-type' found\n");
-			db_vals[nr_keys].val.str_val.s   = ctype.type.s;
-			db_vals[nr_keys].val.str_val.len = ctype.type.len;
+			db_vals[nr_keys].val.str_val   = ctype.type;
 		}
 	}
+	nr_keys++;
 
 	/* check 'expires' -- no more parsing - already done by get_body() */
 	if(msg->expires && msg->expires->body.len > 0)
@@ -548,19 +460,19 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 
 	/* current time */
 	val = (int)time(NULL);
-	
+
 	/* add expiration time */
 	db_keys[nr_keys] = sc_exp_time;
-	db_vals[nr_keys].type = DB_INT;
+	db_vals[nr_keys].type = DB_DATETIME;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.int_val = val+lexpire;
+	db_vals[nr_keys].val.time_val = val+lexpire;
 	nr_keys++;
 
 	/* add incoming time */
 	db_keys[nr_keys] = sc_inc_time;
-	db_vals[nr_keys].type = DB_INT;
+	db_vals[nr_keys].type = DB_DATETIME;
 	db_vals[nr_keys].nul = 0;
-	db_vals[nr_keys].val.int_val = val;
+	db_vals[nr_keys].val.time_val = val;
 	nr_keys++;
 
 	if (msilo_dbf.use_table(db_con, ms_db_table) < 0)
@@ -574,8 +486,9 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 		LOG(L_ERR, "MSILO:m_store: error storing message\n");
 		goto error;
 	}
-	DBG("MSILO:m_store: message stored. T:<%.*s> F:<%.*s>\n",
-		pto->uri.len, pto->uri.s, pfrom->uri.len, pfrom->uri.s);
+	DBG("MSILO:m_store: message stored. uid:<%.*s> F:<%.*s>\n",
+		uid.len, uid.s, from->uri.len, ZSW(from->uri.s));
+
 	if(reg_addr.len > 0
 		&& reg_addr.len+CONTACT_PREFIX_LEN+CONTACT_SUFFIX_LEN+1<1024)
 	{
@@ -588,10 +501,10 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 
 		strncpy(buf, "User [", 6);
 		body.len = 6;
-		if(pto->uri.len+OFFLINE_MESSAGE_LEN+7/*6+1*/ < 512)
+		if(uri.len+OFFLINE_MESSAGE_LEN+7/*6+1*/ < 512)
 		{
-			strncpy(buf+body.len, pto->uri.s, pto->uri.len);
-			body.len += pto->uri.len;
+			strncpy(buf+body.len, uri.s, uri.len);
+			body.len += uri.len;
 		}
 		strncpy(buf+body.len, OFFLINE_MESSAGE, OFFLINE_MESSAGE_LEN);
 		body.len += OFFLINE_MESSAGE_LEN;
@@ -604,18 +517,18 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 				&& msg->contact->body.len > 0)
 		{
 			DBG("MSILO:m_store: contact header found\n");
-			if((msg->contact->parsed!=NULL 
+			if((msg->contact->parsed!=NULL
 				&& ((contact_body_t*)(msg->contact->parsed))->contacts!=NULL)
 				|| (parse_contact(msg->contact)==0
 				&& msg->contact->parsed!=NULL
 				&& ((contact_body_t*)(msg->contact->parsed))->contacts!=NULL))
 			{
 				DBG("MSILO:m_store: using contact header for info msg\n");
-				ctaddr.s = 
+				ctaddr.s =
 				((contact_body_t*)(msg->contact->parsed))->contacts->uri.s;
 				ctaddr.len =
 				((contact_body_t*)(msg->contact->parsed))->contacts->uri.len;
-			
+
 				if(!ctaddr.s || ctaddr.len < 6 || strncmp(ctaddr.s, "sip:", 4)
 					|| ctaddr.s[4]==' ')
 					ctaddr.s = NULL;
@@ -624,18 +537,19 @@ static int m_store(struct sip_msg* msg, char* mode, char* str2)
 							ctaddr.len,ctaddr.s);
 			}
 		}
-		
+
 		tmb.t_request(&msg_type,  /* Type of the message */
-				(ctaddr.s)?&ctaddr:&pfrom->uri,    /* Request-URI */
-				&pfrom->uri,      /* To */
+				(ctaddr.s)?&ctaddr:&from->uri,    /* Request-URI */
+				&from->uri,       /* To */
 				&reg_addr,        /* From */
 				&str_hdr,         /* Optional headers including CRLF */
 				&body,            /* Message body */
+				next_hop.len ? &next_hop: NULL,        /* next hop */
 				NULL,             /* Callback function */
 				NULL              /* Callback parameter */
 			);
 	}
-	
+
 	return 1;
 error:
 	return -1;
@@ -646,68 +560,50 @@ error:
  */
 static int m_dump(struct sip_msg* msg, char* str1, char* str2)
 {
-	struct to_body to, *pto = NULL;
-	db_key_t db_keys[2];
-	db_val_t db_vals[2];
-	db_key_t db_cols[6];
+	db_key_t db_keys[1];
+	db_val_t db_vals[1];
+	db_key_t db_cols[7];
 	db_res_t* db_res = NULL;
-	int i, db_no_cols = 6, db_no_keys = 2, mid, n;
+	int i, db_no_cols = 7, db_no_keys = 1, mid, n;
 	char hdr_buf[1024], body_buf[1024];
-	struct sip_uri puri;
 
-	str str_vals[4], hdr_str , body_str;
+	str str_vals[5], hdr_str , body_str, uid;
 	time_t rtime;
+	str next_hop = STR_NULL;
 	
+	if (str1) {
+		next_hop.s = str1;
+		next_hop.len = strlen(str1);
+	}
+
 	/* init */
-	db_keys[0]=sc_uri_user;
-	db_keys[1]=sc_uri_host;
+	db_keys[0]=sc_uid;
 	db_cols[0]=sc_mid;
-	
+
 	db_cols[1]=sc_from;
 	db_cols[2]=sc_to;
 	db_cols[3]=sc_body;
 	db_cols[4]=sc_ctype;
 	db_cols[5]=sc_inc_time;
+	db_cols[6]=sc_ruri;
+
 
-	
 	DBG("MSILO:m_dump: ------------ start ------------\n");
+
+	if (get_to_uid(&uid, msg) < 0) {
+		LOG(L_ERR, "MSILO:m_dump: Unable to retrieve identity of user\n");
+		goto error;
+	}
+
 	hdr_str.s=hdr_buf;
 	hdr_str.len=1024;
 	body_str.s=body_buf;
 	body_str.len=1024;
-	
-	/* check for TO header */
-	if(msg->to==NULL && (parse_headers(msg, HDR_TO, 0)==-1
-				|| msg->to==NULL || msg->to->body.s==NULL))
-	{
-		LOG(L_ERR,"MSILO:m_dump: ERROR cannot find TO HEADER!\n");
-		goto error;
-	}
-
-	/* get TO header URI */
-	if(msg->to->parsed != NULL)
-	{
-		pto = (struct to_body*)msg->to->parsed;
-		DBG("MSILO:m_dump: 'To' header ALREADY PARSED: <%.*s>\n",
-			pto->uri.len, pto->uri.s );	
-	}
-	else
-	{
-		memset( &to , 0, sizeof(to) );
-		parse_to(msg->to->body.s,
-			msg->to->body.s + msg->to->body.len + 1, &to);
-		if(to.uri.len <= 0) /* || to.error != PARSE_OK) */
-		{
-			DBG("MSILO:m_dump: 'To' header NOT parsed\n");
-			goto error;
-		}
-		pto = &to;
-	}
 
 	/**
 	 * check if has expires=0 (REGISTER)
 	 */
-	if(parse_headers(msg, HDR_EXPIRES, 0) >= 0)
+	if(parse_headers(msg, HDR_EXPIRES_F, 0) >= 0)
 	{
 		/* check 'expires' > 0 */
 		if(msg->expires && msg->expires->body.len > 0)
@@ -716,12 +612,12 @@ static int m_dump(struct sip_msg* msg, char* str1, char* str2)
 			if(i <= 0)
 			{ /* user goes offline */
 				DBG("MSILO:m_dump: user <%.*s> goes offline - expires=%d\n",
-						pto->uri.len, pto->uri.s, i);
+						uid.len, uid.s, i);
 				goto error;
 			}
 			else
 				DBG("MSILO:m_dump: user <%.*s> online - expires=%d\n",
-						pto->uri.len, pto->uri.s, i);
+						uid.len, uid.s, i);
 		}
 	}
 	else
@@ -730,27 +626,9 @@ static int m_dump(struct sip_msg* msg, char* str1, char* str2)
 		goto error;
 	}
 
-	if(parse_uri(pto->uri.s, pto->uri.len, &puri)!=0)
-	{
-		LOG(L_ERR, "MSILO:m_dump: bad R-URI!\n");
-		goto error;
-	}
-	if(puri.user.len<=0 || puri.user.s==NULL
-			|| puri.host.len<=0 || puri.host.s==NULL)
-	{
-		LOG(L_ERR, "MSILO:m_dump: bad URI in To header!\n");
-		goto error;
-	}
-
 	db_vals[0].type = DB_STR;
 	db_vals[0].nul = 0;
-	db_vals[0].val.str_val.s = puri.user.s;
-	db_vals[0].val.str_val.len = puri.user.len;
-
-	db_vals[1].type = DB_STR;
-	db_vals[1].nul = 0;
-	db_vals[1].val.str_val.s = puri.host.s;
-	db_vals[1].val.str_val.len = puri.host.len;
+	db_vals[0].val.str_val = uid;
 
 	if (msilo_dbf.use_table(db_con, ms_db_table) < 0)
 	{
@@ -761,29 +639,30 @@ static int m_dump(struct sip_msg* msg, char* str1, char* str2)
 	if((msilo_dbf.query(db_con,db_keys,NULL,db_vals,db_cols,db_no_keys,
 				db_no_cols, NULL,&db_res)!=0) || (RES_ROW_N(db_res) <= 0))
 	{
-		DBG("MSILO:m_dump: no stored message for <%.*s>!\n", pto->uri.len,
-					pto->uri.s);
+		DBG("MSILO:m_dump: no stored message for <%.*s>!\n", uid.len,
+					ZSW(uid.s));
 		goto done;
 	}
-		
-	DBG("MSILO:m_dump: dumping [%d] messages for <%.*s>!!!\n", 
-			RES_ROW_N(db_res), pto->uri.len, pto->uri.s);
 
-	for(i = 0; i < RES_ROW_N(db_res); i++) 
+	DBG("MSILO:m_dump: dumping [%d] messages for <%.*s>!!!\n",
+			RES_ROW_N(db_res), uid.len, ZSW(uid.s));
+
+	for(i = 0; i < RES_ROW_N(db_res); i++)
 	{
 		mid =  RES_ROWS(db_res)[i].values[0].val.int_val;
 		if(msg_list_check_msg(ml, mid))
 		{
-			DBG("MSILO:m_dump: message[%d] mid=%d already sent.\n", 
+			DBG("MSILO:m_dump: message[%d] mid=%d already sent.\n",
 				i, mid);
 			continue;
 		}
-		
+
 		memset(str_vals, 0, 4*sizeof(str));
 		SET_STR_VAL(str_vals[0], db_res, i, 1); /* from */
 		SET_STR_VAL(str_vals[1], db_res, i, 2); /* to */
 		SET_STR_VAL(str_vals[2], db_res, i, 3); /* body */
 		SET_STR_VAL(str_vals[3], db_res, i, 4); /* ctype */
+		SET_STR_VAL(str_vals[4], db_res, i, 6); /* Request-URI */
 
 		hdr_str.len = 1024;
 		if(m_build_headers(&hdr_str, str_vals[3] /*ctype*/,
@@ -796,26 +675,27 @@ static int m_dump(struct sip_msg* msg, char* str1, char* str2)
 			msg_list_set_flag(ml, mid, MS_MSG_ERRO);
 			goto error;
 		}
-			
+
 		DBG("MSILO:m_dump: msg [%d-%d] for: %.*s\n", i+1, mid,
-				pto->uri.len, pto->uri.s);
-			
+				uid.len, ZSW(uid.s));
+
 		/** sending using TM function: t_uac */
 		body_str.len = 1024;
-		rtime = 
-			(time_t)RES_ROWS(db_res)[i].values[5/*inc time*/].val.int_val;
+		rtime =
+			(time_t)RES_ROWS(db_res)[i].values[5/*inc time*/].val.time_val;
 		n = m_build_body(&body_str, rtime, str_vals[2/*body*/]);
 		if(n<0)
 			DBG("MSILO:m_dump: sending simple body\n");
 		else
 			DBG("MSILO:m_dump: sending composed body\n");
-		
+
 			tmb.t_request(&msg_type,  /* Type of the message */
-					&pto->uri,        /* Request-URI */
+					&str_vals[4],     /* Request-URI */
 					&str_vals[1],     /* To */
 					&str_vals[0],     /* From */
 					&hdr_str,         /* Optional headers including CRLF */
 					(n<0)?&str_vals[2]:&body_str, /* Message body */
+					next_hop.len ? &next_hop: NULL, /* next hop */
 					m_tm_callback,    /* Callback function */
 					(void*)(long)mid        /* Callback parameter */
 				);
@@ -845,9 +725,9 @@ void m_clean_silo(unsigned int ticks, void *param)
 	db_val_t db_vals[MAX_DEL_KEYS];
 	db_op_t  db_ops[1] = { OP_LEQ };
 	int n;
-	
+
 	DBG("MSILO:clean_silo: cleaning stored messages - %d\n", ticks);
-	
+
 	msg_list_check(ml);
 	mle = p = msg_list_reset(ml);
 	n = 0;
@@ -863,7 +743,7 @@ void m_clean_silo(unsigned int ticks, void *param)
 			n++;
 			if(n==MAX_DEL_KEYS)
 			{
-				if (msilo_dbf.delete(db_con, db_keys, NULL, db_vals, n) < 0) 
+				if (msilo_dbf.delete(db_con, db_keys, NULL, db_vals, n) < 0)
 					DBG("MSILO:clean_silo: error cleaning %d messages.\n",n);
 				n = 0;
 			}
@@ -872,22 +752,22 @@ void m_clean_silo(unsigned int ticks, void *param)
 	}
 	if(n>0)
 	{
-		if (msilo_dbf.delete(db_con, db_keys, NULL, db_vals, n) < 0) 
+		if (msilo_dbf.delete(db_con, db_keys, NULL, db_vals, n) < 0)
 			DBG("MSILO:clean_silo: error cleaning %d messages\n", n);
 		n = 0;
 	}
 
 	msg_list_el_free_all(mle);
-	
+
 	/* cleaning expired messages */
 	if(ticks%(ms_check_time*ms_clean_period)<ms_check_time)
 	{
 		DBG("MSILO:clean_silo: cleaning expired messages\n");
 		db_keys[0] = sc_exp_time;
-		db_vals[0].type = DB_INT;
+		db_vals[0].type = DB_DATETIME;
 		db_vals[0].nul = 0;
-		db_vals[0].val.int_val = (int)time(NULL);
-		if (msilo_dbf.delete(db_con, db_keys, db_ops, db_vals, 1) < 0) 
+		db_vals[0].val.time_val = (int)time(NULL);
+		if (msilo_dbf.delete(db_con, db_keys, db_ops, db_vals, 1) < 0)
 			DBG("MSILO:clean_silo: ERROR cleaning expired messages\n");
 	}
 }
@@ -905,17 +785,20 @@ void destroy(void)
 		msilo_dbf.close(db_con);
 }
 
-/** 
+/**
  * TM callback function - delete message from database if was sent OK
  */
 void m_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 {
+	int mid = -1;
+	
 	DBG("MSILO:m_tm_callback: completed with status %d\n", ps->code);
 	if(!ps->param)
 	{
 		DBG("MSILO m_tm_callback: message id not received\n");
 		goto done;
 	}
+	mid = (int)(long)(*ps->param);
 	if(!db_con)
 	{
 		DBG("MSILO:m_tm_callback: db_con is NULL\n");
@@ -923,15 +806,15 @@ void m_tm_callback( struct cell *t, int type, struct tmcb_params *ps)
 	}
 	if(ps->code < 200 || ps->code >= 300)
 	{
-		DBG("MSILO:m_tm_callback: message <%ld> was not sent successfully\n",
-				(long)ps->param);
-		msg_list_set_flag(ml, (int)(long)ps->param, MS_MSG_ERRO);
+		
+		DBG("MSILO:m_tm_callback: message <%d> was not sent successfully\n",
+				mid);
+		msg_list_set_flag(ml, mid, MS_MSG_ERRO);
 		goto done;
 	}
 
-	msg_list_set_flag(ml, (int)(long)ps->param, MS_MSG_DONE);
-	
+	msg_list_set_flag(ml, mid, MS_MSG_DONE);
+
 done:
 	return;
 }
-
