@@ -1,7 +1,7 @@
 /*
  * Presence Agent, domain support
  *
- * $Id: pdomain.c,v 1.15.2.1 2006/12/13 11:05:07 kubartv Exp $
+ * $Id: pdomain.c,v 1.35 2006/12/12 15:59:20 kubartv Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -39,7 +39,9 @@
 #include "../../ut.h"
 #include "../../dprint.h"
 #include "../../mem/shm_mem.h"
-
+#include <cds/logger.h>
+#include "pa_mod.h"
+#include <time.h>
 
 /*
  * Hash function
@@ -69,7 +71,7 @@ int new_pdomain(str* _n, int _s, pdomain_t** _d, register_watcher_t _r, unregist
 	int i;
 	pdomain_t* ptr;
 	
-	ptr = (pdomain_t*)shm_malloc(sizeof(pdomain_t));
+	ptr = (pdomain_t*)mem_alloc(sizeof(pdomain_t));
 	if (!ptr) {
 		paerrno = PA_NO_MEMORY;
 		LOG(L_ERR, "new_pdomain(): No memory left\n");
@@ -77,11 +79,11 @@ int new_pdomain(str* _n, int _s, pdomain_t** _d, register_watcher_t _r, unregist
 	}
 	memset(ptr, 0, sizeof(pdomain_t));
 	
-	ptr->table = (hslot_t*)shm_malloc(sizeof(hslot_t) * _s);
+	ptr->table = (hslot_t*)mem_alloc(sizeof(hslot_t) * _s);
 	if (!ptr->table) {
 		paerrno = PA_NO_MEMORY;
 		LOG(L_ERR, "new_pdomain(): No memory left 2\n");
-		shm_free(ptr);
+		mem_free(ptr);
 		return -2;
 	}
 
@@ -116,73 +118,56 @@ void free_pdomain(pdomain_t* _d)
 		for(i = 0; i < _d->size; i++) {
 			deinit_slot(_d->table + i);
 		}
-		shm_free(_d->table);
+		mem_free(_d->table);
 	}
 	unlock_pdomain(_d);
 
-        shm_free(_d);
+        mem_free(_d);
 }
 
-
-/*
- * Just for debugging
- */
-void print_pdomain(FILE* _f, pdomain_t* _d)
-{
-	struct presentity* p;
-
-	fprintf(_f, "---pdomain---\n");
-	fprintf(_f, "name : '%.*s'\n", _d->name->len, ZSW(_d->name->s));
-	fprintf(_f, "size : %d\n", _d->size);
-	fprintf(_f, "table: %p\n", _d->table);
-	fprintf(_f, "first: %p\n", _d->first);
-	fprintf(_f, "last : %p\n", _d->last);
-	/* fprintf(_f, "lock : %d\n", _d->lock);*/ /*it can be a struct --andrei*/
-
-	if (_d->first) {
-		fprintf(_f, "\n");
-		p = _d->first;
-		while(p) {
-			print_presentity(_f, p);
-			p = p->next;
-		}
-		fprintf(_f, "\n");
-	}
-
-	fprintf(_f, "---pdomain---\n");
-}
-
-
 int timer_pdomain(pdomain_t* _d)
 {
 	struct presentity* presentity, *t;
+	time_t start, stop;
 
+	PROF_START(pa_timer_pdomain)
 	lock_pdomain(_d);
+	start = time(NULL);
 
 	presentity = _d->first;
 
-	if (0) LOG(L_ERR, "timer_pdomain: _d=%s d->first=%p\n", _d->name->s, presentity);
 	while(presentity) {
 		if (timer_presentity(presentity) < 0) {
 			LOG(L_ERR, "timer_pdomain(): Error in timer_pdomain\n");
 			unlock_pdomain(_d);
+			PROF_STOP(pa_timer_pdomain)
 			return -1;
 		}
 		
-		     /* Remove the entire record
-		      * if it is empty
-		      */
-		if (presentity->watchers == 0 && presentity->winfo_watchers==0) {
+		/* Remove the entire record
+		 * if it is empty
+		 */
+		if ( (!presentity->first_watcher) && 
+				(!presentity->first_winfo_watcher) && 
+				(!presentity->data.first_tuple) &&
+				(!presentity->data.first_note) &&
+				(!presentity->data.first_unknown_element) &&
+				(!presentity->first_qsa_subscription) &&
+				(presentity->ref_cnt == 0)) {
+			LOG(L_DBG, "timer_pdomain(): removing empty presentity\n");
 			t = presentity;
 			presentity = presentity->next;
-			remove_presentity(_d, t);
-			free_presentity(t);
+			release_presentity(t);
 		} else {
 			presentity = presentity->next;
 		}
 	}
 	
+	stop = time(NULL);
+	if (stop - start > 1) WARN("timer_pdomain took %d seconds\n", (int) (stop - start));
+			
 	unlock_pdomain(_d);
+	PROF_STOP(pa_timer_pdomain)
 	return 0;
 }
 
@@ -211,51 +196,95 @@ void unlock_pdomain(pdomain_t* _d)
 	     lock_release(&_d->lock);
 }
 
-
 /*
- * Find a presentity in domain
+ * Find a presentity in domain according to uid
  */
-int find_presentity(pdomain_t* _d, str* _uri, struct presentity** _p)
+int find_presentity_uid(pdomain_t* _d, str* uid, struct presentity** _p)
 {
 	unsigned int sl, i;
 	struct presentity* p;
-	
-	if (!_d->first) {
-	     pdomain_load_presentities(_d);
-	}
+	int res = 1;
 
-	sl = hash_func(_d, (unsigned char *)_uri->s, _uri->len);
+	if ((!uid) || (!_p)) return -1;
+
+	sl = hash_func(_d, (unsigned char *)uid->s, uid->len);
 	
 	p = _d->table[sl].first;
 	
 	for(i = 0; i < _d->table[sl].n; i++) {
-		if ((p->uri.len == _uri->len) && !memcmp(p->uri.s, _uri->s, _uri->len)) {
+		if ((p->uuid.len == uid->len) && !memcmp(p->uuid.s, uid->s, uid->len)) {
 			*_p = p;
-			return 0;
+			res = 0;
+			break;
 		}
-		
 		p = p->next;
 	}
-
-	return 1;   /* Nothing found */
+	
+	return res;   /* Nothing found */
 }
 
+/*
+ * contact will be NULL if user is offline
+ */
+static void callback(str* _user, str *_contact, int state, void* data)
+{
+	mq_message_t *msg;
+	tuple_change_info_t *info;
+
+	if ((!_user) || (!_contact) || (!data)) {
+		ERROR_LOG("callback(): error!\n");
+	}
+		
+	/* asynchronous processing */
+	msg = create_message_ex(sizeof(tuple_change_info_t));
+	if (!msg) {
+		LOG(L_ERR, "can't create message with tuple status change\n");
+		return;
+	}
+	set_data_destroy_function(msg, (destroy_function_f)free_tuple_change_info_content);
+	info = get_message_data(msg);
+	if (state == 0) info->state = presence_tuple_closed;
+	else info->state = presence_tuple_open;
+	str_dup(&info->user, _user);
+	str_dup(&info->contact, _contact);
+	if (data) push_message(&((struct presentity*)data)->mq, msg);
+}
 
 void add_presentity(pdomain_t* _d, struct presentity* _p)
 {
 	unsigned int sl;
 
-	LOG(L_WARN, "add_presentity _p=%p p_uri=%.*s\n", _p, _p->uri.len, _p->uri.s);
-
-	sl = hash_func(_d, (unsigned char *)_p->uri.s, _p->uri.len);
+	sl = hash_func(_d, (unsigned char *)_p->uuid.s, _p->uuid.len);
 
 	slot_add(&_d->table[sl], _p, &_d->first, &_d->last);
+
+	if (use_callbacks) {
+		DBG("! registering callback to %.*s, %p\n", _p->uuid.len, _p->uuid.s,_p);
+		_d->reg(&_p->data.uri, &_p->uuid, (void*)callback, _p);
+	}
+	if (subscribe_to_users) {
+		TRACE("! subscribing to %.*s, %p\n", _p->uuid.len, _p->uuid.s,_p);
+		subscribe_to_user(_p);
+	}
 }
 
 
 void remove_presentity(pdomain_t* _d, struct presentity* _p)
 {
-	return;
-	LOG(L_WARN, "remove_presentity _p=%p p_uri=%.*s\n", _p, _p->uri.len, _p->uri.s);
+	if (use_callbacks) {
+		DBG("! unregistering callback to %.*s, %p\n", _p->uuid.len, _p->uuid.s,_p);
+		_d->unreg(&_p->data.uri, &_p->uuid, (void*)callback, _p);
+		DBG("! unregistered callback to %.*s, %p\n", _p->uuid.len, _p->uuid.s,_p);
+	}
+	if (subscribe_to_users) {
+		DBG("! unsubscribing from %.*s, %p\n", _p->uuid.len, _p->uuid.s,_p);
+		unsubscribe_to_user(_p);
+	}
+	
+	LOG(L_DBG, "remove_presentity _p=%p p_uri=%.*s\n", _p, _p->data.uri.len, 
+			_p->data.uri.s);
 	slot_rem(_p->slot, _p, &_d->first, &_d->last);
+
+	/* remove presentity from database */
 }
+
