@@ -1,9 +1,10 @@
 /*
- * $Id: parse_config.h,v 1.2 2003/10/22 14:15:55 janakj Exp $
+ * $Id: parse_config.h,v 1.3 2006/08/11 13:46:07 tirpi Exp $
  *
  * PERMISSIONS module
  *
  * Copyright (C) 2003 Mikls Tirpk (mtirpak@sztaki.hu)
+ * Copyright (C) 2006 iptelorg GmbH
  *
  * This file is part of ser, a free SIP server.
  *
@@ -33,6 +34,6 @@
 
 #include "rule.h"
 
-rule *parse_config_file(char *filename);
+rule *parse_config_file(char *filename, int *err);
 
 #endif /* PARSE_CONFIG_H */
