@@ -1,5 +1,5 @@
 /*
- * $Id: common.h,v 1.10 2004/08/24 09:00:37 janakj Exp $
+ * $Id: common.h,v 1.12 2006/09/25 11:44:22 janakj Exp $
  *
  * Common stuff
  *
@@ -33,11 +33,8 @@
 
 #include "../../str.h"
 
-
-/*
- * Extract Address Of Record
- */
-int extract_aor(str* _uri, str* _a);
-
+#define VALID_AOR(contact, aor_filter) \
+    ((aor_filter).len == (contact)->aor.len && \
+     !strncasecmp((aor_filter).s, (contact)->aor.s, (aor_filter.len)))
 
 #endif /* COMMON_H */
