@@ -1,5 +1,5 @@
 /* 
- * $Id: reg_mod.h,v 1.18.2.1 2006/03/03 10:01:18 janakj Exp $ 
+ * $Id: reg_mod.h,v 1.25 2006/07/04 13:09:49 janakj Exp $ 
  *
  * registrar module interface
  *
@@ -35,27 +35,26 @@
 #include "../../qvalue.h"
 #include "../../usr_avp.h"
 #include "../usrloc/usrloc.h"
+#include "../sl/sl.h"
 
 extern int default_expires;
 extern qvalue_t default_q;
 extern int append_branches;
-extern int case_sensitive;
-extern int desc_time_order;
-extern int nat_flag;
+extern int load_nat_flag;
+extern int save_nat_flag;
 extern int min_expires;
 extern int max_expires;
 extern int received_avp;
-extern int use_domain;
-extern str realm_prefix;
 extern float def_q;
-
+extern int received_to_uri; /*copy received to uri, don't add it to dst_uri*/
 extern str rcv_param;
-extern int rcv_avp_no;
+extern str aor_attr;
 extern int max_contacts;
-extern int retry_after;
 
 extern usrloc_api_t ul;  /* Structure containing pointers to usrloc functions */
 
-extern int (*sl_reply)(struct sip_msg* _m, char* _s1, char* _s2);
+extern sl_api_t sl;
+
+extern avp_ident_t avpid_code, avpid_reason, avpid_contact;
 
 #endif /* REG_MOD_H */
