@@ -1,5 +1,5 @@
 /*
- * $Id: reply.h,v 1.6 2004/08/24 09:00:37 janakj Exp $
+ * $Id: reply.h,v 1.8 2006/09/25 11:44:22 janakj Exp $
  *
  * Send a reply
  *
@@ -31,6 +31,7 @@
 #ifndef REPLY_H
 #define REPLY_H
 
+#include "../../str.h"
 #include "../../parser/msg_parser.h"
 #include "ucontact.h"
 
@@ -43,7 +44,7 @@ int send_reply(struct sip_msg* _m);
 /*
  * Build Contact HF for reply
  */
-int build_contact(ucontact_t* c);
+int build_contact(ucontact_t* c, str* aor);
 
 
 /*
@@ -52,4 +53,9 @@ int build_contact(ucontact_t* c);
 void free_contact_buf(void);
 
 
+/*
+ * Sets the reason and contact attributes.
+ */
+int setup_attrs(struct sip_msg* msg);
+
 #endif /* REPLY_H */
