@@ -1,5 +1,5 @@
 /*
- * $Id: dlg.c,v 1.16 2004/08/24 09:00:40 janakj Exp $
+ * $Id: dlg.c,v 1.24.2.1 2008/06/24 16:12:09 janakj Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -45,6 +45,9 @@
 #include "t_reply.h"
 #include "../../parser/parser_f.h"
 
+/* next added to allow automatical tag generation */
+#include "callid.h"
+#include "uac.h"
 
 #define NORMAL_ORDER 0  /* Create route set in normal order - UAS */
 #define REVERSE_ORDER 1 /* Create route set in reverse order - UAC */
@@ -80,7 +83,6 @@ void get_raw_uri(str* _s)
 }
 
 
-
 /*
  * Make a copy of a str structure using shm_malloc
  */
@@ -130,12 +132,24 @@ static inline int calculate_hooks(dlg_t* _d)
 			_d->hooks.request_uri = &_d->route_set->nameaddr.uri;
 			_d->hooks.next_hop = _d->hooks.request_uri;
 			_d->hooks.first_route = _d->route_set->next;
-			_d->hooks.last_route = &_d->rem_target;
+			if (_d->rem_target.len > 0) 
+				_d->hooks.last_route = &_d->rem_target;
+			else 
+				_d->hooks.last_route = NULL; /* ? */
 		}
 	} else {
 		if (_d->rem_target.s) _d->hooks.request_uri = &_d->rem_target;
 		else _d->hooks.request_uri = &_d->rem_uri;
-		_d->hooks.next_hop = _d->hooks.request_uri;
+		
+		if (_d->dst_uri.s) _d->hooks.next_hop = &_d->dst_uri;
+		else _d->hooks.next_hop = _d->hooks.request_uri;
+
+		/*
+		 * the routes in the hooks need to be reset because if the route_set was dropped somewhere else
+		 * then these will remain set without the actual routes existing any more
+		 */
+		_d->hooks.first_route = 0;
+		_d->hooks.last_route = 0; 
 	}
 
 	if ((_d->hooks.request_uri) && (_d->hooks.request_uri->s) && (_d->hooks.request_uri->len)) {
@@ -169,6 +183,18 @@ int w_calculate_hooks(dlg_t* _d)
 int new_dlg_uac(str* _cid, str* _ltag, unsigned int _lseq, str* _luri, str* _ruri, dlg_t** _d)
 {
 	dlg_t* res;
+	str generated_cid;
+	str generated_ltag;
+
+	if (!_cid) { /* if not given, compute new one */
+		generate_callid(&generated_cid);
+		_cid = &generated_cid;
+	}
+	if (_cid && (!_ltag)) { /* if not given, compute new one */
+		generate_fromtag(&generated_ltag, _cid);
+		_ltag = &generated_ltag;
+	}
+	if (_lseq == 0) _lseq = DEFAULT_CSEQ;
 
 	if (!_cid || !_ltag || !_luri || !_ruri || !_d) {
 		LOG(L_ERR, "new_dlg_uac(): Invalid parameter value\n");
@@ -219,6 +245,7 @@ static inline int get_contact_uri(struct sip_msg* _m, str* _uri)
 	contact_t* c;
 
 	_uri->len = 0;
+	_uri->s = 0;
 
 	if (!_m->contact) return 1;
 
@@ -300,6 +327,21 @@ static inline int get_callid(struct sip_msg* _m, str* _cid)
 	return 0;
 }
 
+static rr_t *revert_route(rr_t *r)
+{
+	rr_t *a, *b;
+
+	a = NULL;
+
+	while (r) {
+		b = r->next;
+		r->next = a;
+		a = r;
+		r = b;
+	}
+
+	return a;
+}
 
 /*
  * Create a copy of route set either in normal or reverse order
@@ -313,41 +355,85 @@ static inline int get_route_set(struct sip_msg* _m, rr_t** _rs, unsigned char _o
 
 	ptr = _m->record_route;
 	while(ptr) {
-		if (ptr->type == HDR_RECORDROUTE) {
+		if (ptr->type == HDR_RECORDROUTE_T) {
 			if (parse_rr(ptr) < 0) {
 				LOG(L_ERR, "get_route_set(): Error while parsing Record-Route body\n");
 				goto error;
 			}
 
 			p = (rr_t*)ptr->parsed;
-			while(p) {
-				if (shm_duplicate_rr(&t, p) < 0) {
-					LOG(L_ERR, "get_route_set(): Error while duplicating rr_t\n");
-					goto error;
-				}
-				if (_order == NORMAL_ORDER) {
-					if (!*_rs) *_rs = t;
-					if (last) last->next = t;
-					last = t;
-				} else {
-					t->next = *_rs;
-					*_rs = t;
-				}
-
-				p = p->next;
+			if (shm_duplicate_rr(&t, p) < 0) {
+				LOG(L_ERR, "get_route_set(): Error while duplicating rr_t\n");
+				goto error;
 			}
-			
+			if (!*_rs) *_rs = t;
+			if (last) last->next = t;
+			last = t;
+			while (last->next) last = last->next; /* !!! there may be more routes in one hdr field !!! */
+
 		}
 		ptr = ptr->next;
 	}
+	if ((*_rs) && (_order != NORMAL_ORDER)) {
+		/* better to revert the route outside of cycle above */
+		*_rs = revert_route(*_rs);
+	}
 	
 	return 0;
 
  error:
-        shm_free_rr(_rs);
+	shm_free_rr(_rs);
 	return -1;
 }
 
+/*
+ * Extract method from CSeq header field
+ */
+static inline int get_cseq_method(struct sip_msg* _m, str* _method)
+{
+	if (!_m->cseq && ((parse_headers(_m, HDR_CSEQ_F, 0) == -1) || !_m->cseq)) {
+		LOG(L_ERR, "get_cseq_method(): Error while parsing CSeq\n");
+		return -1;
+	}
+
+	_method->s = get_cseq(_m)->method.s;
+	_method->len = get_cseq(_m)->method.len;
+	return 0;
+}
+
+
+static inline int refresh_dialog_resp(struct sip_msg* _m, 
+		target_refresh_t is_target_refresh)
+{
+	str method;
+	
+	switch (is_target_refresh) {
+		case IS_NOT_TARGET_REFRESH: 
+			return 0;
+		case IS_TARGET_REFRESH: 
+			return 1;
+		case TARGET_REFRESH_UNKNOWN: 
+			if (get_cseq_method(_m, &method) < 0) return 0; /* error */
+			if ((method.len == 6) && !memcmp("INVITE", method.s, 6)) 
+				return 1;
+			else return 0;
+	}
+	return 0;
+}
+		
+static inline int refresh_dialog_req(struct sip_msg* _m, target_refresh_t is_target_refresh)
+{
+	switch (is_target_refresh) {
+		case IS_NOT_TARGET_REFRESH: 
+			return 0;
+		case IS_TARGET_REFRESH: 
+			return 1;
+		case TARGET_REFRESH_UNKNOWN: 
+			return (_m->first_line.u.request.method_value == METHOD_INVITE);
+			break;
+	}
+	return 0;
+}
 
 /*
  * Extract all necessary information from a response and put it
@@ -358,12 +444,22 @@ static inline int response2dlg(struct sip_msg* _m, dlg_t* _d)
 	str contact, rtag;
 
 	     /* Parse the whole message, we will need all Record-Route headers */
-	if (parse_headers(_m, HDR_EOH, 0) == -1) {
+	if (parse_headers(_m, HDR_EOH_F, 0) == -1) {
 		LOG(L_ERR, "response2dlg(): Error while parsing headers\n");
 		return -1;
 	}
 	
 	if (get_contact_uri(_m, &contact) < 0) return -2;
+	if (_d->rem_target.s) {
+		shm_free(_d->rem_target.s);
+		_d->rem_target.s = 0; 
+		_d->rem_target.len = 0;
+	}
+	if (_d->dst_uri.s) {
+		shm_free(_d->dst_uri.s);
+		_d->dst_uri.s = 0; 
+		_d->dst_uri.len = 0;
+	}
 	if (contact.len && str_duplicate(&_d->rem_target, &contact) < 0) return -3;
 	
 	if (get_to_tag(_m, &rtag) < 0) goto err1;
@@ -451,6 +547,10 @@ static inline int dlg_early_resp_uac(dlg_t* _d, struct sip_msg* _m)
 		     /* We are in early state already, do nothing
 		      */
 	} else if ((code >= 200) && (code <= 299)) {
+		/* Warning - we can handle here response for non-initial request (for
+		 * example UPDATE within early INVITE/BYE dialog) and move into
+		 * confirmed state may be error! But this depends on dialog type... */
+		
 		     /* Same as in dlg_new_resp_uac */
 		     /* A final response, update the structures and transit
 		      * into DLG_CONFIRMED
@@ -473,30 +573,15 @@ static inline int dlg_early_resp_uac(dlg_t* _d, struct sip_msg* _m)
 }
 
 
-/*
- * Extract method from CSeq header field
- */
-static inline int get_cseq_method(struct sip_msg* _m, str* _method)
-{
-	if (!_m->cseq && ((parse_headers(_m, HDR_CSEQ, 0) == -1) || !_m->cseq)) {
-		LOG(L_ERR, "get_cseq_method(): Error while parsing CSeq\n");
-		return -1;
-	}
-
-	_method->s = get_cseq(_m)->method.s;
-	_method->len = get_cseq(_m)->method.len;
-	return 0;
-}
-
-
 /*
  * Handle dialog in DLG_CONFIRMED state, we will be processing
  * a response to a request sent within a dialog
  */
-static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m)
+static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m, 
+		target_refresh_t is_target_refresh)
 {
 	int code;
-	str method, contact;
+	str contact;
 
 	code = _m->first_line.u.reply.statuscode;	
 
@@ -506,10 +591,6 @@ static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m)
 	      * a target refresher. 
 	      */
 
-	     /* FIXME: Currently we support only INVITEs as target refreshers,
-	      * this should be generalized
-	      */
-
 	     /* IF we receive a 481 response, terminate the dialog because
 	      * the remote peer indicated that it didn't have the dialog
 	      * state anymore, signal this termination with a positive return
@@ -523,10 +604,9 @@ static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m)
 	     /* Do nothing if not 2xx */
 	if ((code < 200) || (code >= 300)) return 0;
 	
-	if (get_cseq_method(_m, &method) < 0) return -1;
-	if ((method.len == 6) && !memcmp("INVITE", method.s, 6)) {
+	if (refresh_dialog_resp(_m, is_target_refresh)) {
 		     /* Get contact if any and update remote target */
-		if (parse_headers(_m, HDR_CONTACT, 0) == -1) {
+		if (parse_headers(_m, HDR_CONTACT_F, 0) == -1) {
 			LOG(L_ERR, "dlg_confirmed_resp_uac(): Error while parsing headers\n");
 			return -2;
 		}
@@ -535,11 +615,19 @@ static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m)
 		if (get_contact_uri(_m, &contact) < 0) return -3;
 		     /* If there is a contact URI */
 		if (contact.len) {
-			     /* Free old remote target if any */
+			     /* Free old remote target and destination uri if any */
 			if (_d->rem_target.s) shm_free(_d->rem_target.s);
+			if (_d->dst_uri.s) {
+				shm_free(_d->dst_uri.s);
+				_d->dst_uri.s = 0;
+				_d->dst_uri.len = 0;
+			}
+				
 			     /* Duplicate new remote target */
 			if (str_duplicate(&_d->rem_target, &contact) < 0) return -4;
 		}
+
+		calculate_hooks(_d);
 	}
 
 	return 0;
@@ -549,7 +637,8 @@ static inline int dlg_confirmed_resp_uac(dlg_t* _d, struct sip_msg* _m)
 /*
  * A response arrived, update dialog
  */
-int dlg_response_uac(dlg_t* _d, struct sip_msg* _m)
+int dlg_response_uac(dlg_t* _d, struct sip_msg* _m, 
+		target_refresh_t is_target_refresh)
 {
 	if (!_d || !_m) {
 		LOG(L_ERR, "dlg_response_uac(): Invalid parameter value\n");
@@ -565,10 +654,10 @@ int dlg_response_uac(dlg_t* _d, struct sip_msg* _m)
 		return dlg_early_resp_uac(_d, _m);
 
 	case DLG_CONFIRMED: 
-		return dlg_confirmed_resp_uac(_d, _m);
+		return dlg_confirmed_resp_uac(_d, _m, is_target_refresh);
 
 	case DLG_DESTROYED:
-		LOG(L_ERR, "dlg_response_uac(): Cannot handle destroyed dialog\n");
+		LOG(L_DBG, "dlg_response_uac(): Cannot handle destroyed dialog\n");
 		return -2;
 	}
 
@@ -674,13 +763,21 @@ static inline int request2dlg(struct sip_msg* _m, dlg_t* _d)
 {
 	str contact, rtag, callid;
 
-	if (parse_headers(_m, HDR_EOH, 0) == -1) {
+	if (parse_headers(_m, HDR_EOH_F, 0) == -1) {
 		LOG(L_ERR, "request2dlg(): Error while parsing headers");
 		return -1;
 	}
 
 	if (get_contact_uri(_m, &contact) < 0) return -2;
-	if (contact.len && str_duplicate(&_d->rem_target, &contact) < 0) return -3;
+	if (contact.len) {
+		if (_d->rem_target.s) shm_free(_d->rem_target.s);
+		if (_d->dst_uri.s) {
+			shm_free(_d->dst_uri.s);
+			_d->dst_uri.s = 0;
+			_d->dst_uri.len = 0;
+		}
+		if (str_duplicate(&_d->rem_target, &contact) < 0) return -3;
+	}
 	
 	if (get_from_tag(_m, &rtag) < 0) goto err1;
 	if (rtag.len && str_duplicate(&_d->id.rem_tag, &rtag) < 0) goto err1;
@@ -734,9 +831,8 @@ int new_dlg_uas(struct sip_msg* _req, int _code, /*str* _tag,*/ dlg_t** _d)
 		return -1;
 	}
 
-	if ((_code < 200) || (_code > 299)) {
-		DBG("new_dlg_uas(): Not a 2xx, no dialog created\n");
-		return -2;
+	if (_code > 299) {
+		DBG("new_dlg_uas(): Status code >= 300, no dialog created\n");
 	}
 
 	res = (dlg_t*)shm_malloc(sizeof(dlg_t));
@@ -749,34 +845,82 @@ int new_dlg_uas(struct sip_msg* _req, int _code, /*str* _tag,*/ dlg_t** _d)
 
 	if (request2dlg(_req, res) < 0) {
 		LOG(L_ERR, "new_dlg_uas(): Error while converting request to dialog\n");
+		free_dlg(res);
 		return -4;
 	}
 
-	tag.s = tm_tags;
-	tag.len = TOTAG_VALUE_LEN;
-	calc_crc_suffix(_req, tm_tag_suffix);
-	if (str_duplicate(&res->id.loc_tag, &tag) < 0) {
-		free_dlg(res);
-		return -5;
+	if (_code > 100) {
+		tag.s = tm_tags;
+		tag.len = TOTAG_VALUE_LEN;
+		calc_crc_suffix(_req, tm_tag_suffix);
+		if (str_duplicate(&res->id.loc_tag, &tag) < 0) {
+			free_dlg(res);
+			return -5;
+		}
 	}
 	
 	*_d = res;
 
-	(*_d)->state = DLG_CONFIRMED;
+	if (_code < 100)
+		(*_d)->state = DLG_NEW;
+	else if (_code < 200)
+		(*_d)->state = DLG_EARLY;
+	else
+		(*_d)->state = DLG_CONFIRMED;
+
 	if (calculate_hooks(*_d) < 0) {
 		LOG(L_ERR, "new_dlg_uas(): Error while calculating hooks\n");
-		shm_free(*_d);
+		free_dlg(res);
 		return -6;
 	}
 
 	return 0;
 }
 
+/*
+ * UAS side - update dialog state and to tag
+ */
+int update_dlg_uas(dlg_t *_d, int _code, str* _tag)
+{
+	if (_d->state == DLG_CONFIRMED) {
+		LOG(L_ERR, "update_dlg_uas(): Dialog is already confirmed\n");
+		return -1;
+	} else if (_d->state == DLG_DESTROYED) {
+		LOG(L_ERR, "update_dlg_uas(): Dialog is already destroyed\n");
+		return -2;
+	}
+
+	if (_tag && _tag->s) {
+		if (_d->id.loc_tag.s) {
+			if ((_tag->len == _d->id.loc_tag.len)
+			&& (!memcmp(_tag->s, _d->id.loc_tag.s, _tag->len))) {
+				LOG(L_DBG, "update_dlg_uas(): Local tag is already set\n");
+			} else {
+				LOG(L_ERR, "update_dlg_uas(): ERROR: trying to rewrite local tag\n");
+				return -3;
+			}
+		} else {
+			if (str_duplicate(&_d->id.loc_tag, _tag) < 0) {
+				LOG(L_ERR, "update_dlg_uas(): Not enough memory\n");
+				return -4;
+			}
+		}
+	}
+
+	if ((100 < _code) && (_code < 200))
+		_d->state = DLG_EARLY;
+	else if (_code < 300)
+		_d->state = DLG_CONFIRMED;
+	else
+		_d->state = DLG_DESTROYED;
+
+	return 0;
+}
 
 /*
  * UAS side - update a dialog from a request
  */
-int dlg_request_uas(dlg_t* _d, struct sip_msg* _m)
+int dlg_request_uas(dlg_t* _d, struct sip_msg* _m, target_refresh_t is_target_refresh)
 {
 	str contact;
 	int cseq;
@@ -789,7 +933,7 @@ int dlg_request_uas(dlg_t* _d, struct sip_msg* _m)
 	     /* We must check if the request is not out of order or retransmission
 	      * first, if so then we will not update anything
 	      */
-	if (parse_headers(_m, HDR_CSEQ, 0) == -1) {
+	if (parse_headers(_m, HDR_CSEQ_F, 0) == -1) {
 		LOG(L_ERR, "dlg_request_uas(): Error while parsing headers\n");
 		return -2;
 	}
@@ -803,9 +947,8 @@ int dlg_request_uas(dlg_t* _d, struct sip_msg* _m)
 	     /* We will als update remote target URI if the message 
 	      * is target refresher
 	      */
-	if (_m->first_line.u.request.method_value == METHOD_INVITE) {
-		     /* target refresher */
-		if (parse_headers(_m, HDR_CONTACT, 0) == -1) {
+	if (refresh_dialog_req(_m, is_target_refresh)) { /* target refresher */
+		if (parse_headers(_m, HDR_CONTACT_F, 0) == -1) {
 			LOG(L_ERR, "dlg_request_uas(): Error while parsing headers\n");
 			return -4;
 		}
@@ -813,8 +956,16 @@ int dlg_request_uas(dlg_t* _d, struct sip_msg* _m)
 		if (get_contact_uri(_m, &contact) < 0) return -5;
 		if (contact.len) {
 			if (_d->rem_target.s) shm_free(_d->rem_target.s);
+			if (_d->dst_uri.s) {
+				shm_free(_d->dst_uri.s);
+				_d->dst_uri.s = 0;
+				_d->dst_uri.len = 0;
+			}
 			if (str_duplicate(&_d->rem_target, &contact) < 0) return -6;
 		}
+
+		calculate_hooks(_d);
+		
 	}
 
 	return 0;
@@ -914,6 +1065,7 @@ void free_dlg(dlg_t* _d)
 	if (_d->loc_uri.s) shm_free(_d->loc_uri.s);
 	if (_d->rem_uri.s) shm_free(_d->rem_uri.s);
 	if (_d->rem_target.s) shm_free(_d->rem_target.s);
+	if (_d->dst_uri.s) shm_free(_d->dst_uri.s);
 
 	     /* Free all routes in the route set */
 	shm_free_rr(&_d->route_set);
@@ -937,6 +1089,7 @@ void print_dlg(FILE* out, dlg_t* _d)
 	fprintf(out, "loc_uri       : '%.*s'\n", _d->loc_uri.len, _d->loc_uri.s);
 	fprintf(out, "rem_uri       : '%.*s'\n", _d->rem_uri.len, _d->rem_uri.s);
 	fprintf(out, "rem_target    : '%.*s'\n", _d->rem_target.len, _d->rem_target.s);
+	fprintf(out, "dst_uri       : '%.*s'\n", _d->dst_uri.len, _d->dst_uri.s);
 	fprintf(out, "secure:       : %d\n", _d->secure);
 	fprintf(out, "state         : ");
 	switch(_d->state) {
@@ -957,3 +1110,33 @@ void print_dlg(FILE* out, dlg_t* _d)
 	
 	fprintf(out, "====dlg_t====\n");
 }
+
+/*
+ * set dialog's request uri and destination uri (optional)
+ */
+int set_dlg_target(dlg_t* _d, str* _ruri, str* _duri) {
+
+	if (!_d || !_ruri) {
+		LOG(L_ERR, "set_dlg_target(): Invalid parameter value\n");
+		return -1;
+	}
+
+	if (_d->rem_target.s) shm_free(_d->rem_target.s);
+	if (_d->dst_uri.s) {
+		shm_free(_d->dst_uri.s);
+		_d->dst_uri.s = 0;
+		_d->dst_uri.len = 0;
+	}
+
+	if (str_duplicate(&_d->rem_target, _ruri)) return -1;
+	if (_duri && _duri->len) {
+		if (str_duplicate(&_d->dst_uri, _duri)) return -1;
+	}
+
+	if (calculate_hooks(_d)) {
+		LOG(L_ERR, "set_dlg_target(): Error while calculating hooks\n");
+		return -1;
+	}
+
+	return 0;
+}
