@@ -1,11 +1,11 @@
 /*
- * $Id: sip_msg.c,v 1.84.2.3 2006/01/11 00:28:18 janakj Exp $
- * 
+ * $Id: sip_msg.c,v 1.99.2.1 2007/05/01 22:03:16 sobomax Exp $
+ *
  * cloning a message into shared memory (TM keeps a snapshot
  * of messages in memory); note that many operations, which
  * allocate pkg memory (such as parsing) cannot be used with
  * a cloned message -- it would result in linking pkg structures
- * to shmem msg and eventually in a memory error 
+ * to shmem msg and eventually in a memory error
  *
  * the cloned message is stored in a single memory fragment to
  * save too many shm_mallocs -- these are expensive as they
@@ -31,8 +31,8 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License 
- * along with this program; if not, write to the Free Software 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * History:
@@ -46,6 +46,8 @@
  *  2003-05-07  received, rport & i via shortcuts are also translated (andrei)
  *  2003-11-11  updated cloning of lump_rpl (bogdan)
  *  2004-03-31  alias shortcuts are also translated (andrei)
+ *  2006-04-20  via->comp is also translated (andrei)
+ *  2006-10-16  HDR_{PROXY,WWW}_AUTHENTICATE_T cloned (andrei)
  */
 
 #include "defs.h"
@@ -61,7 +63,7 @@
 #include "../../parser/digest/digest.h"
 
 
-/* rounds to the first 4 byte multiple on 32 bit archs 
+/* rounds to the first 4 byte multiple on 32 bit archs
  * and to the first 8 byte multiple on 64 bit archs */
 #define ROUND4(s) \
 	(((s)+(sizeof(char*)-1))&(~(sizeof(char*)-1)))
@@ -136,7 +138,7 @@ inline struct via_body* via_body_cloner( char* new_buf,
 				new_vp->name.s=translate_pointer(new_buf,org_buf,vp->name.s);
 				new_vp->value.s=translate_pointer(new_buf,org_buf,vp->value.s);
 				new_vp->start=translate_pointer(new_buf,org_buf,vp->start);
-				
+
 				/* "translate" the shortcuts */
 				switch(new_vp->type){
 					case PARAM_BRANCH:
@@ -154,6 +156,12 @@ inline struct via_body* via_body_cloner( char* new_buf,
 					case PARAM_ALIAS:
 							new_via->alias = new_vp;
 							break;
+
+#ifdef USE_COMP
+					case PARAM_COMP:
+							new_via->comp = new_vp;
+							break;
+#endif
 				}
 
 				if (last_new_vp)
@@ -197,7 +205,7 @@ static inline struct auth_body* auth_body_cloner(char* new_buf, char *org_buf, s
 	new_auth = (struct auth_body*)(*p);
 	memcpy(new_auth , auth , sizeof(struct auth_body));
 	(*p) += ROUND4(sizeof(struct auth_body));
-	
+
 	/* authorized field must be cloned elsewhere */
 	new_auth->digest.username.whole.s =
 		translate_pointer(new_buf, org_buf, auth->digest.username.whole.s);
@@ -235,7 +243,7 @@ static inline int clone_authorized_hooks(struct sip_msg* new,
 
 	get_authorized_cred(old->authorization, &hook1);
 	if (!hook1) stop = 1;
-	
+
 	get_authorized_cred(old->proxy_auth, &hook2);
 	if (!hook2) stop |= 2;
 
@@ -252,7 +260,7 @@ static inline int clone_authorized_hooks(struct sip_msg* new,
 				new_ptr;
 			stop |= 1;
 		}
-		
+
 		if (ptr == hook2) {
 			if (!new->proxy_auth || !new->proxy_auth->parsed) {
 				LOG(L_CRIT, "BUG: Error in message cloner (proxy_auth)\n");
@@ -276,6 +284,8 @@ static inline int clone_authorized_hooks(struct sip_msg* new,
 
 #define HOOK_SET(hook) (new_msg->hook != org_msg->hook)
 
+/* Warning: Cloner does not clone all hdr_field headers (From, To, etc.). Pointers will reference pkg memory. Dereferencing will crash ser!!! */
+
 struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len )
 {
 	unsigned int      len;
@@ -287,7 +297,6 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len )
 	struct lump_rpl   *rpl_lump, **rpl_lump_anchor;
 	char              *p;
 
-
 	/*computing the length of entire sip_msg structure*/
 	len = ROUND4(sizeof( struct sip_msg ));
 	/*we will keep only the original msg +ZT */
@@ -303,74 +312,87 @@ struct sip_msg*  sip_msg_cloner( struct sip_msg *org_msg, int *sip_msg_len )
 	{
 		/*size of header struct*/
 		len += ROUND4(sizeof( struct hdr_field));
-		switch (hdr->type)
-		{
-			case HDR_VIA:
-				for (via=(struct via_body*)hdr->parsed;via;via=via->next)
-				{
-					len+=ROUND4(sizeof(struct via_body));
-					/*via param*/
-					for(prm=via->param_lst;prm;prm=prm->next)
-						len+=ROUND4(sizeof(struct via_param ));
-				}
-				break;
+		switch (hdr->type) {
+			     /* Safely ignore auxiliary header types */
+		case HDR_ERROR_T:
+		case HDR_OTHER_T:
+		case HDR_VIA2_T:
+		case HDR_EOH_T:
+			break;
 
-			case HDR_TO:
-			case HDR_FROM:
-				/* From header might be unparsed */
-				if (hdr->parsed) {
-					len+=ROUND4(sizeof(struct to_body));
-					     /*to param*/
-					to_prm = ((struct to_body*)(hdr->parsed))->param_lst;
-					for(;to_prm;to_prm=to_prm->next)
-						len+=ROUND4(sizeof(struct to_param ));
-				}
-				break;
+		case HDR_VIA_T:
+			for (via=(struct via_body*)hdr->parsed;via;via=via->next) {
+				len+=ROUND4(sizeof(struct via_body));
+				     /*via param*/
+				for(prm=via->param_lst;prm;prm=prm->next)
+					len+=ROUND4(sizeof(struct via_param ));
+			}
+			break;
 
-			case HDR_CSEQ:
-				len+=ROUND4(sizeof(struct cseq_body));
-				break;
+		case HDR_TO_T:
+		case HDR_FROM_T:
+			     /* From header might be unparsed */
+			if (hdr->parsed) {
+				len+=ROUND4(sizeof(struct to_body));
+				     /*to param*/
+				to_prm = ((struct to_body*)(hdr->parsed))->param_lst;
+				for(;to_prm;to_prm=to_prm->next)
+					len+=ROUND4(sizeof(struct to_param ));
+			}
+			break;
+
+		case HDR_CSEQ_T:
+			len+=ROUND4(sizeof(struct cseq_body));
+			break;
 
 
-			case HDR_AUTHORIZATION:
-			case HDR_PROXYAUTH:
-				if (hdr->parsed) {
-					len += ROUND4(AUTH_BODY_SIZE);
-				}
-				break;
+		case HDR_AUTHORIZATION_T:
+		case HDR_PROXYAUTH_T:
+			if (hdr->parsed) {
+				len += ROUND4(AUTH_BODY_SIZE);
+			}
+			break;
 
-			case HDR_CALLID:
-			case HDR_CONTACT:
-			case HDR_MAXFORWARDS:
-			case HDR_ROUTE:
-			case HDR_RECORDROUTE:
-			case HDR_CONTENTTYPE:
-			case HDR_CONTENTLENGTH:
-			case HDR_EXPIRES:
-			case HDR_SUPPORTED:
-			case HDR_PROXYREQUIRE:
-			case HDR_UNSUPPORTED:
-			case HDR_ALLOW:
-			case HDR_EVENT:
-			case HDR_ACCEPT:
-			case HDR_ACCEPTLANGUAGE:
-			case HDR_ORGANIZATION:
-			case HDR_PRIORITY:
-			case HDR_SUBJECT:
-			case HDR_USERAGENT:
-			case HDR_ACCEPTDISPOSITION:
-			case HDR_CONTENTDISPOSITION:
-		        case HDR_DIVERSION:
-		        case HDR_RPID:
-				/* we ignore them for now even if they have something parsed*/
-				break;
-
-			default:
-				if (hdr->parsed) {
-					LOG(L_WARN, "WARNING: sip_msg_cloner: "
-						"header body ignored: %d\n", hdr->type );
-				}
-				break;
+		case HDR_CALLID_T:
+		case HDR_CONTACT_T:
+		case HDR_MAXFORWARDS_T:
+		case HDR_ROUTE_T:
+		case HDR_RECORDROUTE_T:
+		case HDR_CONTENTTYPE_T:
+		case HDR_CONTENTLENGTH_T:
+		case HDR_EXPIRES_T:
+		case HDR_SUPPORTED_T:
+		case HDR_REQUIRE_T:
+		case HDR_PROXYREQUIRE_T:
+		case HDR_UNSUPPORTED_T:
+		case HDR_ALLOW_T:
+		case HDR_EVENT_T:
+		case HDR_ACCEPT_T:
+		case HDR_ACCEPTLANGUAGE_T:
+		case HDR_ORGANIZATION_T:
+		case HDR_PRIORITY_T:
+		case HDR_SUBJECT_T:
+		case HDR_USERAGENT_T:
+		case HDR_SERVER_T:
+		case HDR_ACCEPTDISPOSITION_T:
+		case HDR_CONTENTDISPOSITION_T:
+		case HDR_DIVERSION_T:
+		case HDR_RPID_T:
+		case HDR_REFER_TO_T:
+		case HDR_SIPIFMATCH_T:
+		case HDR_SESSIONEXPIRES_T:
+		case HDR_MIN_SE_T:
+		case HDR_SUBSCRIPTION_STATE_T:
+		case HDR_ACCEPTCONTACT_T:
+		case HDR_ALLOWEVENTS_T:
+		case HDR_CONTENTENCODING_T:
+		case HDR_REFERREDBY_T:
+		case HDR_REJECTCONTACT_T:
+		case HDR_REQUESTDISPOSITION_T:
+		case HDR_WWW_AUTHENTICATE_T:
+		case HDR_PROXY_AUTHENTICATE_T:
+			/* we ignore them for now even if they have something parsed*/
+			break;
 		}/*switch*/
 	}/*for all headers*/
 
@@ -400,7 +422,7 @@ do { \
 
 	LUMP_LIST_LEN(len, org_msg->add_rm);
 	LUMP_LIST_LEN(len, org_msg->body_lumps);
-	
+
 	/*length of reply lump structures*/
 	for(rpl_lump=org_msg->reply_lump;rpl_lump;rpl_lump=rpl_lump->next)
 			len+=ROUND4(sizeof(struct lump_rpl))+ROUND4(rpl_lump->text.len);
@@ -418,6 +440,8 @@ do { \
 	new_msg = (struct sip_msg*)p;
 	/* sip msg structure */
 	memcpy( new_msg , org_msg , sizeof(struct sip_msg) );
+
+	new_msg->msg_flags |= FL_SHM_CLONE;
 	p += ROUND4(sizeof(struct sip_msg));
 	new_msg->add_rm = 0;
 	new_msg->body_lumps = 0;
@@ -473,9 +497,10 @@ do { \
 			org_msg->first_line.u.reply.reason.s );
 	}
 
-	/*headers list*/
-	new_msg->via1=0;
-	new_msg->via2=0;
+       /*headers list*/
+       new_msg->via1=0;
+       new_msg->via2=0;
+
 	for( hdr=org_msg->headers,last_hdr=0 ; hdr ; hdr=hdr->next )
 	{
 		new_hdr = (struct hdr_field*)p;
@@ -493,232 +518,275 @@ do { \
 
 		switch (hdr->type)
 		{
-			case HDR_VIA:
-				/*fprintf(stderr,"prepare to clone via |%.*s|\n",
-					via_len((struct via_body*)hdr->parsed),
-					via_s((struct via_body*)hdr->parsed,org_msg));*/
-				if ( !new_msg->via1 )
-				{
-					new_msg->h_via1 = new_hdr;
-					new_msg->via1 = via_body_cloner(new_msg->buf,
-						org_msg->buf, (struct via_body*)hdr->parsed, &p);
-					new_hdr->parsed  = (void*)new_msg->via1;
-					/*fprintf(stderr,"setting via1 |%.*s|\n",
-						via_len(new_msg->via1),
-						via_s(new_msg->via1,new_msg));*/
-					if ( new_msg->via1->next )
-						new_msg->via2 = new_msg->via1->next;
+			     /* Ignore auxiliary header types */
+		case HDR_ERROR_T:
+		case HDR_OTHER_T:
+		case HDR_VIA2_T:
+		case HDR_EOH_T:
+		case HDR_ACCEPTCONTACT_T:
+		case HDR_ALLOWEVENTS_T:
+		case HDR_CONTENTENCODING_T:
+		case HDR_REFERREDBY_T:
+		case HDR_REJECTCONTACT_T:
+		case HDR_REQUESTDISPOSITION_T:
+		case HDR_WWW_AUTHENTICATE_T:
+		case HDR_PROXY_AUTHENTICATE_T:
+			break;
+
+		case HDR_VIA_T:
+			if ( !new_msg->via1 ) {
+				new_msg->h_via1 = new_hdr;
+				new_msg->via1 = via_body_cloner(new_msg->buf,
+								org_msg->buf, (struct via_body*)hdr->parsed, &p);
+				new_hdr->parsed  = (void*)new_msg->via1;
+				if ( new_msg->via1->next ) {
+					new_msg->via2 = new_msg->via1->next;
 				}
-				else if ( !new_msg->via2 && new_msg->via1 )
-				{
-					new_msg->h_via2 = new_hdr;
-					if ( new_msg->via1->next )
-						new_hdr->parsed = (void*)new_msg->via1->next;
-					else{
-						new_msg->via2 = via_body_cloner( new_msg->buf,
-							org_msg->buf, (struct via_body*)hdr->parsed, &p);
-						new_hdr->parsed  = (void*)new_msg->via2;
-					}
-				}
-				else if ( new_msg->via2 && new_msg->via1 )
-				{
-					new_hdr->parsed =  
-						via_body_cloner( new_msg->buf , org_msg->buf ,
-						(struct via_body*)hdr->parsed , &p);
-				}
-				break;
-			case HDR_CSEQ:
-				new_hdr->parsed = p;
-				p +=ROUND4(sizeof(struct cseq_body));
-				memcpy(new_hdr->parsed, hdr->parsed, sizeof(struct cseq_body));
-				((struct cseq_body*)new_hdr->parsed)->number.s =
-					translate_pointer(new_msg->buf ,org_msg->buf,
-					((struct cseq_body*)hdr->parsed)->number.s );
-				((struct cseq_body*)new_hdr->parsed)->method.s =
-					translate_pointer(new_msg->buf ,org_msg->buf,
-					((struct cseq_body*)hdr->parsed)->method.s );
-				if (!HOOK_SET(cseq)) new_msg->cseq = new_hdr;
-				break;
-			case HDR_TO:
-			case HDR_FROM:
-				if (hdr->type == HDR_TO) {
-					if (!HOOK_SET(to)) new_msg->to = new_hdr;
+			} else if ( !new_msg->via2 && new_msg->via1 ) {
+				new_msg->h_via2 = new_hdr;
+				if ( new_msg->via1->next ) {
+					new_hdr->parsed = (void*)new_msg->via1->next;
 				} else {
-					if (!HOOK_SET(from)) new_msg->from = new_hdr;
+					new_msg->via2 = via_body_cloner( new_msg->buf,
+									 org_msg->buf, (struct via_body*)hdr->parsed, &p);
+					new_hdr->parsed  = (void*)new_msg->via2;
 				}
-				/* From header might be unparsed */
-				if (!hdr->parsed) break;
-				new_hdr->parsed = p;
-				p +=ROUND4(sizeof(struct to_body));
-				memcpy(new_hdr->parsed, hdr->parsed, sizeof(struct to_body));
-				((struct to_body*)new_hdr->parsed)->body.s =
+			} else if ( new_msg->via2 && new_msg->via1 ) {
+				new_hdr->parsed = via_body_cloner( new_msg->buf , org_msg->buf ,
+								   (struct via_body*)hdr->parsed , &p);
+			}
+			break;
+		case HDR_CSEQ_T:
+			new_hdr->parsed = p;
+			p +=ROUND4(sizeof(struct cseq_body));
+			memcpy(new_hdr->parsed, hdr->parsed, sizeof(struct cseq_body));
+			((struct cseq_body*)new_hdr->parsed)->number.s =
+				translate_pointer(new_msg->buf ,org_msg->buf,
+						  ((struct cseq_body*)hdr->parsed)->number.s );
+			((struct cseq_body*)new_hdr->parsed)->method.s =
+				translate_pointer(new_msg->buf ,org_msg->buf,
+						  ((struct cseq_body*)hdr->parsed)->method.s );
+			if (!HOOK_SET(cseq)) new_msg->cseq = new_hdr;
+			break;
+		case HDR_TO_T:
+		case HDR_FROM_T:
+			if (hdr->type == HDR_TO_T) {
+				if (!HOOK_SET(to)) new_msg->to = new_hdr;
+			} else {
+				if (!HOOK_SET(from)) new_msg->from = new_hdr;
+			}
+			     /* From header might be unparsed */
+			if (!hdr->parsed) break;
+			new_hdr->parsed = p;
+			p +=ROUND4(sizeof(struct to_body));
+			memcpy(new_hdr->parsed, hdr->parsed, sizeof(struct to_body));
+			((struct to_body*)new_hdr->parsed)->body.s =
+				translate_pointer( new_msg->buf , org_msg->buf ,
+						   ((struct to_body*)hdr->parsed)->body.s );
+			((struct to_body*)new_hdr->parsed)->display.s =
+				translate_pointer( new_msg->buf, org_msg->buf,
+						   ((struct to_body*)hdr->parsed)->display.s);
+			((struct to_body*)new_hdr->parsed)->uri.s =
+				translate_pointer( new_msg->buf , org_msg->buf ,
+						   ((struct to_body*)hdr->parsed)->uri.s );
+			if ( ((struct to_body*)hdr->parsed)->tag_value.s )
+				((struct to_body*)new_hdr->parsed)->tag_value.s =
 					translate_pointer( new_msg->buf , org_msg->buf ,
-					((struct to_body*)hdr->parsed)->body.s );
-				((struct to_body*)new_hdr->parsed)->uri.s =
-					translate_pointer( new_msg->buf , org_msg->buf ,
-					((struct to_body*)hdr->parsed)->uri.s );
-				if ( ((struct to_body*)hdr->parsed)->tag_value.s )
-					((struct to_body*)new_hdr->parsed)->tag_value.s =
-						translate_pointer( new_msg->buf , org_msg->buf ,
-						((struct to_body*)hdr->parsed)->tag_value.s );
-				/*to params*/
-				to_prm = ((struct to_body*)(hdr->parsed))->param_lst;
-				for(;to_prm;to_prm=to_prm->next)
-				{
-					/*alloc*/
-					new_to_prm = (struct to_param*)p;
-					p +=ROUND4(sizeof(struct to_param ));
-					/*coping*/
-					memcpy( new_to_prm, to_prm, sizeof(struct to_param ));
-					((struct to_body*)new_hdr->parsed)->param_lst = 0;
-					new_to_prm->name.s = translate_pointer( new_msg->buf,
-						org_msg->buf , to_prm->name.s );
-					new_to_prm->value.s = translate_pointer( new_msg->buf,
-						org_msg->buf , to_prm->value.s );
-					/*linking*/
-					if ( !((struct to_body*)new_hdr->parsed)->param_lst )
-						((struct to_body*)new_hdr->parsed)->param_lst
-							= new_to_prm;
-					else
-						((struct to_body*)new_hdr->parsed)->last_param->next
-							= new_to_prm;
-					((struct to_body*)new_hdr->parsed)->last_param
+							   ((struct to_body*)hdr->parsed)->tag_value.s );
+			     /*to params*/
+			to_prm = ((struct to_body*)(hdr->parsed))->param_lst;
+			for(;to_prm;to_prm=to_prm->next) {
+				     /*alloc*/
+				new_to_prm = (struct to_param*)p;
+				p +=ROUND4(sizeof(struct to_param ));
+				     /*coping*/
+				memcpy( new_to_prm, to_prm, sizeof(struct to_param ));
+				((struct to_body*)new_hdr->parsed)->param_lst = 0;
+				new_to_prm->name.s = translate_pointer( new_msg->buf,
+									org_msg->buf , to_prm->name.s );
+				new_to_prm->value.s = translate_pointer( new_msg->buf,
+									 org_msg->buf , to_prm->value.s );
+				     /*linking*/
+				if ( !((struct to_body*)new_hdr->parsed)->param_lst )
+					((struct to_body*)new_hdr->parsed)->param_lst
 						= new_to_prm;
-				}
-				break;
-			case HDR_CALLID:
-				if (!HOOK_SET(callid)) {
-					new_msg->callid = new_hdr;
-				}
-				break;
-			case HDR_CONTACT:
-				if (!HOOK_SET(contact)) {
-					new_msg->contact = new_hdr;
-				}
-				break;
-			case HDR_MAXFORWARDS :
-				if (!HOOK_SET(maxforwards)) {
-					new_msg->maxforwards = new_hdr;
-				}
-				break;
-			case HDR_ROUTE :
-				if (!HOOK_SET(route)) {
-					new_msg->route = new_hdr;
-				}
-				break;
-			case HDR_RECORDROUTE :
-				if (!HOOK_SET(record_route)) {
-					new_msg->record_route = new_hdr;
-				}
-				break;
-			case HDR_CONTENTTYPE :
-				if (!HOOK_SET(content_type)) {
-					new_msg->content_type = new_hdr;
-					new_msg->content_type->parsed = hdr->parsed;
-				}
-				break;
-			case HDR_CONTENTLENGTH :
-				if (!HOOK_SET(content_length)) {
-					new_msg->content_length = new_hdr;
-					new_msg->content_length->parsed = hdr->parsed;
-				}
-				break;
-			case HDR_AUTHORIZATION :
-				if (!HOOK_SET(authorization)) {
-					new_msg->authorization = new_hdr;
-				}
-				if (hdr->parsed) {
-					new_hdr->parsed = auth_body_cloner(new_msg->buf ,
-						org_msg->buf , (struct auth_body*)hdr->parsed , &p);
-				}
-				break;
-			case HDR_EXPIRES :
-				if (!HOOK_SET(expires)) {
-					new_msg->expires = new_hdr;
-				}
-				break;
-			case HDR_PROXYAUTH :
-				if (!HOOK_SET(proxy_auth)) {
-					new_msg->proxy_auth = new_hdr;
-				}
-				if (hdr->parsed) {
-					new_hdr->parsed = auth_body_cloner(new_msg->buf ,
-						org_msg->buf , (struct auth_body*)hdr->parsed , &p);
-				}
-				break;
-			case HDR_SUPPORTED :
-				if (!HOOK_SET(supported)) {
-					new_msg->supported = new_hdr;
-				}
-				break;
-			case HDR_PROXYREQUIRE :
-				if (!HOOK_SET(proxy_require)) {
-					new_msg->proxy_require = new_hdr;
-				}
-				break;
-			case HDR_UNSUPPORTED :
-				if (!HOOK_SET(unsupported)) {
-					new_msg->unsupported = new_hdr;
-				}
-				break;
-			case HDR_ALLOW :
-				if (!HOOK_SET(allow)) {
-					new_msg->allow = new_hdr;
-				}
-				break;
-			case HDR_EVENT:
-				if (!HOOK_SET(event)) {
-					new_msg->event = new_hdr;
-				}
-				break;
-			case HDR_ACCEPT:
-				if (!HOOK_SET(accept)) {
-					new_msg->accept = new_hdr;
-				}
-				break;
-			case HDR_ACCEPTLANGUAGE:
-				if (!HOOK_SET(accept_language)) {
-					new_msg->accept_language = new_hdr;
-				}
-				break;
-			case HDR_ORGANIZATION:
-				if (!HOOK_SET(organization)) {
-					new_msg->organization = new_hdr;
-				}
-				break;
-			case HDR_PRIORITY:
-				if (!HOOK_SET(priority)) {
-					new_msg->priority = new_hdr;
-				}
-				break;
-			case HDR_SUBJECT:
-				if (!HOOK_SET(subject)) {
-					new_msg->subject = new_hdr;
-				}
-				break;
-			case HDR_USERAGENT:
-				if (!HOOK_SET(user_agent)) {
-					new_msg->user_agent = new_hdr;
-				}
-				break;
-			case HDR_ACCEPTDISPOSITION:
-				if (!HOOK_SET(accept_disposition)) {
-					new_msg->accept_disposition = new_hdr;
-				}
-				break;
-			case HDR_CONTENTDISPOSITION:
-				if (!HOOK_SET(content_disposition)) {
-					new_msg->content_disposition = new_hdr;
-				}
-			case HDR_DIVERSION:
-				if (!HOOK_SET(diversion)) {
-					new_msg->diversion = new_hdr;
-				}
-			case HDR_RPID:
-				if (!HOOK_SET(rpid)) {
-					new_msg->rpid = new_hdr;
-				}
-				break;
+				else
+					((struct to_body*)new_hdr->parsed)->last_param->next
+						= new_to_prm;
+				((struct to_body*)new_hdr->parsed)->last_param
+					= new_to_prm;
+			}
+			break;
+		case HDR_CALLID_T:
+			if (!HOOK_SET(callid)) {
+				new_msg->callid = new_hdr;
+			}
+			break;
+		case HDR_CONTACT_T:
+			if (!HOOK_SET(contact)) {
+				new_msg->contact = new_hdr;
+			}
+			break;
+		case HDR_MAXFORWARDS_T:
+			if (!HOOK_SET(maxforwards)) {
+				new_msg->maxforwards = new_hdr;
+			}
+			break;
+		case HDR_ROUTE_T:
+			if (!HOOK_SET(route)) {
+				new_msg->route = new_hdr;
+			}
+			break;
+		case HDR_RECORDROUTE_T:
+			if (!HOOK_SET(record_route)) {
+				new_msg->record_route = new_hdr;
+			}
+			break;
+		case HDR_CONTENTTYPE_T:
+			if (!HOOK_SET(content_type)) {
+				new_msg->content_type = new_hdr;
+				new_msg->content_type->parsed = hdr->parsed;
+			}
+			break;
+		case HDR_CONTENTLENGTH_T:
+			if (!HOOK_SET(content_length)) {
+				new_msg->content_length = new_hdr;
+				new_msg->content_length->parsed = hdr->parsed;
+			}
+			break;
+		case HDR_AUTHORIZATION_T:
+			if (!HOOK_SET(authorization)) {
+				new_msg->authorization = new_hdr;
+			}
+			if (hdr->parsed) {
+				new_hdr->parsed = auth_body_cloner(new_msg->buf ,
+								   org_msg->buf , (struct auth_body*)hdr->parsed , &p);
+			}
+			break;
+		case HDR_EXPIRES_T:
+			if (!HOOK_SET(expires)) {
+				new_msg->expires = new_hdr;
+			}
+			break;
+		case HDR_PROXYAUTH_T:
+			if (!HOOK_SET(proxy_auth)) {
+				new_msg->proxy_auth = new_hdr;
+			}
+			if (hdr->parsed) {
+				new_hdr->parsed = auth_body_cloner(new_msg->buf ,
+								   org_msg->buf , (struct auth_body*)hdr->parsed , &p);
+			}
+			break;
+		case HDR_SUPPORTED_T:
+			if (!HOOK_SET(supported)) {
+				new_msg->supported = new_hdr;
+			}
+			break;
+		case HDR_REQUIRE_T:
+			if (!HOOK_SET(require)) {
+				new_msg->require = new_hdr;
+			}
+			break;
+		case HDR_PROXYREQUIRE_T:
+			if (!HOOK_SET(proxy_require)) {
+				new_msg->proxy_require = new_hdr;
+			}
+			break;
+		case HDR_UNSUPPORTED_T:
+			if (!HOOK_SET(unsupported)) {
+				new_msg->unsupported = new_hdr;
+			}
+			break;
+		case HDR_ALLOW_T:
+			if (!HOOK_SET(allow)) {
+				new_msg->allow = new_hdr;
+			}
+			break;
+		case HDR_EVENT_T:
+			if (!HOOK_SET(event)) {
+				new_msg->event = new_hdr;
+			}
+			break;
+		case HDR_ACCEPT_T:
+			if (!HOOK_SET(accept)) {
+				new_msg->accept = new_hdr;
+			}
+			break;
+		case HDR_ACCEPTLANGUAGE_T:
+			if (!HOOK_SET(accept_language)) {
+				new_msg->accept_language = new_hdr;
+			}
+			break;
+		case HDR_ORGANIZATION_T:
+			if (!HOOK_SET(organization)) {
+				new_msg->organization = new_hdr;
+			}
+			break;
+		case HDR_PRIORITY_T:
+			if (!HOOK_SET(priority)) {
+				new_msg->priority = new_hdr;
+			}
+			break;
+		case HDR_SUBJECT_T:
+			if (!HOOK_SET(subject)) {
+				new_msg->subject = new_hdr;
+			}
+			break;
+		case HDR_USERAGENT_T:
+			if (!HOOK_SET(user_agent)) {
+				new_msg->user_agent = new_hdr;
+			}
+			break;
+		case HDR_SERVER_T:
+			if (!HOOK_SET(server)) {
+				new_msg->server = new_hdr;
+			}
+			break;
+		case HDR_ACCEPTDISPOSITION_T:
+			if (!HOOK_SET(accept_disposition)) {
+				new_msg->accept_disposition = new_hdr;
+			}
+			break;
+		case HDR_CONTENTDISPOSITION_T:
+			if (!HOOK_SET(content_disposition)) {
+				new_msg->content_disposition = new_hdr;
+			}
+			break;
+		case HDR_DIVERSION_T:
+			if (!HOOK_SET(diversion)) {
+				new_msg->diversion = new_hdr;
+			}
+			break;
+		case HDR_RPID_T:
+			if (!HOOK_SET(rpid)) {
+				new_msg->rpid = new_hdr;
+			}
+			break;
+		case HDR_REFER_TO_T:
+			if (!HOOK_SET(refer_to)) {
+				new_msg->refer_to = new_hdr;
+			}
+			break;
+		case HDR_SESSIONEXPIRES_T:
+			if (!HOOK_SET(session_expires)) {
+				new_msg->session_expires = new_hdr;
+			}
+			break;
+		case HDR_MIN_SE_T:
+			if (!HOOK_SET(min_se)) {
+				new_msg->min_se = new_hdr;
+			}
+			break;
+		case HDR_SUBSCRIPTION_STATE_T:
+			if (!HOOK_SET(subscription_state)) {
+				new_msg->subscription_state = new_hdr;
+			}
+			break;
+		case HDR_SIPIFMATCH_T:
+			if (!HOOK_SET(sipifmatch)) {
+				new_msg->sipifmatch = new_hdr;
+			}
+			break;
 		}/*switch*/
 
 		if ( last_hdr )
@@ -794,8 +862,3 @@ do { \
 
 	return new_msg;
 }
-
-
-
-
-
