@@ -1,5 +1,5 @@
 /*
- * $Id: t_fifo.c,v 1.15.2.7 2005/08/29 14:27:15 rco Exp $
+ * $Id: t_fifo.c,v 1.29.2.1 2008/02/11 16:02:46 janakj Exp $
  *
  * transaction maintenance functions
  *
@@ -82,7 +82,7 @@
 
 
 
-int tm_unix_tx_timeout = 2; /* Default is 2 seconds */
+int tm_unix_tx_timeout = 500; /* Default is 500 ms */
 
 #define TWRITE_PARAMS          20
 #define TWRITE_VERSION_S       "0.3"
@@ -198,6 +198,7 @@ int parse_tw_append( modparam_t type, void* val)
 	char bar;
 	str foo;
 	int n;
+	int index;
 	
 	if (val==0 || ((char*)val)[0]==0)
 		return 0;
@@ -314,20 +315,20 @@ int parse_tw_append( modparam_t type, void* val)
 		/* process and optimize the element name */
 		if (ha->type==ELEM_IS_AVP) {
 			/* element is AVP */
-			if ( parse_avp_spec( &foo, &n, &avp_name)!=0 ) {
+			if ( parse_avp_spec( &foo, &n, &avp_name, &index)!=0 ) {
 				LOG(L_ERR,"ERROR:tm:parse_tw_append: bad alias spec "
 					"<%.*s>\n",foo.len, foo.s);
 				goto error;
 			}
 			if (n&AVP_NAME_STR) {
 				/* string name */
-				ha->sval.s = (char*)pkg_malloc(avp_name.s->len+1);
+				ha->sval.s = (char*)pkg_malloc(avp_name.s.len+1);
 				if (ha->sval.s==0) {
 					LOG(L_ERR,"ERROR:tm:parse_tw_append: no more pkg mem\n");
 					goto error;
 				}
-				memcpy( ha->sval.s, avp_name.s->s, avp_name.s->len);
-				ha->sval.len = avp_name.s->len;
+				memcpy( ha->sval.s, avp_name.s.s, avp_name.s.len);
+				ha->sval.len = avp_name.s.len;
 				ha->sval.s[ha->sval.len] = 0;
 				if (ha->title.s==0)
 					ha->title = ha->sval;
@@ -359,7 +360,7 @@ int parse_tw_append( modparam_t type, void* val)
 			}
 			foo.s[foo.len] = bar;
 			ha->ival = hdr.type;
-			if (hdr.type==HDR_OTHER || ha->title.s==0) {
+			if (hdr.type==HDR_OTHER_T || ha->title.s==0) {
 				/* duplicate hdr name */
 				ha->sval.s = (char*)pkg_malloc(foo.len+1);
 				if (ha->sval.s==0) {
@@ -597,18 +598,18 @@ static inline char* append2buf( char *buf, int len, struct sip_msg *req,
 		if (ha->type==ELEM_IS_AVP) {
 			/* search for the AVP */
 			if (ha->sval.s) {
-				avp_name.s=&ha->sval;
-				avp = search_first_avp( AVP_NAME_STR, avp_name, &avp_val);
-				DBG("AVP <%.*s>: %p\n",avp_name.s->len,avp_name.s->s, avp);
+				avp_name.s=ha->sval;
+				avp = search_first_avp( AVP_NAME_STR, avp_name, &avp_val, 0);
+				DBG("AVP <%.*s>: %p\n",avp_name.s.len, avp_name.s.s, avp);
 			} else {
 				avp_name.n=ha->ival;
-				avp = search_first_avp( 0, avp_name, &avp_val);
-				DBG("AVP <%i>: %p\n",avp_name.n, avp);
+				avp = search_first_avp( 0, avp_name, &avp_val, 0);
+				DBG("AVP <%i>: %p\n",avp_name.n,avp);
 			}
 			if (avp) {
 				if (avp->flags&AVP_VAL_STR) {
 					buf=add2buf( buf, end, ha->title.s, ha->title.len,
-						avp_val.s->s , avp_val.s->len);
+						avp_val.s.s , avp_val.s.len);
 					if (!buf)
 						goto overflow_err;
 				} else {
@@ -622,14 +623,14 @@ static inline char* append2buf( char *buf, int len, struct sip_msg *req,
 		} else if (ha->type==ELEM_IS_HDR) {
 			/* parse the HDRs */
 			if (!msg_parsed) {
-				if (parse_headers( req, HDR_EOH, 0)!=0) {
+				if (parse_headers( req, HDR_EOH_F, 0)!=0) {
 					LOG(L_ERR,"ERROR:tm:append2buf: parsing hdrs failed\n");
 					goto error;
 				}
 				msg_parsed = 1;
 			}
 			/* search the HDR */
-			if (ha->ival==HDR_OTHER) {
+			if (ha->ival==HDR_OTHER_T) {
 				for(hdr=req->headers;hdr;hdr=hdr->next)
 					if (ha->sval.len==hdr->name.len &&
 					strncasecmp( ha->sval.s, hdr->name.s, hdr->name.len)==0)
@@ -669,7 +670,7 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 	static char     route_buffer[ROUTE_BUFFER_MAX];
 	static char     append_buf[APPEND_BUFFER_MAX];
 	static char     cmd_buf[CMD_BUFFER_MAX];
-	static str      empty_param = {".",1};
+	static str      empty_param = STR_STATIC_INIT(".");
 	unsigned int      hash_index, label;
 	contact_body_t*   cb=0;
 	contact_t*        c=0;
@@ -679,6 +680,7 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 	param_hooks_t     hooks;
 	int               l;
 	char*             s, fproxy_lr;
+	int               first_rr;
 	str               route, next_hop, append, tmp_s, body, str_uri;
 
 	if(msg->first_line.type != SIP_REQUEST){
@@ -688,7 +690,7 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 	}
 
 	/* parse all -- we will need every header field for a UAS */
-	if ( parse_headers(msg, HDR_EOH, 0)==-1) {
+	if ( parse_headers(msg, HDR_EOH_F, 0)==-1) {
 		LOG(L_ERR,"assemble_msg: parse_headers failed\n");
 		goto error;
 	}
@@ -754,6 +756,8 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 		record_route = 0;
 	}
 
+	first_rr = 1;
+
 	if( record_route ) {
 		if ( (tmp_s.s=find_not_quoted(&record_route->nameaddr.uri,';'))!=0 &&
 		tmp_s.s+1!=record_route->nameaddr.uri.s+
@@ -771,14 +775,15 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 			DBG("assemble_msg: record_route->nameaddr.uri: %.*s\n",
 				record_route->nameaddr.uri.len,record_route->nameaddr.uri.s);
 			if(fproxy_lr){
+				first_rr = 0;
 				DBG("assemble_msg: first proxy has loose routing.\n");
 				copy_route(s,route.len,record_route->nameaddr.uri.s,
 					record_route->nameaddr.uri.len);
 			}
 		}
-		for(p_hdr = p_hdr->next;p_hdr;p_hdr = p_hdr->next) {
+		for(;p_hdr;p_hdr = p_hdr->next) {
 			/* filter out non-RR hdr and empty hdrs */
-			if( (p_hdr->type!=HDR_RECORDROUTE) || p_hdr->body.len==0)
+			if( (p_hdr->type!=HDR_RECORDROUTE_T) || p_hdr->body.len==0)
 				continue;
 
 			if(p_hdr->parsed==0 && parse_rr(p_hdr)!=0 ){
@@ -791,8 +796,11 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 				DBG("assemble_msg: record_route->nameaddr.uri: "
 				    "<%.*s>\n", record_route->nameaddr.uri.len,
 					record_route->nameaddr.uri.s);
-				copy_route(s,route.len,record_route->nameaddr.uri.s,
-					record_route->nameaddr.uri.len);
+				if (!first_rr) { 
+ 				  copy_route(s,route.len,record_route->nameaddr.uri.s,
+ 					     record_route->nameaddr.uri.len);
+ 				} 
+ 				first_rr=0;
 			}
 		}
 
@@ -810,6 +818,7 @@ static int assemble_msg(struct sip_msg* msg, struct tw_info *twi)
 	    str_uri.len,str_uri.len ? str_uri.s : "");
 	
 	if ( REQ_LINE(msg).method_value==METHOD_INVITE || 
+	     REQ_LINE(msg).method_value==METHOD_INFO ||
 			(twi->append && twi->append->add_body) ) {
 		/* get body */
 		if( (body.s = get_body(msg)) == 0 ){
@@ -951,7 +960,7 @@ static int write_to_unixsock(char* sockname, int cnt)
 		return -1;
 	}
 
-	if (tsend_dgram_ev(sock, iov_lines_eol, 2 * cnt, tm_unix_tx_timeout * 1000) < 0) {
+	if (tsend_dgram_ev(sock, iov_lines_eol, 2 * cnt, tm_unix_tx_timeout) < 0) {
 		LOG(L_ERR, "write_to_unixsock: writev failed: %s\n", strerror(errno));
 		return -1;
 	}
