@@ -1,5 +1,5 @@
 /*
- * $Id: t_lookup.c,v 1.91.2.5 2005/11/29 19:39:45 andrei Exp $
+ * $Id: t_lookup.c,v 1.108.2.1 2008/03/31 16:30:51 tirpi Exp $
  *
  * This C-file takes care of matching requests and replies with
  * existing transactions. Note that we do not do SIP-compliant
@@ -79,6 +79,17 @@
  * 2004-10-10: use of mhomed disabled for replies (jiri)
  * 2005-02-01: use the incoming request interface for sending the replies
  *             - changes in init_rb() (bogdan)
+ * 2005-12-09  added t_set_fr()  (andrei)
+ * 2006-01-27  transaction lookup function will set up a cancel flag
+ *             if the searched transaction was pre-canceled (andrei)
+ * 2006-10-16  401 & 407 replies are completely parsed if tm_aggregate_auth is
+ *              set (andrei)
+ * 2006-11-10  a valid msg->hash_index is now marked by FL_HASH_INDEX in 
+ *              msg_flags
+ *             t_lookupOriginalT computes the hash_index by itself  if 
+ *              needed (andrei)
+ * 2008-03-31  message flags are updated in shared memory even if they are set
+ *             after t_newtran() (Miklos)
  */
 
 #include "defs.h"
@@ -91,6 +102,7 @@
 #include "../../parser/parse_from.h"
 #include "../../ut.h"
 #include "../../timer.h"
+#include "../../timer_ticks.h"
 #include "../../hash_func.h"
 #include "../../globals.h"
 #include "../../forward.h"
@@ -98,6 +110,7 @@
 #include "config.h"
 #include "sip_msg.h"
 #include "t_hooks.h"
+#include "t_fwd.h"
 #include "t_lookup.h"
 #include "dlg.h" /* for t_lookup_callid */
 #include "t_msgbuilder.h" /* for t_lookup_callid */
@@ -164,7 +177,7 @@ void init_t() {global_msg_id=0; set_t(T_UNDEFINED);}
 
 static inline int parse_dlg( struct sip_msg *msg )
 {
-	if (parse_headers(msg, HDR_FROM | HDR_CSEQ | HDR_TO, 0)==-1) {
+	if (parse_headers(msg, HDR_FROM_F | HDR_CSEQ_F | HDR_TO_F, 0)==-1) {
 		LOG(L_ERR, "ERROR: parse_dlg: From or Cseq or To invalid\n");
 		return 0;
 	}
@@ -236,7 +249,7 @@ static inline int ack_matching(struct cell *p_cell, struct sip_msg *p_msg)
 	 * done now -- we ignore to-tags; the ACK simply belongs to
 	 * this UAS part of dialog, whatever to-tag it gained
 	 */
-	if (p_cell->relaied_reply_branch!=-2) {
+	if (p_cell->relayed_reply_branch!=-2) {
 		return 2; /* e2e proxied ACK */
 	}
 	/* it's a local dialog -- we wish to verify to-tags too */
@@ -282,10 +295,11 @@ static inline int via_matching( struct via_body *inv_via,
 	 2 if e2e ACK for a proxied transaction found
      1  if found (covers ACK for local UAS)
 	 0  if not found (trans undefined)
+	It also sets *cancel if a cancel was found for the searched transaction
 */
 
 static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
-			enum request_method skip_method)
+			enum request_method skip_method, int* cancel)
 {
 	struct cell *p_cell;
 	struct sip_msg  *t_msg;
@@ -295,6 +309,7 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 	int ret = 0;
 	struct cell *e2e_ack_trans;
 
+	*cancel=0;
 	e2e_ack_trans=0;
 	via1=p_msg->via1;
 	is_ack=p_msg->REQ_METHOD==METHOD_ACK;
@@ -308,20 +323,25 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 	{
 		t_msg=p_cell->uas.request;
 		if (!t_msg) continue;  /* don't try matching UAC transactions */
-		if (skip_method & t_msg->REQ_METHOD) continue;
+		/* we want to set *cancel for transaction for which there is
+		 * already a canceled transaction (e.g. re-ordered INV-CANCEL, or
+		 *  INV blocked in dns lookup); we don't care about ACKs */
+		if ((is_ack || (t_msg->REQ_METHOD!=METHOD_CANCEL)) && 
+				(skip_method & t_msg->REQ_METHOD)) 
+			continue;
 
-		     /* here we do an exercise which will be removed from future code
-		      *	versions: we try to match end-2-end ACKs if they appear at our
-		      * server. This allows some applications bound to TM via callbacks
-		      * to correlate the e2e ACKs with transaction context, e.g., for
-		      * purpose of accounting. We think it is a bad place here, among
-		      * other things because it is not reliable. If a transaction loops
-		      * via SER the ACK can't be matched to proper INVITE transaction
-		      * (it is a separate transactino with its own branch ID) and it
-		      * matches all transaction instances in the loop dialog-wise.
-		      * Eventually, regardless to which transaction in the loop the
-		      * ACK belongs, only the first one will match.
-		      */
+		/* here we do an exercise which will be removed from future code
+		   versions: we try to match end-2-end ACKs if they appear at our
+		   server. This allows some applications bound to TM via callbacks
+		   to correlate the e2e ACKs with transaction context, e.g., for
+		   purpose of accounting. We think it is a bad place here, among
+		   other things because it is not reliable. If a transaction loops
+		   via SER the ACK can't be matched to proper INVITE transaction
+		   (it is a separate transactino with its own branch ID) and it
+		   matches all transaction instances in the loop dialog-wise.
+		   Eventually, regardless to which transaction in the loop the
+		   ACK belongs, only the first one will match.
+		*/
 
 		/* dialog matching needs to be applied for ACK/200s */
 		if (is_ack && p_cell->uas.status<300 && e2e_ack_trans==0) {
@@ -348,6 +368,13 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
 	 	 * other requests */
 		if (!via_matching(t_msg->via1 /* inv via */, via1 /* ack */ ))
 			continue;
+		if (t_msg->REQ_METHOD==METHOD_CANCEL){
+			if ((p_msg->REQ_METHOD!=METHOD_CANCEL) && !is_ack){
+			/* found an existing cancel for the searched transaction */
+				*cancel=1;
+			}
+			if (skip_method & t_msg->REQ_METHOD) continue;
+		}
 		/* all matched -- we found the transaction ! */
 		DBG("DEBUG: RFC3261 transaction matched, tid=%.*s\n",
 			via1->tid.len, via1->tid.s);
@@ -371,9 +398,11 @@ static int matching_3261( struct sip_msg *p_msg, struct cell **trans,
  *      negative - transaction wasn't found
  *			(-2 = possibly e2e ACK matched )
  *      positive - transaction found
+ * It also sets *cancel if a there is already a cancel transaction
  */
 
-int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
+int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked,
+						int* cancel)
 {
 	struct cell         *p_cell;
 	unsigned int       isACK;
@@ -393,8 +422,10 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 	}
 
 	/* start searching into the table */
-	if (!p_msg->hash_index)
-		p_msg->hash_index=hash( p_msg->callid->body , get_cseq(p_msg)->number ) ;
+	if (!(p_msg->msg_flags & FL_HASH_INDEX)){
+		p_msg->hash_index=hash( p_msg->callid->body , get_cseq(p_msg)->number);
+		p_msg->msg_flags|=FL_HASH_INDEX;
+	}
 	isACK = p_msg->REQ_METHOD==METHOD_ACK;
 	DBG("t_lookup_request: start searching: hash=%d, isACK=%d\n",
 		p_msg->hash_index,isACK);
@@ -402,7 +433,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 
 	/* assume not found */
 	ret=-1;
-	e2e_ack_trans=0;
+	e2e_ack_trans = 0;
 
 	/* first of all, look if there is RFC3261 magic cookie in branch; if
 	 * so, we can do very quick matching and skip the old-RFC bizzar
@@ -419,9 +450,10 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 		/* huhuhu! the cookie is there -- let's proceed fast */
 		LOCK_HASH(p_msg->hash_index);
 		match_status=matching_3261(p_msg,&p_cell, 
-				/* skip transactions with different method; otherwise CANCEL would 
-	 	 		 * match the previous INVITE trans.  */
-				isACK ? ~METHOD_INVITE: ~p_msg->REQ_METHOD);
+				/* skip transactions with different method; otherwise CANCEL 
+				 * would  match the previous INVITE trans.  */
+				isACK ? ~METHOD_INVITE: ~p_msg->REQ_METHOD, 
+				cancel);
 		switch(match_status) {
 				case 0:	goto notfound;	/* no match */
 				case 1:	goto found; 	/* match */
@@ -433,7 +465,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 	 * a bit simplified to be fast -- we don't do all the comparisons
 	 * of parsed uri, which was simply too bloated */
 	DBG("DEBUG: proceeding to pre-RFC3261 transaction matching\n");
-
+	*cancel=0;
 	/* lock the whole entry*/
 	LOCK_HASH(p_msg->hash_index);
 
@@ -446,9 +478,17 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 		if (!t_msg) continue; /* skip UAC transactions */
 
 		if (!isACK) {	
+			/* for non-ACKs we want same method matching, we 
+			 * make an exception for pre-exisiting CANCELs because we
+			 * want to set *cancel */
+			if ((t_msg->REQ_METHOD!=p_msg->REQ_METHOD) &&
+					(t_msg->REQ_METHOD!=METHOD_CANCEL))
+					continue;
 			/* compare lengths first */ 
 			if (!EQ_LEN(callid)) continue;
-			if (!EQ_LEN(cseq)) continue;
+			/* CSeq only the number without method ! */
+			if (get_cseq(t_msg)->number.len!=get_cseq(p_msg)->number.len)
+				continue;
 			if (!EQ_LEN(from)) continue;
 			if (!EQ_LEN(to)) continue;
 			if (ruri_matching && !EQ_REQ_URI_LEN) continue;
@@ -456,12 +496,20 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 
 			/* length ok -- move on */
 			if (!EQ_STR(callid)) continue;
-			if (!EQ_STR(cseq)) continue;
+			if (memcmp(get_cseq(t_msg)->number.s, get_cseq(p_msg)->number.s,
+				get_cseq(p_msg)->number.len)!=0) continue;
 			if (!EQ_STR(from)) continue;
 			if (!EQ_STR(to)) continue;
 			if (ruri_matching && !EQ_REQ_URI_STR) continue;
 			if (via1_matching && !EQ_VIA_STR(via1)) continue;
-
+			
+			if ((t_msg->REQ_METHOD==METHOD_CANCEL) &&
+				(p_msg->REQ_METHOD!=METHOD_CANCEL)){
+				/* we've matched an existing CANCEL */
+				*cancel=1;
+				continue;
+			}
+			
 			/* request matched ! */
 			DBG("DEBUG: non-ACK matched\n");
 			goto found;
@@ -489,7 +537,7 @@ int t_lookup_request( struct sip_msg* p_msg , int leave_new_locked )
 			/* it is e2e ACK/200 */
 			if (p_cell->uas.status<300 && e2e_ack_trans==0) {
 				/* all criteria for proxied ACK are ok */
-				if (p_cell->relaied_reply_branch!=-2) {
+				if (p_cell->relayed_reply_branch!=-2) {
 					e2e_ack_trans=p_cell;
 					continue;
 				}
@@ -561,10 +609,23 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 	unsigned int     hash_index;
 	struct sip_msg  *t_msg;
 	struct via_param *branch;
+	int foo;
 	int ret;
 
 
 	/* start searching in the table */
+	if (!(p_msg->msg_flags & FL_HASH_INDEX)){
+		/* parse all*/
+		if (check_transaction_quadruple(p_msg)==0)
+		{
+			LOG(L_ERR, "ERROR: TM module: t_lookupOriginalT:"
+					" too few headers\n");
+			/* stop processing */
+			return 0;
+		}
+		p_msg->hash_index=hash( p_msg->callid->body , get_cseq(p_msg)->number);
+		p_msg->msg_flags|=FL_HASH_INDEX;
+	}
 	hash_index = p_msg->hash_index;
 	DBG("DEBUG: t_lookupOriginalT: searching on hash entry %d\n",hash_index );
 
@@ -575,7 +636,6 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 	 */
 	if (!p_msg->via1) {
 		LOG(L_ERR, "ERROR: t_lookupOriginalT: no via\n");
-		set_t(0);
 		return 0;
 	}
 	branch=p_msg->via1->branch;
@@ -587,7 +647,7 @@ struct cell* t_lookupOriginalT(  struct sip_msg* p_msg )
 				/* we are seeking the original transaction --
 				 * skip CANCEL transactions during search
 				 */
-				METHOD_CANCEL);
+				METHOD_CANCEL, &foo);
 		if (ret==1) goto found; else goto notfound;
 	}
 
@@ -832,7 +892,7 @@ int t_reply_matching( struct sip_msg *p_msg , int *p_branch )
 				has_tran_tmcbs(p_cell,TMCB_RESPONSE_OUT|TMCB_E2EACK_IN) )
 			|| (is_local(p_cell)&&has_tran_tmcbs(p_cell,TMCB_LOCAL_COMPLETED))
 		)) {
-			if (parse_headers(p_msg, HDR_TO, 0)==-1) {
+			if (parse_headers(p_msg, HDR_TO_F, 0)==-1) {
 				LOG(L_ERR, "ERROR: t_reply_matching: to parsing failed\n");
 			}
 		}
@@ -866,6 +926,7 @@ nomatch2:
 int t_check( struct sip_msg* p_msg , int *param_branch )
 {
 	int local_branch;
+	int canceled;
 
 	/* is T still up-to-date ? */
 	DBG("DEBUG: t_check: msg id=%d global id=%d T start=%p\n", 
@@ -877,7 +938,7 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 		/* transaction lookup */
 		if ( p_msg->first_line.type==SIP_REQUEST ) {
 			/* force parsing all the needed headers*/
-			if (parse_headers(p_msg, HDR_EOH, 0 )==-1) {
+			if (parse_headers(p_msg, HDR_EOH_F, 0 )==-1) {
 				LOG(L_ERR, "ERROR: t_check: parsing error\n");
 				return -1;
 			}
@@ -891,23 +952,35 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 				LOG(L_ERR, "ERROR: t_check: from parsing failed\n");
 				return -1;
 			}
-			t_lookup_request( p_msg , 0 /* unlock before returning */ );
+			t_lookup_request( p_msg , 0 /* unlock before returning */,
+								&canceled);
 		} else {
 			/* we need Via for branch and Cseq method to distinguish
 			   replies with the same branch/cseqNr (CANCEL)
+			   and we need all the WWW/Proxy Authenticate headers for
+			   401 & 407 replies
 			*/
-			if ( parse_headers(p_msg, HDR_VIA1|HDR_CSEQ, 0 )==-1
-			|| !p_msg->via1 || !p_msg->cseq ) {
+			if (tm_aggregate_auth && 
+					(p_msg->REPLY_STATUS==401 || p_msg->REPLY_STATUS==407)){
+				if (parse_headers(p_msg, HDR_EOH_F,0)==-1){
+					LOG(L_WARN, "WARNING: the reply cannot be "
+								"completely parsed\n");
+					/* try to continue, via1 & cseq are checked below */
+				}
+			}else if ( parse_headers(p_msg, HDR_VIA1_F|HDR_CSEQ_F, 0 )==-1) {
 				LOG(L_ERR, "ERROR: reply cannot be parsed\n");
 				return -1;
 			}
-
+			if ((p_msg->via1==0) || (p_msg->cseq==0)){
+				LOG(L_ERR, "ERROR: reply doesn't have a via or cseq header\n");
+				return -1;
+			}
 			/* if that is an INVITE, we will also need to-tag
 			   for later ACK matching
 			*/
             if ( get_cseq(p_msg)->method.len==INVITE_LEN 
 				&& memcmp( get_cseq(p_msg)->method.s, INVITE, INVITE_LEN )==0 ) {
-					if (parse_headers(p_msg, HDR_TO, 0)==-1
+					if (parse_headers(p_msg, HDR_TO_F, 0)==-1
 						|| !p_msg->to)  {
 						LOG(L_ERR, "ERROR: INVITE reply cannot be parsed\n");
 						return -1;
@@ -919,9 +992,9 @@ int t_check( struct sip_msg* p_msg , int *param_branch )
 
 		}
 #ifdef EXTRA_DEBUG
-		if ( T && T!=T_UNDEFINED && T->damocles) {
+		if ( T && T!=T_UNDEFINED && T->flags & (T_IN_AGONY)) {
 			LOG( L_ERR, "ERROR: transaction %p scheduled for deletion "
-				"and called from t_check\n", T);
+				"and called from t_check (flags=%x)\n", T, T->flags);
 			abort();
 		}
 #endif
@@ -944,7 +1017,9 @@ int init_rb( struct retr_buf *rb, struct sip_msg *msg)
 	int proto;
 	int backup_mhomed;
 
+	/* rb. timers are init. init_t()/new_cell() */
 	via=msg->via1;
+	/* rb->dst is already init (0) by new_t()/build_cell() */
 	if (!reply_to_via) {
 		update_sock_struct_from_ip( &rb->dst.to, msg );
 		proto=msg->rcv.proto;
@@ -959,7 +1034,10 @@ int init_rb( struct retr_buf *rb, struct sip_msg *msg)
 		proto=via->proto;
 	}
 	rb->dst.proto=proto;
-	rb->dst.proto_reserved1=msg->rcv.proto_reserved1;
+	rb->dst.id=msg->rcv.proto_reserved1;
+#ifdef USE_COMP
+	rb->dst.comp=via->comp_no;
+#endif
 	/* turn off mhomed for generating replies -- they are ideally sent to where
 	   request came from to make life with NATs and other beasts easier
 	*/
@@ -982,6 +1060,7 @@ int init_rb( struct retr_buf *rb, struct sip_msg *msg)
 static inline void init_new_t(struct cell *new_cell, struct sip_msg *p_msg)
 {
 	struct sip_msg *shm_msg;
+	unsigned int timeout; /* avp timeout gets stored here (in s) */
 
 	shm_msg=new_cell->uas.request;
 	new_cell->from.s=shm_msg->from->name.s;
@@ -999,6 +1078,28 @@ static inline void init_new_t(struct cell *new_cell, struct sip_msg *p_msg)
 	if (p_msg->REQ_METHOD==METHOD_INVITE) new_cell->flags |= T_IS_INVITE_FLAG;
 	new_cell->on_negative=get_on_negative();
 	new_cell->on_reply=get_on_reply();
+	new_cell->fr_timeout=(ticks_t)get_msgid_val(user_fr_timeout,
+												p_msg->id, int);
+	new_cell->fr_inv_timeout=(ticks_t)get_msgid_val(user_fr_inv_timeout,
+												p_msg->id, int);
+	if (new_cell->fr_timeout==0){
+		if (!fr_avp2timer(&timeout)) {
+			DBG("init_new_t: FR__TIMER = %d s\n", timeout);
+			new_cell->fr_timeout=S_TO_TICKS((ticks_t)timeout);
+		}else{
+			new_cell->fr_timeout=fr_timeout;
+		}
+	}
+	if (new_cell->fr_inv_timeout==0){
+		if (!fr_inv_avp2timer(&timeout)) {
+			DBG("init_new_t: FR_INV_TIMER = %d s\n", timeout);
+			new_cell->fr_inv_timeout=S_TO_TICKS((ticks_t)timeout);
+			new_cell->flags |= T_NOISY_CTIMER_FLAG;
+		}else{
+			new_cell->fr_inv_timeout=fr_inv_timeout;
+		}
+	}
+	new_cell->on_branch=get_on_branch();
 }
 
 static inline int new_t(struct sip_msg *p_msg)
@@ -1049,6 +1150,7 @@ static inline int new_t(struct sip_msg *p_msg)
 int t_newtran( struct sip_msg* p_msg )
 {
 	int lret, my_err;
+	int canceled;
 
 
 	/* is T still up-to-date ? */
@@ -1056,8 +1158,16 @@ int t_newtran( struct sip_msg* p_msg )
 		" T on entrance=%p\n",p_msg->id,global_msg_id,T);
 
 	if ( T && T!=T_UNDEFINED  ) {
-		LOG(L_ERR, "ERROR: t_newtran: "
+		/* ERROR message moved to w_t_newtran */
+		DBG("DEBUG: t_newtran: "
 			"transaction already in process %p\n", T );
+
+		/* t_newtran() has been already called, and the script
+		might changed the flags after it, so we must update the flags
+		in shm memory -- Miklos */
+		if (T->uas.request)
+			T->uas.request->flags = p_msg->flags;
+
 		return E_SCRIPT;
 	}
 
@@ -1070,11 +1180,11 @@ int t_newtran( struct sip_msg* p_msg )
 	   shmem with pkg_mem
 	*/
 	
-	if (parse_headers(p_msg, HDR_EOH, 0 )) {
+	if (parse_headers(p_msg, HDR_EOH_F, 0 )) {
 		LOG(L_ERR, "ERROR: t_newtran: parse_headers failed\n");
 		return E_BAD_REQ;
 	}
-	if ((p_msg->parsed_flag & HDR_EOH)!=HDR_EOH) {
+	if ((p_msg->parsed_flag & HDR_EOH_F)!=HDR_EOH_F) {
 			LOG(L_ERR, "ERROR: t_newtran: EoH not parsed\n");
 			return E_OUT_OF_MEM;
 	}
@@ -1082,7 +1192,8 @@ int t_newtran( struct sip_msg* p_msg )
 	   it also calls check_transaction_quadruple -> it is
 	   safe to assume we have from/callid/cseq/to
 	*/ 
-	lret = t_lookup_request( p_msg, 1 /* leave locked if not found */ );
+	lret = t_lookup_request( p_msg, 1 /* leave locked if not found */,
+								&canceled );
 
 	/* on error, pass the error in the stack ... nothing is locked yet
 	   if 0 is returned */
@@ -1135,6 +1246,7 @@ int t_newtran( struct sip_msg* p_msg )
 		LOG(L_ERR, "ERROR: t_newtran: new_t failed\n");
 		goto new_err;
 	}
+	if (canceled) T->flags|=T_CANCELED; /* mark it for future ref. */
 
 
 	UNLOCK_HASH(p_msg->hash_index);
@@ -1320,3 +1432,36 @@ int t_lookup_callid(struct cell ** trans, str callid, str cseq) {
 	return -1;
 }
 
+
+
+/* params: fr_inv & fr value in ms, 0 means "do not touch"
+ * ret: 1 on success, -1 on error (script safe)*/
+int t_set_fr(struct sip_msg* msg, unsigned int fr_inv_to, unsigned int fr_to)
+{
+	struct cell *t;
+	ticks_t fr_inv, fr;
+	
+	
+	fr_inv=MS_TO_TICKS((ticks_t)fr_inv_to);
+	if ((fr_inv==0) && (fr_inv_to!=0)){
+		ERR("t_set_fr_inv: fr_inv_timeout too small (%d)\n", fr_inv_to);
+		return -1;
+	}
+	fr=MS_TO_TICKS((ticks_t)fr_to);
+	if ((fr==0) && (fr_to!=0)){
+		ERR("t_set_fr_inv: fr_timeout too small (%d)\n", fr_to);
+		return -1;
+	}
+	
+	t=get_t();
+	/* in MODE_REPLY and MODE_ONFAILURE T will be set to current transaction;
+	 * in MODE_REQUEST T will be set only if the transaction was already
+	 * created; if not -> use the static variable */
+	if (!t || t==T_UNDEFINED ){
+		set_msgid_val(user_fr_inv_timeout, msg->id, int, (int)fr_inv);
+		set_msgid_val(user_fr_timeout, msg->id, int, (int)fr);
+	}else{
+		change_fr(t, fr_inv, fr); /* change running uac timers */
+	}
+	return 1;
+}
