@@ -1,5 +1,5 @@
 /*
- * $Id: t_msgbuilder.c,v 1.40 2004/11/09 15:15:12 andrei Exp $
+ * $Id: t_msgbuilder.c,v 1.44.2.2 2007/10/02 20:47:55 andrei Exp $
  *
  * message printing
  *
@@ -38,6 +38,10 @@
  * 2003-10-02  added via_builder set host/port support (andrei)
  * 2004-02-11  FIFO/CANCEL + alignments (hash=f(callid,cseq)) (uli+jiri)
  * 2004-02-13: t->is_invite and t->local replaced with flags (bogdan)
+ * 2006-04-21  build_uac_req, assemble_via use struct dest_info now;
+ *              uri2sock replaced with uri2dst (andrei)
+ * 2006-08-11  build_dlg_ack: use the first dns ip for which a send_sock
+ *              is found (andrei)
  */
 
 #include "defs.h"
@@ -55,6 +59,9 @@
 #include "../../parser/contact/parse_contact.h"
 #include "t_msgbuilder.h"
 #include "uac.h"
+#ifdef USE_DNS_FAILOVER
+#include "../../dns_cache.h"
+#endif
 
 
 #define ROUTE_PREFIX "Route: "
@@ -87,16 +94,12 @@ char *build_local(struct cell *Trans,unsigned int branch,
 	char branch_buf[MAX_BRANCH_PARAM_LEN];
 	int branch_len;
 	str branch_str;
+	str via_id;
 	struct hostport hp;
 
-#ifdef _OBSO
-	if ( Trans->uac[branch].last_received<100)
-	{
-		DBG("DEBUG: build_local: no response ever received"
-			" : dropping local request! \n");
-		goto error;
-	}
-#endif
+	/* init */
+	via_id.s=0;
+	via_id.len=0;
 
 	/* method, separators, version: "CANCEL sip:p2@iptel.org SIP/2.0" */
 	*len=SIP_VERSION_LEN + method_len + 2 /* spaces */ + CRLF_LEN;
@@ -109,8 +112,29 @@ char *build_local(struct cell *Trans,unsigned int branch,
 	branch_str.s=branch_buf;
 	branch_str.len=branch_len;
 	set_hostport(&hp, (is_local(Trans))?0:(Trans->uas.request));
-	via=via_builder(&via_len, Trans->uac[branch].request.dst.send_sock,
-		&branch_str, 0, Trans->uac[branch].request.dst.proto, &hp );
+#ifdef USE_TCP
+	if (!is_local(Trans) && ((Trans->uas.request->rcv.proto==PROTO_TCP)
+#ifdef USE_TLS
+				|| (Trans->uas.request->rcv.proto==PROTO_TLS)
+#endif /* USE_TLS */
+		)){
+		if ((via_id.s=id_builder(Trans->uas.request,
+									(unsigned int*)&via_id.len))==0){
+			LOG(L_ERR, "ERROR: build_local: id builder failed\n");
+			/* try to continue without id */
+		}
+	}
+#endif /* USE_TCP */
+	via=via_builder(&via_len, &Trans->uac[branch].request.dst,
+		&branch_str, via_id.s?&via_id:0 , &hp );
+	
+	/* via_id.s not needed anylonger => free it */
+	if (via_id.s){
+		pkg_free(via_id.s);
+		via_id.s=0;
+		via_id.len=0;
+	}
+	
 	if (!via)
 	{
 		LOG(L_ERR, "ERROR: build_local: "
@@ -126,7 +150,7 @@ char *build_local(struct cell *Trans,unsigned int branch,
 	/* copy'n'paste Route headers */
 	if (!is_local(Trans)) {
 		for ( hdr=Trans->uas.request->headers ; hdr ; hdr=hdr->next )
-			 if (hdr->type==HDR_ROUTE)
+			 if (hdr->type==HDR_ROUTE_T)
 				*len+=hdr->len;
 	}
 
@@ -165,7 +189,7 @@ char *build_local(struct cell *Trans,unsigned int branch,
 
 	if (!is_local(Trans))  {
 		for ( hdr=Trans->uas.request->headers ; hdr ; hdr=hdr->next )
-			if(hdr->type==HDR_ROUTE) {
+			if(hdr->type==HDR_ROUTE_T) {
 				append_mem_block(p, hdr->name.s, hdr->len );
 			}
 	}
@@ -216,7 +240,7 @@ static inline int process_routeset(struct sip_msg* msg, str* contact, struct rte
 	ptr = msg->record_route;
 	head = 0;
 	while(ptr) {
-		if (ptr->type == HDR_RECORDROUTE) {
+		if (ptr->type == HDR_RECORDROUTE_T) {
 			if (parse_rr(ptr) < 0) {
 				LOG(L_ERR, "process_routeset: Error while parsing Record-Route header\n");
 				return -1;
@@ -364,9 +388,9 @@ static inline int get_contact_uri(struct sip_msg* msg, str* uri)
 
      /*
       * The function creates an ACK to 200 OK. Route set will be created
-      * and parsed and next_hop parameter will contain uri the which the
+      * and parsed and next_hop parameter will contain the uri to which the
       * request should be send. The function is used by tm when it generates
-      * local ACK to 200 OK (on behalf of applications using uac
+      * local ACK to 200 OK (on behalf of applications using uac)
       */
 char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans, unsigned int branch,
 		    str* to, unsigned int *len, str *next_hop)
@@ -379,8 +403,10 @@ char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans, unsigned int branch
 	struct hostport hp;
 	struct rte* list;
 	str contact, ruri, *cont;
-	struct socket_info* send_sock;
-	union sockaddr_union to_su;
+	struct dest_info dst;
+#ifdef USE_DNS_FAILOVER
+	struct dns_srv_handle dns_h;
+#endif
 	
 	if (get_contact_uri(rpl, &contact) < 0) {
 		return 0;
@@ -407,18 +433,36 @@ char *build_dlg_ack(struct sip_msg* rpl, struct cell *Trans, unsigned int branch
 	*len += ruri.len;
 	
 	
-	     /* via */
-	send_sock = uri2sock(rpl, next_hop, &to_su, PROTO_NONE);
-	if (!send_sock) {
-		LOG(L_ERR, "build_dlg_ack: no socket found\n");
+	 /* via */
+#ifdef USE_DNS_FAILOVER
+	if (use_dns_failover){
+		dns_srv_handle_init(&dns_h);
+		if ((uri2dst(&dns_h , &dst, rpl, next_hop, PROTO_NONE)==0) ||
+				(dst.send_sock==0)){
+			dns_srv_handle_put(&dns_h);
+			LOG(L_ERR, "build_dlg_ack: no socket found\n");
+			goto error;
+		}
+		dns_srv_handle_put(&dns_h); /* not needed any more */
+	}else{
+		if ((uri2dst(0 , &dst, rpl, next_hop, PROTO_NONE)==0) ||
+				(dst.send_sock==0)){
+			LOG(L_ERR, "build_dlg_ack: no socket found\n");
+			goto error;
+		}
+	}
+#else
+	if ( (uri2dst( &dst, rpl, next_hop, PROTO_NONE)==0) || (dst.send_sock==0)){
+			LOG(L_ERR, "build_dlg_ack: no socket found\n");
 		goto error;
 	}
+#endif
 	
 	if (!t_calc_branch(Trans,  branch, branch_buf, &branch_len)) goto error;
 	branch_str.s = branch_buf;
 	branch_str.len = branch_len;
 	set_hostport(&hp, 0);
-	via = via_builder(&via_len, send_sock, &branch_str, 0, send_sock->proto, &hp);
+	via = via_builder(&via_len, &dst, &branch_str, 0, &hp);
 	if (!via) {
 		LOG(L_ERR, "build_dlg_ack: No via header got from builder\n");
 		goto error;
@@ -493,22 +537,21 @@ static inline int print_content_length(str* dest, str* body)
 {
 	static char content_length[10];
 	int len;
+	int b_len;
 	char* tmp;
 
 	     /* Print Content-Length */
-	if (body) {
-		tmp = int2str(body->len, &len);
-		if (len >= sizeof(content_length)) {
-			LOG(L_ERR, "ERROR: print_content_length: content_len too big\n");
-			return -1;
-		}
-		memcpy(content_length, tmp, len); 
-		dest->s = content_length;
-		dest->len = len;
-	} else {
+	b_len=body?body->len:0;
+	tmp = int2str(b_len, &len);
+	if (len >= sizeof(content_length)) {
+		LOG(L_ERR, "ERROR: print_content_length: content_len too big\n");
 		dest->s = 0;
 		dest->len = 0;
+		return -1;
 	}
+	memcpy(content_length, tmp, len); 
+	dest->s = content_length;
+	dest->len = len;
 	return 0;
 }
 
@@ -538,7 +581,8 @@ static inline int print_cseq_num(str* _s, dlg_t* _d)
 /*
  * Create Via header
  */
-static inline int assemble_via(str* dest, struct cell* t, struct socket_info* sock, int branch)
+static inline int assemble_via(str* dest, struct cell* t, 
+								struct dest_info* dst, int branch)
 {
 	static char branch_buf[MAX_BRANCH_PARAM_LEN];
 	char* via;
@@ -560,7 +604,7 @@ static inline int assemble_via(str* dest, struct cell* t, struct socket_info* so
 #endif
 
 	set_hostport(&hp, 0);
-	via = via_builder(&via_len, sock, &branch_str, 0, sock->proto, &hp);
+	via = via_builder(&via_len, dst, &branch_str, 0, &hp);
 	if (!via) {
 		LOG(L_ERR, "assemble_via: via building failed\n");
 		return -2;
@@ -683,7 +727,7 @@ static inline char* print_callid(char* w, dlg_t* dialog, struct cell* t)
  * Create a request
  */
 char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int branch, 
-			struct cell *t, int* len, struct socket_info* send_sock)
+			struct cell *t, int* len, struct dest_info* dst)
 {
 	char* buf, *w;
 	str content_length, cseq, via;
@@ -702,7 +746,7 @@ char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int bra
 	}
 	*len = method->len + 1 + dialog->hooks.request_uri->len + 1 + SIP_VERSION_LEN + CRLF_LEN;
 
-	if (assemble_via(&via, t, send_sock, branch) < 0) {
+	if (assemble_via(&via, t, dst, branch) < 0) {
 		LOG(L_ERR, "build_uac_req(): Error while assembling Via\n");
 		return 0;
 	}
@@ -715,7 +759,8 @@ char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int bra
 	*len += CALLID_LEN + dialog->id.call_id.len + CRLF_LEN;                                      /* Call-ID */
 	*len += CSEQ_LEN + cseq.len + 1 + method->len + CRLF_LEN;                                    /* CSeq */
 	*len += calculate_routeset_length(dialog);                                                   /* Route set */
-	*len += (body ? (CONTENT_LENGTH_LEN + content_length.len + CRLF_LEN) : 0);                   /* Content-Length */
+	*len += CONTENT_LENGTH_LEN + content_length.len + CRLF_LEN; /* Content-
+																	 Length */
 	*len += (server_signature ? (USER_AGENT_LEN + CRLF_LEN) : 0);                                /* Signature */
 	*len += (headers ? headers->len : 0);                                                        /* Additional headers */
 	*len += (body ? body->len : 0);                                                              /* Message body */
@@ -737,12 +782,10 @@ char* build_uac_req(str* method, str* headers, str* body, dlg_t* dialog, int bra
 	w = print_callid(w, dialog, t);                       /* Call-ID */
 	w = print_routeset(w, dialog);                        /* Route set */
 
-	     /* Content-Length */
-	if (body) {
-		memapp(w, CONTENT_LENGTH, CONTENT_LENGTH_LEN);
-		memapp(w, content_length.s, content_length.len);
-		memapp(w, CRLF, CRLF_LEN);
-	}
+     /* Content-Length */
+	memapp(w, CONTENT_LENGTH, CONTENT_LENGTH_LEN);
+	memapp(w, content_length.s, content_length.len);
+	memapp(w, CRLF, CRLF_LEN);
 	
 	     /* Server signature */
 	if (server_signature) memapp(w, USER_AGENT CRLF, USER_AGENT_LEN + CRLF_LEN);
