@@ -1,5 +1,5 @@
 /*
- * $Id: t_reply.h,v 1.19.2.1 2007/01/25 16:14:49 andrei Exp $
+ * $Id: t_reply.h,v 1.24.2.1 2007/11/15 09:33:47 tirpi Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -31,12 +31,16 @@
 #define _T_REPLY_H
 
 #include "defs.h"
+#include "../../rpc.h"
 #include "../../tags.h"
 
 #include "h_table.h"
 
 
 extern int restart_fr_on_each_reply;
+extern int tm_aggregate_auth; /* aggregate authorization header value */
+extern unsigned int tm_blst_methods_add;
+extern unsigned int tm_blst_methods_lookup;
 
 /* reply processing status */
 enum rps {
@@ -72,6 +76,9 @@ typedef int (*treply_wb_f)( struct cell* trans,
 	unsigned int code, char * text, char * body, 
 	char * new_header, char * to_tag);
 
+/* wrapper function needed after changes in w_t_reply */
+int w_t_reply_wrp(struct sip_msg *m, unsigned int code, char *txt);
+
 #define LOCK_REPLIES(_t) lock(&(_t)->reply_mutex )
 #define UNLOCK_REPLIES(_t) unlock(&(_t)->reply_mutex )
 
@@ -82,6 +89,11 @@ typedef int (*treply_wb_f)( struct cell* trans,
  */
 int reply_received( struct sip_msg  *p_msg ) ;
 
+/* return 1 if a failure_route processes */
+int run_failure_handlers(struct cell *t, struct sip_msg *rpl,
+					int code, int extra_flags);
+typedef int (*run_failure_handlers_f)(struct cell*, struct sip_msg*, int, int);
+
 
 /* Retransmits the last sent inbound reply.
  * Returns  -1 - error
@@ -95,11 +107,6 @@ int t_retransmit_reply( /* struct sip_msg * */  );
  * returns 1 if everything was OK or -1 for error
  */
 
-#ifdef _OBSO
-int t_reply_light( struct cell *trans, char* buf, unsigned int len,
-		   unsigned int code, char * text,
-		   char *to_tag, unsigned int to_tag_len);
-#endif
 
 int t_reply_with_body( struct cell *trans, unsigned int code, 
 		       char * text, char * body, char * new_header, char * to_tag );
@@ -125,7 +132,6 @@ enum rps local_reply( struct cell *t, struct sip_msg *p_msg, int branch,
 void set_final_timer( /* struct s_table *h_table,*/ struct cell *t );
 
 void cleanup_uac_timers( struct cell *t );
-void cleanup_uac_timers_unsafe( struct cell *t );
 
 void on_negative_reply( struct cell* t, struct sip_msg* msg,
 	int code, void *param  );
@@ -145,9 +151,7 @@ void tm_init_tags();
 /* selects the branch for fwd-ing the reply */
 int t_pick_branch(int inc_branch, int inc_code, struct cell *t, int *res_code);
 
-int fifo_t_reply( FILE *stream, char *response_file );
-
-int unixsock_t_reply(str* msg);
+extern const char* rpc_reply_doc[2];
+void rpc_reply(rpc_t* rpc, void* c);
 
 #endif
-
