@@ -1,5 +1,5 @@
 /*
- * $Id: tm_load.h,v 1.20.2.2 2006/08/28 11:21:02 andrei Exp $
+ * $Id: tm_load.h,v 1.27 2006/11/14 18:11:07 andrei Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -44,7 +44,9 @@
 #include "t_fwd.h"
 #include "t_reply.h"
 #include "t_lookup.h"
+#include "t_reply.h"
 #include "dlg.h"
+#include "t_cancel.h"
 
 /* export not usable from scripts */
 #define NO_SCRIPT	-1
@@ -93,13 +95,25 @@ struct tm_binds {
 	new_dlg_uac_f      new_dlg_uac;
 	dlg_response_uac_f dlg_response_uac;
 	new_dlg_uas_f      new_dlg_uas;
+	update_dlg_uas_f   update_dlg_uas;
 	dlg_request_uas_f  dlg_request_uas;
+	set_dlg_target_f   set_dlg_target;
 	free_dlg_f         free_dlg;
 	print_dlg_f        print_dlg;
 	tgett_f            t_gett;
+	calculate_hooks_f  calculate_hooks;
+	t_uac_t            t_uac;
+	t_uac_with_ids_t   t_uac_with_ids;
+	trelease_f         t_release;
+	tunref_f           t_unref;
+	run_failure_handlers_f run_failure_handlers;
+	cancel_uacs_f      cancel_uacs;
+	prepare_request_within_f  prepare_request_within;
+	send_prepared_request_f   send_prepared_request;
 	enum route_mode*   route_mode;
 };
 
+extern int tm_init;
 
 typedef int(*load_tm_f)( struct tm_binds *tmb );
 int load_tm( struct tm_binds *tmb);
