@@ -1,5 +1,5 @@
 /*
- * $Id: uac.c,v 1.52.2.1 2006/06/12 08:22:48 rco Exp $
+ * $Id: uac.c,v 1.64.2.2 2008/06/12 15:37:35 alfredh Exp $
  *
  * simple UAC for things such as SUBSCRIBE or SMS gateway;
  * no authentication and other UAC features -- just send
@@ -50,6 +50,9 @@
  *  2004-02-11  FIFO/CANCEL + alignments (hash=f(callid,cseq)) (uli+jiri)
  *  2004-02-13  t->is_invite, t->local, t->noisy_ctimer replaced (bogdan)
  *  2004-08-23  avp support in t_uac (bogdan)
+ *  2005-12-16  t_uac will set the new_cell timers to the default values,
+ *               fixes 0 fr_timer bug (andrei)
+ *  2006-08-11  t_uac uses dns failover until it finds a send socket (andrei)
  */
 
 #include <string.h>
@@ -162,37 +165,72 @@ static inline unsigned int dlg2hash( dlg_t* dlg )
 	return hashid;
 }
 
-/*
- * Send a request using data from the dialog structure
- */
-int t_uac(str* method, str* headers, str* body, dlg_t* dialog,
-	  transaction_cb cb, void* cbp)
+static inline int t_uac_prepare(str* method, str* headers, str* body, 
+		dlg_t* dialog, transaction_cb cb, void* cbp, struct retr_buf **dst_req,
+		struct cell **dst_cell)
 {
-	struct socket_info* send_sock;
-	union sockaddr_union to_su;
+	struct dest_info dst;
 	struct cell *new_cell;
 	struct retr_buf *request;
 	char* buf;
         int buf_len, ret, flags;
 	unsigned int hi;
+	int is_ack;
+#ifdef USE_DNS_FAILOVER
+	struct dns_srv_handle dns_h;
+#endif
 
 	ret=-1;
+	hi=0; /* make gcc happy */
+	/*if (dst_req) *dst_req = NULL;*/
+	is_ack = (((method->len == 3) && (memcmp("ACK", method->s, 3)==0)) ? 1 : 0);
 	
 	/*** added by dcm 
 	 * - needed by external ua to send a request within a dlg
 	 */
-	if(!dialog->hooks.next_hop && w_calculate_hooks(dialog)<0)
+	if (w_calculate_hooks(dialog)<0 && !dialog->hooks.next_hop)
 		goto error2;
 
+	if (!dialog->loc_seq.is_set) {
+		/* this is the first request in the dialog,
+		set cseq to default value now - Miklos */
+		dialog->loc_seq.value = DEFAULT_CSEQ;
+		dialog->loc_seq.is_set = 1;
+	}
+
 	DBG("DEBUG:tm:t_uac: next_hop=<%.*s>\n",dialog->hooks.next_hop->len,
 			dialog->hooks.next_hop->s);
 	/* it's a new message, so we will take the default socket */
-	send_sock = uri2sock(0, dialog->hooks.next_hop, &to_su, PROTO_NONE);
-	if (!send_sock) {
+#ifdef USE_DNS_FAILOVER
+	if (use_dns_failover){
+		dns_srv_handle_init(&dns_h);
+		if ((uri2dst(&dns_h, &dst, 0, dialog->hooks.next_hop, PROTO_NONE)==0)
+				|| (dst.send_sock==0)){
+			dns_srv_handle_put(&dns_h);
+			ser_error = E_NO_SOCKET;
+			ret=ser_error;
+			LOG(L_ERR, "t_uac: no socket found\n");
+			goto error2;
+		}
+		dns_srv_handle_put(&dns_h); /* not needed anymore */
+	}else{
+		if ((uri2dst(0, &dst, 0, dialog->hooks.next_hop, PROTO_NONE)==0) ||
+				(dst.send_sock==0)){
+			ser_error = E_NO_SOCKET;
+			ret=ser_error;
+			LOG(L_ERR, "t_uac: no socket found\n");
+			goto error2;
+		}
+	}
+#else
+	if ((uri2dst(&dst, 0, dialog->hooks.next_hop, PROTO_NONE)==0) ||
+			(dst.send_sock==0)){
+		ser_error = E_NO_SOCKET;
 		ret=ser_error;
 		LOG(L_ERR, "t_uac: no socket found\n");
 		goto error2;
-	}	
+	}
+#endif
 
 	new_cell = build_cell(0); 
 	if (!new_cell) {
@@ -200,19 +238,26 @@ int t_uac(str* method, str* headers, str* body, dlg_t* dialog,
 		LOG(L_ERR, "t_uac: short of cell shmem\n");
 		goto error2;
 	}
+	/* init timers hack, new_cell->fr_timer and new_cell->fr_inv_timer
+	 * must be set, or else the fr will happen immediately
+	 * we can't call init_new_t() because we don't have a sip msg
+	 * => we'll ignore t_set_fr() or avp timer value and will use directly the
+	 * module params fr_inv_timer and fr_timer -- andrei */
+	new_cell->fr_timeout=fr_timeout;
+	new_cell->fr_inv_timeout=fr_inv_timeout;
 
 	/* better reset avp list now - anyhow, it's useless from
 	 * this point (bogdan) */
 	reset_avps();
 
-	/* add the callback the the transaction for LOCAL_COMPLETED event */
+	/* add the callback the transaction for LOCAL_COMPLETED event */
  
-        flags = TMCB_LOCAL_COMPLETED;
-             /* Add also TMCB_LOCAL_REPLY_OUT if provisional replies are desired */
-        if (pass_provisional_replies) flags |= TMCB_LOCAL_RESPONSE_OUT;
- 
-        if(cb && insert_tmcb(&(new_cell->tmcb_hl), flags, cb, cbp)!=1){
-                ret=E_OUT_OF_MEM; 
+	flags = TMCB_LOCAL_COMPLETED;
+	/* Add also TMCB_LOCAL_REPLY_OUT if provisional replies are desired */
+	if (pass_provisional_replies) flags |= TMCB_LOCAL_RESPONSE_OUT;
+
+	if(cb && insert_tmcb(&(new_cell->tmcb_hl), flags, cb, cbp)!=1){
+		ret=E_OUT_OF_MEM; 
 		LOG(L_ERR, "t_uac: short of tmcb shmem\n");
 		goto error2;
 	}
@@ -223,18 +268,18 @@ int t_uac(str* method, str* headers, str* body, dlg_t* dialog,
 	set_kr(REQ_FWDED);
 
 	request = &new_cell->uac[0].request;
-	request->dst.to = to_su;
-	request->dst.send_sock = send_sock;
-	request->dst.proto = send_sock->proto;
-	request->dst.proto_reserved1 = 0;
+	
+	request->dst = dst;
 
-	hi=dlg2hash(dialog);
-	LOCK_HASH(hi);
-	insert_into_hash_table_unsafe(new_cell, hi);
-	UNLOCK_HASH(hi);
+	if (!is_ack) {
+		hi=dlg2hash(dialog);
+		LOCK_HASH(hi);
+		insert_into_hash_table_unsafe(new_cell, hi);
+		UNLOCK_HASH(hi);
+	}
 
 	buf = build_uac_req(method, headers, body, dialog, 0, new_cell,
-		&buf_len, send_sock);
+		&buf_len, &dst);
 	if (!buf) {
 		LOG(L_ERR, "t_uac: Error while building message\n");
 		ret=E_OUT_OF_MEM;
@@ -248,29 +293,28 @@ int t_uac(str* method, str* headers, str* body, dlg_t* dialog,
 	request->buffer_len = buf_len;
 	new_cell->nr_of_outgoings++;
 	
-	if (SEND_BUFFER(request) == -1) {
-		LOG(L_ERR, "t_uac: Attempt to send to '%.*s' failed\n", 
-			dialog->hooks.next_hop->len,
-			dialog->hooks.next_hop->s);
-	}
+	if (dst_req) *dst_req = request;
+	if (dst_cell) *dst_cell = new_cell;
 	
-	start_retr(request);
 	return 1;
 
  error1:
-	LOCK_HASH(hi);
-	remove_from_hash_table_unsafe(new_cell);
-	UNLOCK_HASH(hi);
+ 	if (!is_ack) {
+		LOCK_HASH(hi);
+		remove_from_hash_table_unsafe(new_cell);
+		UNLOCK_HASH(hi);
+	}
 	free_cell(new_cell);
 error2:
 	return ret;
 }
 
-
 /*
- * Send a message within a dialog
+ * Prepare a message within a dialog
  */
-int req_within(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb completion_cb, void* cbp)
+int prepare_req_within(str* method, str* headers, 
+		str* body, dlg_t* dialog, transaction_cb completion_cb, 
+		void* cbp, struct retr_buf **dst_req)
 {
 	if (!method || !dialog) {
 		LOG(L_ERR, "req_within: Invalid parameter value\n");
@@ -282,6 +326,94 @@ int req_within(str* method, str* headers, str* body, dlg_t* dialog, transaction_
 		goto err;
 	}
 
+	if ((method->len == 3) && (!memcmp("ACK", method->s, 3))) goto send;
+	if ((method->len == 6) && (!memcmp("CANCEL", method->s, 6))) goto send;
+	dialog->loc_seq.value++; /* Increment CSeq */
+ send:
+	return t_uac_prepare(method, headers, body, dialog, completion_cb, cbp, dst_req, 0);
+
+ err:
+	/* if (cbp) shm_free(cbp); */
+	/* !! never free cbp here because if t_uac_prepare fails, cbp is not freed
+	 * and thus caller has no chance to discover if it is freed or not !! */
+	return -1;
+}
+
+static inline void send_prepared_request_impl(struct retr_buf *request, int retransmit)
+{
+	if (SEND_BUFFER(request) == -1) {
+		LOG(L_ERR, "t_uac: Attempt to send to precreated request failed\n");
+	}
+	
+	if (retransmit && (start_retr(request)!=0))
+		LOG(L_CRIT, "BUG: t_uac: failed to start retr. for %p\n", request);
+}
+
+void send_prepared_request(struct retr_buf *request)
+{
+	send_prepared_request_impl(request, 1 /* retransmit */);
+}
+
+/*
+ * Send a request using data from the dialog structure
+ */
+int t_uac(str* method, str* headers, str* body, dlg_t* dialog,
+	  transaction_cb cb, void* cbp)
+{
+	struct retr_buf *request;
+	struct cell *cell;
+	int ret;
+	int is_ack;
+
+	ret = t_uac_prepare(method, headers, body, dialog, cb, cbp, &request, &cell);
+	if (ret < 0) return ret;
+	is_ack = (method->len == 3) && (memcmp("ACK", method->s, 3)==0) ? 1 : 0;
+	send_prepared_request_impl(request, !is_ack /* retransmit */);
+	if (cell && is_ack)
+		free_cell(cell);
+	return ret;
+}
+
+/*
+ * Send a request using data from the dialog structure
+ * ret_index and ret_label will identify the new cell
+ */
+int t_uac_with_ids(str* method, str* headers, str* body, dlg_t* dialog,
+	transaction_cb cb, void* cbp,
+	unsigned int *ret_index, unsigned int *ret_label)
+{
+	struct retr_buf *request;
+	struct cell *cell;
+	int ret;
+	int is_ack;
+
+	ret = t_uac_prepare(method, headers, body, dialog, cb, cbp, &request, &cell);
+	if (ret < 0) return ret;
+	is_ack = (method->len == 3) && (memcmp("ACK", method->s, 3)==0) ? 1 : 0;
+	send_prepared_request_impl(request, !is_ack /* retransmit */);
+	if (is_ack) {
+		if (cell) free_cell(cell);
+		if (ret_index && ret_label)
+			*ret_index = *ret_label = 0;
+	} else {
+		if (ret_index && ret_label) {
+			*ret_index = cell->hash_index;
+			*ret_label = cell->label;
+		}
+	}
+	return ret;
+}
+
+/*
+ * Send a message within a dialog
+ */
+int req_within(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb completion_cb, void* cbp)
+{
+	if (!method || !dialog) {
+		LOG(L_ERR, "req_within: Invalid parameter value\n");
+		goto err;
+	}
+
 	if ((method->len == 3) && (!memcmp("ACK", method->s, 3))) goto send;
 	if ((method->len == 6) && (!memcmp("CANCEL", method->s, 6))) goto send;
 	dialog->loc_seq.value++; /* Increment CSeq */
@@ -322,7 +454,7 @@ int req_outside(str* method, str* to, str* from, str* headers, str* body, dlg_t*
 /*
  * Send a transactional request, no dialogs involved
  */
-int request(str* m, str* ruri, str* to, str* from, str* h, str* b, transaction_cb c, void* cp)
+int request(str* m, str* ruri, str* to, str* from, str* h, str* b, str *next_hop, transaction_cb c, void* cp)
 {
 	str callid, fromtag;
 	dlg_t* dialog;
@@ -341,12 +473,27 @@ int request(str* m, str* ruri, str* to, str* from, str* h, str* b, transaction_c
 	if (ruri) {
 		dialog->rem_target.s = ruri->s;
 		dialog->rem_target.len = ruri->len;
-		dialog->hooks.request_uri = &dialog->rem_target;
+		/* hooks will be set from w_calculate_hooks */
 	}
+
+	if (next_hop) dialog->dst_uri = *next_hop;
 	w_calculate_hooks(dialog);
 
+	/* WARNING:
+	 * to be clean it should be called 
+	 *   set_dlg_target(dialog, ruri, next_hop);
+	 * which sets both uris if given [but it duplicates them in shm!]
+	 *
+	 * but in this case the _ruri parameter in set_dlg_target
+	 * must be optional (it is needed now) and following hacks
+	 *   dialog->rem_target.s = 0;
+	 *   dialog->dst_uri.s = 0;
+	 * before freeing dialog here must be removed
+	 */
+
 	res = t_uac(m, h, b, dialog, c, cp);
 	dialog->rem_target.s = 0;
+	dialog->dst_uri.s = 0;
 	free_dlg(dialog);
 	return res;
 
