@@ -1,5 +1,5 @@
 /*
- * $Id: uac.h,v 1.21.2.1 2006/06/12 08:22:48 rco Exp $
+ * $Id: uac.h,v 1.25 2006/11/14 18:11:07 andrei Exp $
  *
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -37,6 +37,7 @@
 #include "../../str.h"
 #include "dlg.h"
 #include "t_hooks.h"
+#include "h_table.h"
 
 #define DEFAULT_CSEQ 10 /* Default CSeq number */
 
@@ -48,7 +49,14 @@ extern int pass_provisional_replies; /* Pass provisional replies to fifo applica
  */
 typedef int (*reqwith_t)(str* m, str* h, str* b, dlg_t* d, transaction_cb c, void* cp);
 typedef int (*reqout_t)(str* m, str* t, str* f, str* h, str* b, dlg_t** d, transaction_cb c, void* cp);
-typedef int (*req_t)(str* m, str* ruri, str* t, str* f, str* h, str* b, transaction_cb c, void* cp);
+typedef int (*req_t)(str* m, str* ruri, str* t, str* f, str* h, str* b, str *next_hop, transaction_cb c, void* cp);
+typedef int (*t_uac_t)(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb cb, void* cbp);
+typedef int (*t_uac_with_ids_t)(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb cb, void* cbp,
+		unsigned int *ret_index, unsigned int *ret_label);
+typedef int (*prepare_request_within_f)(str* method, str* headers, 
+		str* body, dlg_t* dialog, transaction_cb cb, void* cbp,
+		struct retr_buf **request_dst);
+typedef void (*send_prepared_request_f)(struct retr_buf *request_dst);
 
 
 /*
@@ -68,7 +76,12 @@ int uac_init(void);
  */
 int t_uac(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb cb, void* cbp);
 
-
+/*
+ * Send a request
+ * ret_index and ret_label will identify the new cell
+ */
+int t_uac_with_ids(str* method, str* headers, str* body, dlg_t* dialog, transaction_cb cb, void* cbp,
+			unsigned int *ret_index, unsigned int *ret_label);
 /*
  * Send a message within a dialog
  */
@@ -80,11 +93,15 @@ int req_within(str* m, str* h, str* b, dlg_t* d, transaction_cb c, void* cp);
  */
 int req_outside(str* m, str* t, str* f, str* h, str* b, dlg_t** d, transaction_cb c, void* cp);
 
-
 /*
  * Send a transactional request, no dialogs involved
  */
-int request(str* m, str* ruri, str* to, str* from, str* h, str* b, transaction_cb c, void* cp);
+int request(str* m, str* ruri, str* to, str* from, str* h, str* b, str *next_hop, transaction_cb c, void* cp);
+
+int prepare_req_within(str* method, str* headers, str* body, dlg_t* dialog,
+	  transaction_cb cb, void* cbp, struct retr_buf **dst_req);
+
+void send_prepared_request(struct retr_buf *request);
 
 
 #endif
