@@ -1,5 +1,5 @@
 /*
- * $Id: uridb_mod.h,v 1.2 2004/06/08 10:55:31 andrei Exp $
+ * $Id: uridb_mod.h,v 1.3 2005/11/30 17:01:28 janakj Exp $
  *
  * Various URI related functions
  *
@@ -41,14 +41,11 @@
 /*
  * Module parameters variables
  */
-extern str uri_table;                 /* Name of URI table */
-extern str uri_user_col;              /* Name of username column in URI table */
-extern str uri_domain_col;            /* Name of domain column in URI table */
-extern str uri_uriuser_col;           /* Name of uri_user column in URI table */
-extern str subscriber_table;          /* Name of subscriber table */
-extern str subscriber_user_col;       /* Name of user column in subscriber table */
-extern str subscriber_domain_col;     /* Name of domain column in subscriber table */
-extern int use_uri_table;             /* Whether or not should be uri table used */
-extern int use_domain;                /* Should does_uri_exist honor the domain part ? */
+extern str db_url;
+extern str uri_table;
+extern str uid_col;
+extern str did_col;
+extern str username_col;
+extern str flags_col;
 
 #endif /* URI_MOD_H */
