@@ -1,5 +1,5 @@
 /* 
- * $Id: udomain.c,v 1.41.2.2 2006/11/24 17:38:44 andrei Exp $ 
+ * $Id: udomain.c,v 1.51.2.2 2008/02/02 16:33:49 janakj Exp $ 
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -31,6 +31,8 @@
  * 2004-06-07 updated to the new DB api (andrei)
  * 2004-08-23  hash function changed to process characters as unsigned
  *             -> no negative results occur (jku)
+ * 2005-02-25 incoming socket is saved in ucontact record (bogdan)
+ * 2006-11-23 switched to better hash functions and fixed hash size (andrei)
  *   
  */
 
@@ -40,48 +42,30 @@
 #include "../../dprint.h"
 #include "../../db/db.h"
 #include "../../ut.h"
+#include "../../parser/parse_param.h"
+#include "../../parser/parse_uri.h"
+#include "../../resolve.h"
+#include "../../socket_info.h"
 #include "ul_mod.h"            /* usrloc module parameters */
 #include "notify.h"
+#include "reg_avps.h"
+#include "../../hashes.h"
+
+/* #define HASH_STRING_OPTIMIZE */
 
 
 
-/* a little slower than hash_* , but better distribution for 
- * numbers and about the same for strings */
-#define hash_update_str2(s, end, p, v, h) \
-	do{ \
-		for ((p)=(s); (p)<=((end)-4); (p)+=4){ \
-			(v)=(*(p)*16777213)+((p)[1]*65537)+((p)[2]*257)+(p)[3]; \
-			(h)=16777259*(h)+((v)^((v)<<17)); \
-		} \
-		(v)=0; \
-		for (;(p)<(end); (p)++){ (v)*=251; (v)+=*(p);} \
-		(h)=16777259*(h)+((v)^((v)<<17)); \
-	}while(0)
-
-/* internal use: call it to adjust the h from hash_update_str */
-#define hash_finish2(h) (((h)+((h)>>7))+(((h)>>13)+((h)>>23)))
-
-/* a little slower than get_hash1_raw() , but better distribution for 
- * numbers and about the same for strings */
-inline static unsigned int get_hash1_raw2(char* s, int len)
-{
-	char* p;
-	register unsigned v;
-	register unsigned h;
-	
-	h=0;
-	
-	hash_update_str2(s, s+len, p, v, h);
-	return hash_finish2(h);
-}
-
 
 /*
  * Hash function
  */
 static inline int hash_func(udomain_t* _d, unsigned char* _s, int _l)
 {
+#ifdef HASH_STRING_OPTIMIZE
+	return get_hash1_raw((char*)_s, _l) % UDOMAIN_HASH_SIZE;
+#else
 	return get_hash1_raw2((char*)_s, _l) % UDOMAIN_HASH_SIZE;
+#endif
 }
 
 
@@ -227,26 +211,91 @@ void print_udomain(FILE* _f, udomain_t* _d)
 }
 
 
+static struct socket_info* find_socket(str* received)
+{
+	struct sip_uri puri;
+	param_hooks_t hooks;
+	struct hostent* he;
+	struct ip_addr ip;
+	struct socket_info* si;
+	param_t* params;
+	unsigned short port;
+	char* buf;
+	int error;
+
+	if (!received) return 0;
+
+	si = 0;
+	if (parse_uri(received->s, received->len, &puri) < 0) {
+		LOG(L_ERR, "find_socket: Error while parsing received URI\n");
+		return 0;
+	}
+	
+	if (parse_params(&puri.params, CLASS_URI, &hooks, &params) < 0) {
+		LOG(L_ERR, "find_socket: Error while parsing received URI parameters\n");
+		return 0;
+	}
+
+	if (!hooks.uri.dstip || !hooks.uri.dstip->body.s || !hooks.uri.dstip->body.len) goto end;
+
+	buf = (char*)pkg_malloc(hooks.uri.dstip->body.len + 1);
+	if (!buf) {
+		LOG(L_ERR, "find_socket: No memory left\n");
+		goto end;
+	}
+	memcpy(buf, hooks.uri.dstip->body.s, hooks.uri.dstip->body.len);
+	buf[hooks.uri.dstip->body.len] = '\0';
+
+	he = resolvehost(buf);
+	if (he == 0) {
+		LOG(L_ERR, "find_socket: Unable to resolve '%s'\n", buf);
+		pkg_free(buf);
+		goto end;
+	}
+	pkg_free(buf);
+
+	if (hooks.uri.dstport && hooks.uri.dstport->body.s && hooks.uri.dstport->body.len) {
+		port = str2s(hooks.uri.dstport->body.s, hooks.uri.dstport->body.len, &error);
+		if (error != 0) {
+			LOG(L_ERR, "find_socket: Unable to convert port number\n");		
+			goto end;
+		}
+	} else {
+		port = 0;
+	}
+
+	hostent2ip_addr(&ip, he, 0);
+	si = find_si(&ip, port, puri.proto);
+	if (si == 0) {
+		LOG(L_ERR, "find_socket: Unable to find socket, using the default one\n");
+		goto end;
+	}
+	
+ end:
+	if (params) free_params(params);
+	return si;
+}
+
+
 
 int preload_udomain(db_con_t* _c, udomain_t* _d)
 {
 	char b[256];
-	db_key_t columns[10];
+	db_key_t columns[11];
 	db_res_t* res;
 	db_row_t* row;
 	int i, cseq;
 	unsigned int flags;
-
-	str user, contact, callid, ua, received;
+	struct socket_info* sock;
+	str uid, contact, callid, ua, received, instance, aor;
 	str* rec;
-	char* domain;
 	time_t expires;
 	qvalue_t q;
 
 	urecord_t* r;
 	ucontact_t* c;
 
-	columns[0] = user_col.s;
+	columns[0] = uid_col.s;
 	columns[1] = contact_col.s;
 	columns[2] = expires_col.s;
 	columns[3] = q_col.s;
@@ -255,7 +304,8 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	columns[6] = flags_col.s;
 	columns[7] = user_agent_col.s;
 	columns[8] = received_col.s;
-	columns[9] = domain_col.s;
+	columns[9] = instance_col.s;
+	columns[10] = aor_col.s;
 	
 	memcpy(b, _d->name->s, _d->name->len);
 	b[_d->name->len] = '\0';
@@ -265,8 +315,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 		return -1;
 	}
 
-	if (ul_dbf.query(_c, 0, 0, 0, columns, 0, (use_domain) ? 10 : 9, 0,
-				&res) < 0) {
+	if (ul_dbf.query(_c, 0, 0, 0, columns, 0, 11, 0, &res) < 0) {
 		LOG(L_ERR, "preload_udomain(): Error while doing db_query\n");
 		return -1;
 	}
@@ -282,14 +331,14 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 	for(i = 0; i < RES_ROW_N(res); i++) {
 		row = RES_ROWS(res) + i;
 		
-		user.s      = (char*)VAL_STRING(ROW_VALUES(row));
-		if (user.s == 0) {
-			LOG(L_CRIT, "preload_udomain: ERROR: bad username "
+		uid.s      = (char*)VAL_STRING(ROW_VALUES(row));
+		if (uid.s == 0) {
+			LOG(L_CRIT, "preload_udomain: ERROR: bad uid "
 							"record in table %s\n", b);
 			LOG(L_CRIT, "preload_udomain: ERROR: skipping...\n");
 			continue;
 		} else {
-			user.len = strlen(user.s);
+			uid.len = strlen(uid.s);
 		}
 
 		contact.s = (char*)VAL_STRING(ROW_VALUES(row) + 1);
@@ -297,7 +346,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 			LOG(L_CRIT, "preload_udomain: ERROR: bad contact "
 							"record in table %s\n", b);
 			LOG(L_CRIT, "preload_udomain: ERROR: for username %.*s\n",
-							user.len, user.s);
+							uid.len, uid.s);
 			LOG(L_CRIT, "preload_udomain: ERROR: skipping...\n");
 			continue;
 		} else {
@@ -312,7 +361,7 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 							" table %s\n", b);
 			LOG(L_CRIT, "preload_udomain: ERROR: for username %.*s,"
 							" contact %.*s\n",
-							user.len, user.s, contact.len, contact.s);
+							uid.len, uid.s, contact.len, contact.s);
 			LOG(L_CRIT, "preload_udomain: ERROR: skipping...\n");
 			continue;
 		} else {
@@ -333,23 +382,46 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 			if (received.s) {
 				received.len = strlen(received.s);
 				rec = &received;
+
+				sock = find_socket(&received);
 			} else {
 				received.len = 0;
 				rec = 0;
+				sock = 0;
 			}
 		} else {
+			received.s = 0;
+			received.len = 0;
 			rec = 0;
+			sock = 0;
 		}
 
-		if (use_domain) {
-			domain  = (char*)VAL_STRING(ROW_VALUES(row) + 9);
-			snprintf(b, 256, "%.*s@%s", user.len, ZSW(user.s), domain);
-			user.s = b;
-			user.len = strlen(b);
+		if (!VAL_NULL(ROW_VALUES(row) + 9)) {
+			instance.s  = (char*)VAL_STRING(ROW_VALUES(row) + 9);
+			if (instance.s) {
+				instance.len = strlen(instance.s);
+			} else {
+				instance.len = 0;
+			}
+		} else {
+			instance.s = 0;
+			instance.len = 0;
 		}
 
-		if (get_urecord(_d, &user, &r) > 0) {
-			if (mem_insert_urecord(_d, &user, &r) < 0) {
+		if (!VAL_NULL(ROW_VALUES(row) + 10)) {
+			aor.s  = (char*)VAL_STRING(ROW_VALUES(row) + 10);
+			if (aor.s) {
+				aor.len = strlen(aor.s);
+			} else {
+				aor.len = 0;
+			}
+		} else {
+			aor.s = 0;
+			aor.len = 0;
+		}
+
+		if (get_urecord(_d, &uid, &r) > 0) {
+			if (mem_insert_urecord(_d, &uid, &r) < 0) {
 				LOG(L_ERR, "preload_udomain(): Can't create a record\n");
 				ul_dbf.free_result(_c, res);
 				unlock_udomain(_d);
@@ -357,13 +429,15 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 			}
 		}
 		
-		if (mem_insert_ucontact(r, &contact, expires, q, &callid, cseq, flags, &c, &ua, rec) < 0) {
+		if (mem_insert_ucontact(r, &aor, &contact, expires, q, &callid, cseq, flags, &c, &ua, rec, sock, &instance) < 0) {
 			LOG(L_ERR, "preload_udomain(): Error while inserting contact\n");
 			ul_dbf.free_result(_c, res);
 			unlock_udomain(_d);
 			return -3;
 		}
 
+		db_read_reg_avps(_c, c);
+
 		     /* We have to do this, because insert_ucontact sets state to CS_NEW
 		      * and we have the contact in the database already
 			  * we also store zombies in database so we have to restore
@@ -381,16 +455,16 @@ int preload_udomain(db_con_t* _c, udomain_t* _d)
 /*
  * Insert a new record into domain
  */
-int mem_insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
+int mem_insert_urecord(udomain_t* _d, str* _uid, struct urecord** _r)
 {
 	int sl;
 	
-	if (new_urecord(_d->name, _aor, _r) < 0) {
+	if (new_urecord(_d->name, _uid, _r) < 0) {
 		LOG(L_ERR, "insert_urecord(): Error while creating urecord\n");
 		return -1;
 	}
 
-	sl = hash_func(_d, (unsigned char*)_aor->s, _aor->len);
+	sl = hash_func(_d, (unsigned char*)_uid->s, _uid->len);
 	slot_add(&_d->table[sl], *_r);
 	udomain_add(_d, *_r);
 	_d->users++;
@@ -468,9 +542,9 @@ void unlock_udomain(udomain_t* _d)
 /*
  * Create and insert a new record
  */
-int insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
+int insert_urecord(udomain_t* _d, str* _uid, struct urecord** _r)
 {
-	if (mem_insert_urecord(_d, _aor, _r) < 0) {
+	if (mem_insert_urecord(_d, _uid, _r) < 0) {
 		LOG(L_ERR, "insert_urecord(): Error while inserting record\n");
 		return -1;
 	}
@@ -481,17 +555,17 @@ int insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
 /*
  * Obtain a urecord pointer if the urecord exists in domain
  */
-int get_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
+int get_urecord(udomain_t* _d, str* _uid, struct urecord** _r)
 {
 	int sl, i;
 	urecord_t* r;
 
-	sl = hash_func(_d, (unsigned char*)_aor->s, _aor->len);
+	sl = hash_func(_d, (unsigned char*)_uid->s, _uid->len);
 
 	r = _d->table[sl].first;
 
 	for(i = 0; i < _d->table[sl].n; i++) {
-		if ((r->aor.len == _aor->len) && !memcmp(r->aor.s, _aor->s, _aor->len)) {
+		if ((r->uid.len == _uid->len) && !memcmp(r->uid.s, _uid->s, _uid->len)) {
 			*_r = r;
 			return 0;
 		}
@@ -506,12 +580,12 @@ int get_urecord(udomain_t* _d, str* _aor, struct urecord** _r)
 /*
  * Delete a urecord from domain
  */
-int delete_urecord(udomain_t* _d, str* _aor)
+int delete_urecord(udomain_t* _d, str* _uid)
 {
 	struct ucontact* c, *t;
 	struct urecord* r;
 
-	if (get_urecord(_d, _aor, &r) > 0) {
+	if (get_urecord(_d, _uid, &r) > 0) {
 		return 0;
 	}
 		
