@@ -1,5 +1,5 @@
 /* 
- * $Id: udomain.h,v 1.11.2.1 2006/11/24 17:38:44 andrei Exp $ 
+ * $Id: udomain.h,v 1.13 2006/11/24 17:20:08 andrei Exp $ 
  *
  * Usrloc domain structure
  *
@@ -30,6 +30,7 @@
  * History:
  * --------
  *  2003-03-11  changed to new locking scheme: locking.h (andrei)
+ *  2006-11-23 switched to fixed hash size (andrei)
  */
 
 
@@ -45,8 +46,10 @@
 #include "hslot.h"
 
 
-/* udomain hash size, for best performance always use a 2^k value */
-#define UDOMAIN_HASH_SIZE	8192
+/* udomain hash size, for best performance always use a 2^k value
+ * good values 8192: 9% increase over 4096, 16384: 5% inc. over 8192,
+ *  32768 4-5% inc over 16384 */
+#define UDOMAIN_HASH_SIZE	16384
 
 struct hslot;   /* Hash table slot */
 struct urecord; /* Usrloc record */
@@ -107,7 +110,7 @@ int timer_udomain(udomain_t* _d);
 /*
  * Insert record into domain
  */
-int mem_insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r);
+int mem_insert_urecord(udomain_t* _d, str* _uid, struct urecord** _r);
 
 
 /*
@@ -136,22 +139,22 @@ void unlock_udomain(udomain_t* _d);
 /*
  * Create and insert a new record
  */
-typedef int (*insert_urecord_t)(udomain_t* _d, str* _aor, struct urecord** _r);
-int insert_urecord(udomain_t* _d, str* _aor, struct urecord** _r);
+typedef int (*insert_urecord_t)(udomain_t* _d, str* _uid, struct urecord** _r);
+int insert_urecord(udomain_t* _d, str* _uid, struct urecord** _r);
 
 
 /*
  * Obtain a urecord pointer if the urecord exists in domain
  */
-typedef int  (*get_urecord_t)(udomain_t* _d, str* _a, struct urecord** _r);
-int get_urecord(udomain_t* _d, str* _aor, struct urecord** _r);
+typedef int  (*get_urecord_t)(udomain_t* _d, str* _uid, struct urecord** _r);
+int get_urecord(udomain_t* _d, str* _uid, struct urecord** _r);
 
 
 /*
  * Delete a urecord from domain
  */
-typedef int  (*delete_urecord_t)(udomain_t* _d, str* _a);
-int delete_urecord(udomain_t* _d, str* _aor);
+typedef int  (*delete_urecord_t)(udomain_t* _d, str* _uid);
+int delete_urecord(udomain_t* _d, str* _uid);
 
 
 #endif /* UDOMAIN_H */
