@@ -1,5 +1,5 @@
 /*
- * $Id: digest_parser.c,v 1.11.2.1 2006/02/28 09:26:16 janakj Exp $
+ * $Id: digest_parser.c,v 1.14 2006/07/24 16:43:55 janakj Exp $
  *
  * Digest credentials parser
  *
@@ -220,7 +220,7 @@ static inline int parse_digest_param(str* _s, dig_cred_t* _c)
 /*
  * Parse qop parameter body
  */
-static inline void parse_qop(struct qp* _q)
+void parse_qop(struct qp* _q)
 {
 	str s;
 
@@ -229,7 +229,9 @@ static inline void parse_qop(struct qp* _q)
 
 	trim(&s);
 
-	if ((s.len == QOP_AUTH_STR_LEN) &&
+	if (s.len == 0) {
+	    _q->qop_parsed = QOP_UNSPEC;
+	} else if ((s.len == QOP_AUTH_STR_LEN) &&
 	    !strncasecmp(s.s, QOP_AUTH_STR, QOP_AUTH_STR_LEN)) {
 		_q->qop_parsed = QOP_AUTH;
 	} else if ((s.len == QOP_AUTHINT_STR_LEN) &&
