@@ -1,5 +1,5 @@
 /*
- * $Id: parse_uri.c,v 1.17.2.1 2005/07/20 17:11:52 andrei Exp $
+ * $Id: parse_uri.c,v 1.23.2.4 2008/02/02 11:08:10 janakj Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -33,9 +33,16 @@
  * 2003-04-11  ser_error is now set in parse_uri (andrei)
  * 2003-04-26  ZSW (jiri)
  * 2003-07-03  sips:, r2, lr=on support added (andrei)
+ * 2005-02-25  preliminary tel uri support (andrei)
+ * 2005-03-03  more tel uri fixes (andrei)
+ * 2006-04-20  comp uri param. support (rfc3486) if defined USE_COMP  (andrei)
+ * 2007-09-10  introduced phone2tel option which allows NOT to consider
+ *             user=phone URIs as TEL URIs
+ *
  */
 
 
+#include "../globals.h"
 #include "parse_uri.h"
 #include <string.h>
 #include "../dprint.h"
@@ -60,7 +67,7 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 					PT_T, PT_R, PT_A, PT_N, PT_S, PT_P, PT_O, PT_R2, PT_T2,
 					PT_eq,
 					/* ttl */
-					      PTTL_T2, PTTL_L, PTTL_eq,
+					PTTL_T2, PTTL_L, PTTL_eq,
 					/* user */
 					PU_U, PU_S, PU_E, PU_R, PU_eq,
 					/* method */
@@ -71,6 +78,19 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 					PLR_L, PLR_R_FIN, PLR_eq,
 					/* r2 */
 					PR2_R, PR2_2_FIN, PR2_eq,
+#ifdef USE_COMP
+					/* comp */
+					PCOMP_C, PCOMP_O, PCOMP_M, PCOMP_P, PCOMP_eq,
+					
+					/* comp values */
+					/* sigcomp */
+					VCOMP_S, VCOMP_SIGC_I, VCOMP_SIGC_G,
+					VCOMP_SIGC_C, VCOMP_SIGC_O,  VCOMP_SIGC_M,
+					VCOMP_SIGC_P_FIN,
+					/* sergz */
+							VCOMP_SGZ_E, VCOMP_SGZ_R, VCOMP_SGZ_G,
+							VCOMP_SGZ_Z_FIN,
+#endif
 					
 					/* transport values */
 					/* udp */
@@ -96,6 +116,16 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 	char* pass;
 	int found_user;
 	int error_headers;
+	unsigned int scheme;
+	uri_type backup;
+#ifdef USE_COMP
+	str comp_str; /* not returned for now */
+	str comp_val; /* not returned for now */
+#endif
+	
+#define SIP_SCH		0x3a706973
+#define SIPS_SCH	0x73706973
+#define TEL_SCH		0x3a6c6574
 	
 #define case_port( ch, var) \
 	case ch: \
@@ -112,9 +142,12 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 							}else{ \
 								user.len=p-user.s; \
 							}\
+							/* save the uri type/scheme */ \
+							backup=uri->type; \
 							/* everything else is 0 */ \
 							memset(uri, 0, sizeof(struct sip_uri)); \
-							/* copy user & pass */ \
+							/* restore the scheme, copy user & pass */ \
+							uri->type=backup; \
 							uri->user=user; \
 							if (pass)	uri->passwd=password;  \
 							s=p+1; \
@@ -302,7 +335,30 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 				} \
 				break
 			
-	
+
+#ifdef USE_COMP
+#define comp_fin(c_state, comp_no) \
+			case c_state: \
+				switch(*p){ \
+					case '@': \
+						still_at_user; \
+						break; \
+					semicolon_case; \
+						/* param_set(b, v); */ \
+						uri->comp=(comp_no); \
+						break; \
+					question_case; \
+						/* param_set(b, v) */; \
+						uri->comp=(comp_no); \
+						break; \
+					colon_case;  \
+					default: \
+						state=URI_VAL_P; \
+						break; \
+				} \
+				break
+			
+#endif
 
 	/* init */
 	end=buf+len;
@@ -317,15 +373,18 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 	port_no=0;
 	state=URI_INIT;
 	memset(uri, 0, sizeof(struct sip_uri)); /* zero it all, just to be sure*/
-	/*look for sip:*/
+	/*look for sip:, sips: or tel:*/
 	if (len<5) goto error_too_short;
-	if (! ( ((buf[0]|0x20)=='s')&&((buf[1]|0x20)=='i')&&((buf[2]|0x20)=='p')))
-		goto error_bad_uri;
-	if (buf[3]!=':'){
-		/* parse also sips: */
-		if  (((buf[3]|0x20)=='s')&&(buf[4]==':')) {p++; uri->secure=1;}
+	scheme=buf[0]+(buf[1]<<8)+(buf[2]<<16)+(buf[3]<<24);
+	scheme|=0x20202020;
+	if (scheme==SIP_SCH){
+		uri->type=SIP_URI_T;
+	}else if(scheme==SIPS_SCH){
+		if(buf[4]==':'){ p++; uri->type=SIPS_URI_T;}
 		else goto error_bad_uri;
-	}
+	}else if (scheme==TEL_SCH){
+		uri->type=TEL_URI_T;
+	}else goto error_bad_uri;
 	
 	s=p;
 	for(;p<end; p++){
@@ -564,6 +623,14 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 					case 'R':
 						b=p;
 						state=PR2_R;
+						break;
+#ifdef USE_COMP
+					case 'c':
+					case 'C':
+						b=p;
+						state=PCOMP_C;
+						break;
+#endif
 					default:
 						state=URI_PARAM_P;
 				}
@@ -769,6 +836,44 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 						state=URI_VAL_P;
 				}
 				break;
+#ifdef USE_COMP
+			param_switch(PCOMP_C,  'o', 'O' , PCOMP_O);
+			param_switch(PCOMP_O,  'm', 'M' , PCOMP_M);
+			param_switch(PCOMP_M,  'p', 'P' , PCOMP_P);
+			param_switch1(PCOMP_P,  '=', PCOMP_eq);
+			/* value */
+			case PCOMP_eq:
+				param=&comp_str;
+				param_val=&comp_val;
+				switch (*p){
+					param_common_cases;
+					case 's':
+					case 'S':
+						v=p;
+						state=VCOMP_S;
+						break;
+					default:
+						v=p;
+						state=URI_VAL_P;
+				}
+				break;
+			/* sigcomp*/
+			value_switch_big(VCOMP_S, 'i', 'I', 'e', 'E',
+									VCOMP_SIGC_I, VCOMP_SGZ_E);
+			value_switch(VCOMP_SIGC_I, 'g', 'G', VCOMP_SIGC_G);
+			value_switch(VCOMP_SIGC_G, 'c', 'C', VCOMP_SIGC_C);
+			value_switch(VCOMP_SIGC_C, 'o', 'O', VCOMP_SIGC_O);
+			value_switch(VCOMP_SIGC_O, 'm', 'M', VCOMP_SIGC_M);
+			value_switch(VCOMP_SIGC_M, 'p', 'P', VCOMP_SIGC_P_FIN);
+			comp_fin(VCOMP_SIGC_P_FIN, COMP_SIGCOMP);
+			
+			/* sergz*/
+			value_switch(VCOMP_SGZ_E, 'r', 'R', VCOMP_SGZ_R);
+			value_switch(VCOMP_SGZ_R, 'g', 'G', VCOMP_SGZ_G);
+			value_switch(VCOMP_SGZ_G, 'z', 'Z', VCOMP_SGZ_Z_FIN);
+			comp_fin(VCOMP_SGZ_Z_FIN, COMP_SERGZ);
+#endif
+				
 				
 				
 			case URI_HEADERS:
@@ -878,6 +983,13 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 		case PM_eq:
 		case PLR_L: /* lr */
 		case PR2_R:  /* r2 */
+#ifdef USE_COMP
+		case PCOMP_C:
+		case PCOMP_O:
+		case PCOMP_M:
+		case PCOMP_P:
+		case PCOMP_eq:
+#endif
 			uri->params.s=s;
 			uri->params.len=p-s;
 			break;
@@ -910,6 +1022,22 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 			uri->params.len=p-s;
 			param_set(b, v);
 			break;
+#ifdef USE_COMP
+		case VCOMP_S:
+		case VCOMP_SIGC_I:
+		case VCOMP_SIGC_G:
+		case VCOMP_SIGC_C:
+		case VCOMP_SIGC_O:
+		case VCOMP_SIGC_M:
+		case VCOMP_SGZ_E:
+		case VCOMP_SGZ_R:
+		case VCOMP_SGZ_G:
+			/* unrecognized comp method, assume none */
+			uri->params.s=s;
+			uri->params.len=p-s;
+			/* uri->comp=COMP_NONE ; */
+			break;
+#endif
 		/* fin value states */
 		case VU_P_FIN:
 			uri->params.s=s;
@@ -935,6 +1063,20 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 			param_set(b, v);
 			uri->proto=PROTO_SCTP;
 			break;
+#ifdef USE_COMP
+		case VCOMP_SIGC_P_FIN:
+			uri->params.s=s;
+			uri->params.len=p-s;
+			/* param_set(b, v); */
+			uri->comp=COMP_SIGCOMP;
+			break;
+		case VCOMP_SGZ_Z_FIN:
+			uri->params.s=s;
+			uri->params.len=p-s;
+			/* param_set(b, v); */
+			uri->comp=COMP_SERGZ;
+			break;
+#endif
 		/* headers */
 		case URI_HEADERS:
 			uri->headers.s=s;
@@ -944,11 +1086,54 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 		default:
 			goto error_bug;
 	}
-	
-#if EXTRA_DEBUG
+	switch(uri->type){
+		case SIP_URI_T:
+			if ((phone2tel) &&
+			     (uri->user_param_val.len == 5) &&
+				 (strncmp(uri->user_param_val.s, "phone", 5) == 0)
+				) {
+				uri->type = TEL_URI_T;
+				/* move params from user into uri->params */
+				p=q_memchr(uri->user.s, ';', uri->user.len);
+				if (p){
+					uri->params.s=p+1;
+					uri->params.len=uri->user.s+uri->user.len-uri->params.s;
+					uri->user.len=p-uri->user.s;
+				}
+			}
+			break;
+		case SIPS_URI_T:
+			if ((phone2tel) &&
+			     (uri->user_param_val.len == 5) &&
+				 (strncmp(uri->user_param_val.s, "phone", 5) == 0)
+				) {
+				uri->type = TELS_URI_T;
+				p=q_memchr(uri->user.s, ';', uri->user.len);
+				if (p){
+					uri->params.s=p+1;
+					uri->params.len=uri->user.s+uri->user.len-uri->params.s;
+					uri->user.len=p-uri->user.s;
+				}
+			}
+			break;
+		case TEL_URI_T:
+		case TELS_URI_T:
+			/* fix tel uris, move the number in uri and empty the host */
+			uri->user=uri->host;
+			uri->host.s="";
+			uri->host.len=0;
+			break;
+		case ERROR_URI_T:
+			LOG(L_ERR, "ERROR: parse_uri unexpected error (BUG?)\n"); 
+			goto error_bad_uri;
+			break; /* do nothing, avoids a compilation warning */
+	}
+#ifdef EXTRA_DEBUG
 	/* do stuff */
-	DBG("parsed uri:\n user=<%.*s>(%d)\n passwd=<%.*s>(%d)\n host=<%.*s>(%d)\n"
-			" port=<%.*s>(%d): %d\n params=<%.*s>(%d)\n headers=<%.*s>(%d)\n",
+	DBG("parsed uri:\n type=%d user=<%.*s>(%d)\n passwd=<%.*s>(%d)\n"
+			" host=<%.*s>(%d)\n port=<%.*s>(%d): %d\n params=<%.*s>(%d)\n"
+			" headers=<%.*s>(%d)\n",
+			uri->type,
 			uri->user.len, ZSW(uri->user.s), uri->user.len,
 			uri->passwd.len, ZSW(uri->passwd.s), uri->passwd.len,
 			uri->host.len, ZSW(uri->host.s), uri->host.len,
@@ -972,49 +1157,55 @@ int parse_uri(char* buf, int len, struct sip_uri* uri)
 			uri->maddr.len, ZSW(uri->maddr.s), 
 			uri->maddr_val.len, ZSW(uri->maddr_val.s));
 	DBG("   lr=<%.*s>\n", uri->lr.len, ZSW(uri->lr.s)); 
+	DBG("   r2=<%.*s>\n", uri->r2.len, ZSW(uri->r2.s));
+#ifdef USE_COMP
+	DBG("   comp=%d\n", uri->comp);
+#endif
+
 #endif
 	return 0;
 	
 error_too_short:
-	LOG(L_ERR, "ERROR: parse_uri: uri too short: <%.*s> (%d)\n",
-			len, ZSW(buf), len);
-	ser_error=E_BAD_URI;
-	return E_BAD_URI;
+	DBG("parse_uri: uri too short: <%.*s> (%d)\n",
+		len, ZSW(buf), len);
+	goto error_exit;
 error_bad_char:
-	LOG(L_ERR, "ERROR: parse_uri: bad char '%c' in state %d"
-			" parsed: <%.*s> (%d) / <%.*s> (%d)\n",
-			*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf), len, ZSW(buf), len);
-	return E_BAD_URI;
+	DBG("parse_uri: bad char '%c' in state %d"
+		" parsed: <%.*s> (%d) / <%.*s> (%d)\n",
+		*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf),
+		len, ZSW(buf), len);
+	goto error_exit;
 error_bad_host:
-	LOG(L_ERR, "ERROR: parse_uri: bad host in uri (error at char %c in"
-			" state %d) parsed: <%.*s>(%d) /<%.*s> (%d)\n",
-			*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf), len, ZSW(buf), len);
-	ser_error=E_BAD_URI;
-	return E_BAD_URI;
+	DBG("parse_uri: bad host in uri (error at char %c in"
+		" state %d) parsed: <%.*s>(%d) /<%.*s> (%d)\n",
+		*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf),
+		len, ZSW(buf), len);
+	goto error_exit;
 error_bad_port:
-	LOG(L_ERR, "ERROR: parse_uri: bad port in uri (error at char %c in"
-			" state %d) parsed: <%.*s>(%d) /<%.*s> (%d)\n",
-			*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf), len, ZSW(buf), len);
-	ser_error=E_BAD_URI;
-	return E_BAD_URI;
+	DBG("parse_uri: bad port in uri (error at char %c in"
+		" state %d) parsed: <%.*s>(%d) /<%.*s> (%d)\n",
+		*p, state, (int)(p-buf), ZSW(buf), (int)(p-buf),
+		len, ZSW(buf), len);
+	goto error_exit;
 error_bad_uri:
-	LOG(L_ERR, "ERROR: parse_uri: bad uri,  state %d"
-			" parsed: <%.*s> (%d) / <%.*s> (%d)\n",
-			 state, (int)(p-buf), ZSW(buf), (int)(p-buf), len, ZSW(buf), len);
-	ser_error=E_BAD_URI;
-	return E_BAD_URI;
+	DBG("parse_uri: bad uri,  state %d"
+		" parsed: <%.*s> (%d) / <%.*s> (%d)\n",
+		state, (int)(p-buf), ZSW(buf), (int)(p-buf), len,
+		ZSW(buf), len);
+	goto error_exit;
 error_headers:
-	LOG(L_ERR, "ERROR: parse_uri: bad uri headers: <%.*s>(%d)"
-			" / <%.*s>(%d)\n",
-			uri->headers.len, ZSW(uri->headers.s), uri->headers.len,
-			len, ZSW(buf), len);
-	ser_error=E_BAD_URI;
-	return E_BAD_URI;
+	DBG("parse_uri: bad uri headers: <%.*s>(%d)"
+		" / <%.*s>(%d)\n",
+		uri->headers.len, ZSW(uri->headers.s), uri->headers.len,
+		len, ZSW(buf), len);
+	goto error_exit;
 error_bug:
 	LOG(L_CRIT, "BUG: parse_uri: bad  state %d"
 			" parsed: <%.*s> (%d) / <%.*s> (%d)\n",
 			 state, (int)(p-buf), ZSW(buf), (int)(p-buf), len, ZSW(buf), len);
+error_exit:
 	ser_error=E_BAD_URI;
+	uri->type=ERROR_URI_T;
 	return E_BAD_URI;
 }
 
