@@ -1,5 +1,5 @@
 /*
- * $Id: pass_fd.h,v 1.3 2004/08/24 08:45:10 janakj Exp $
+ * $Id: pass_fd.h,v 1.4 2005/06/16 14:05:24 andrei Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -30,9 +30,9 @@
 
 
 int send_fd(int unix_socket, void* data, int data_len, int fd);
-int receive_fd(int unix_socket, void* data, int data_len, int* fd);
+int receive_fd(int unix_socket, void* data, int data_len, int* fd, int flags);
 
-int recv_all(int socket, void* data, int data_len);
+int recv_all(int socket, void* data, int data_len, int flags);
 int send_all(int socket, void* data, int data_len);
 
 
