@@ -1,4 +1,4 @@
-/* $Id: resolve.c,v 1.19.2.2 2006/11/29 14:39:05 andrei Exp $*/
+/* $Id: resolve.c,v 1.24 2006/11/29 14:35:05 andrei Exp $*/
 /*
  *
  * Copyright (C) 2001-2003 FhG Fokus
@@ -29,6 +29,12 @@
  * -------
  *  2003-02-13  added proto to sip_resolvehost, for SRV lookups (andrei)
  *  2003-07-03  default port value set according to proto (andrei)
+ *  2005-07-11  added resolv_init (timeouts a.s.o) (andrei)
+ *  2006-04-13  added sip_hostport2su()  (andrei)
+ *  2006-07-13  rdata structures put on diet (andrei)
+ *  2006-07-17  rdata contains now also the record name (andrei)
+ *  2006-08-18  get_record can append also the additional records to the
+ *               returned list (andrei)
  */ 
 
 
@@ -42,6 +48,11 @@
 #include "dprint.h"
 #include "mem/mem.h"
 #include "ip_addr.h"
+#include "error.h"
+
+#ifdef USE_DNS_CACHE
+#include "dns_cache.h"
+#endif
 
 
 
@@ -49,6 +60,52 @@
 #define local_malloc pkg_malloc
 #define local_free   pkg_free
 
+#ifdef USE_IPV6
+int dns_try_ipv6=1; /* default on */
+#else
+int dns_try_ipv6=0; /* off, if no ipv6 support */
+#endif
+/* declared in globals.h */
+int dns_retr_time=-1;
+int dns_retr_no=-1;
+int dns_servers_no=-1;
+int dns_search_list=-1;
+
+
+/* init. the resolver
+ * params: retr_time  - time before retransmitting (must be >0)
+ *         retr_no    - retransmissions number
+ *         servers_no - how many dns servers will be used
+ *                      (from the one listed in /etc/resolv.conf)
+ *         search     - if 0 the search list in /etc/resolv.conf will
+ *                      be ignored (HINT: even if you don't have a
+ *                      search list in resolv.conf, it's still better
+ *                      to set search to 0, because an empty seachlist
+ *                      means in fact search "" => it takes more time)
+ * If any of the parameters <0, the default (system specific) value
+ * will be used. See also resolv.conf(5).
+ * returns: 0 on success, -1 on error
+ */
+int resolv_init()
+{
+	res_init();
+#ifdef HAVE_RESOLV_RES
+	if (dns_retr_time>0)
+		_res.retrans=dns_retr_time;
+	if (dns_retr_no>0)
+		_res.retry=dns_retr_no;
+	if (dns_servers_no>=0)
+		_res.nscount=dns_servers_no;
+	if (dns_search_list==0)
+		_res.options&=~(RES_DEFNAMES|RES_DNSRCH);
+#else
+#warning "no resolv timeout support"
+	LOG(L_WARN, "WARNING: resolv_init: no resolv options support - resolv"
+			" options will be ignored\n");
+#endif
+	return 0;
+}
+
 
 
  /*  skips over a domain name in a dns message
@@ -72,7 +129,7 @@ unsigned char* dns_skipname(unsigned char* p, unsigned char* end)
 		/* normal label */
 		p+=*p+1;
 	}
-	return (p>=end)?0:p;
+	return (p>end)?0:p;
 }
 
 
@@ -101,26 +158,37 @@ struct srv_rdata* dns_srv_parser( unsigned char* msg, unsigned char* end,
 								  unsigned char* rdata)
 {
 	struct srv_rdata* srv;
+	unsigned short priority;
+	unsigned short weight;
+	unsigned short port;
 	int len;
+	char name[MAX_DNS_NAME];
 	
 	srv=0;
-	if ((rdata+6)>=end) goto error;
-	srv=(struct srv_rdata*)local_malloc(sizeof(struct srv_rdata));
+	if ((rdata+6+1)>end) goto error;
+	
+	memcpy((void*)&priority, rdata, 2);
+	memcpy((void*)&weight,   rdata+2, 2);
+	memcpy((void*)&port,     rdata+4, 2);
+	rdata+=6;
+	if (dn_expand(msg, end, rdata, name, MAX_DNS_NAME-1)<0)
+		goto error;
+	len=strlen(name);
+	if (len>255)
+		goto error;
+	/* alloc enought space for the struct + null terminated name */
+	srv=local_malloc(sizeof(struct srv_rdata)-1+len+1);
 	if (srv==0){
 		LOG(L_ERR, "ERROR: dns_srv_parser: out of memory\n");
 		goto error;
 	}
+	srv->priority=ntohs(priority);
+	srv->weight=ntohs(weight);
+	srv->port=ntohs(port);
+	srv->name_len=len;
+	memcpy(srv->name, name, srv->name_len);
+	srv->name[srv->name_len]=0;
 	
-	memcpy((void*)&srv->priority, rdata, 2);
-	memcpy((void*)&srv->weight,   rdata+2, 2);
-	memcpy((void*)&srv->port,     rdata+4, 2);
-	rdata+=6;
-	srv->priority=ntohs(srv->priority);
-	srv->weight=ntohs(srv->weight);
-	srv->port=ntohs(srv->port);
-	if ((len=dn_expand(msg, end, rdata, srv->name, MAX_DNS_NAME-1))==-1)
-		goto error;
-	/* add terminating 0 ? (warning: len=compressed name len) */
 	return srv;
 error:
 	if (srv) local_free(srv);
@@ -158,36 +226,63 @@ struct naptr_rdata* dns_naptr_parser( unsigned char* msg, unsigned char* end,
 								  unsigned char* rdata)
 {
 	struct naptr_rdata* naptr;
+	unsigned char* flags;
+	unsigned char* services;
+	unsigned char* regexp;
+	unsigned short order;
+	unsigned short pref;
+	unsigned char flags_len;
+	unsigned char services_len;
+	unsigned char regexp_len;
 	int len;
+	char repl[MAX_DNS_NAME];
 	
 	naptr = 0;
-	if ((rdata + 7) >= end) goto error;
-	naptr=(struct naptr_rdata*)local_malloc(sizeof(struct naptr_rdata));
+	if ((rdata + 7 + 1)>end) goto error;
+	
+	memcpy((void*)&order, rdata, 2);
+	memcpy((void*)&pref, rdata + 2, 2);
+	flags_len = rdata[4];
+	if ((rdata + 7 + 1 +  flags_len) > end)
+		goto error;
+	flags=rdata+5;
+	services_len = rdata[5 + flags_len];
+	if ((rdata + 7 + 1 + flags_len + services_len) > end)
+		goto error;
+	services=rdata + 6 + flags_len;
+	regexp_len = rdata[6 + flags_len + services_len];
+	if ((rdata + 7 +1 + flags_len + services_len + regexp_len) > end)
+		goto error;
+	regexp=rdata + 7 + flags_len + services_len;
+	rdata = rdata + 7 + flags_len + services_len + regexp_len;
+	if (dn_expand(msg, end, rdata, repl, MAX_DNS_NAME-1) == -1)
+		goto error;
+	len=strlen(repl);
+	if (len>255)
+		goto error;
+	naptr=local_malloc(sizeof(struct naptr_rdata)+flags_len+services_len+
+						regexp_len+len+1-1);
 	if (naptr == 0){
 		LOG(L_ERR, "ERROR: dns_naptr_parser: out of memory\n");
 		goto error;
 	}
-	
-	memcpy((void*)&naptr->order, rdata, 2);
 	naptr->order=ntohs(naptr->order);
-	memcpy((void*)&naptr->pref, rdata + 2, 2);
 	naptr->pref=ntohs(naptr->pref);
-	naptr->flags_len = (int)rdata[4];
-	if ((rdata + 7 +  naptr->flags_len) >= end) goto error;
-	memcpy((void*)&naptr->flags, rdata + 5, naptr->flags_len);
-	naptr->services_len = (int)rdata[5 + naptr->flags_len];
-	if ((rdata + 7 + naptr->flags_len + naptr->services_len) >= end) goto error;
-	memcpy((void*)&naptr->services, rdata + 6 + naptr->flags_len, naptr->services_len);
-	naptr->regexp_len = (int)rdata[6 + naptr->flags_len + naptr->services_len];
-	if ((rdata + 7 + naptr->flags_len + naptr->services_len +
-					naptr->regexp_len) >= end) goto error;
-	memcpy((void*)&naptr->regexp, rdata + 7 + naptr->flags_len +
-				naptr->services_len, naptr->regexp_len);
-	rdata = rdata + 7 + naptr->flags_len + naptr->services_len + 
-			naptr->regexp_len;
-	if ((len=dn_expand(msg, end, rdata, naptr->repl, MAX_DNS_NAME-1)) == -1)
-		goto error;
-	/* add terminating 0 ? (warning: len=compressed name len) */
+	
+	naptr->flags=&naptr->str_table[0];
+	naptr->flags_len=flags_len;
+	memcpy(naptr->flags, flags, naptr->flags_len);
+	naptr->services=&naptr->str_table[flags_len];
+	naptr->services_len=services_len;
+	memcpy(naptr->services, services, naptr->services_len);
+	naptr->regexp=&naptr->str_table[flags_len+services_len];
+	naptr->regexp_len=regexp_len;
+	memcpy(naptr->regexp, regexp, naptr->regexp_len);
+	naptr->repl=&naptr->str_table[flags_len+services_len+regexp_len];
+	naptr->repl_len=len;
+	memcpy(naptr->repl, repl, len);
+	naptr->repl[len]=0; /* null term. */
+	
 	return naptr;
 error:
 	if (naptr) local_free(naptr);
@@ -202,15 +297,23 @@ struct cname_rdata* dns_cname_parser( unsigned char* msg, unsigned char* end,
 {
 	struct cname_rdata* cname;
 	int len;
+	char name[MAX_DNS_NAME];
 	
 	cname=0;
-	cname=(struct cname_rdata*)local_malloc(sizeof(struct cname_rdata));
+	if (dn_expand(msg, end, rdata, name, MAX_DNS_NAME-1)==-1)
+		goto error;
+	len=strlen(name);
+	if (len>255)
+		goto error;
+	/* alloc sizeof struct + space for the null terminated name */
+	cname=local_malloc(sizeof(struct cname_rdata)-1+len+1);
 	if(cname==0){
 		LOG(L_ERR, "ERROR: dns_cname_parser: out of memory\n");
 		goto error;
 	}
-	if ((len=dn_expand(msg, end, rdata, cname->name, MAX_DNS_NAME-1))==-1)
-		goto error;
+	cname->name_len=len;
+	memcpy(cname->name, name, cname->name_len);
+	cname->name[cname->name_len]=0;
 	return cname;
 error:
 	if (cname) local_free(cname);
@@ -226,7 +329,7 @@ struct a_rdata* dns_a_parser(unsigned char* rdata, unsigned char* end)
 {
 	struct a_rdata* a;
 	
-	if (rdata+4>=end) goto error;
+	if (rdata+4>end) goto error;
 	a=(struct a_rdata*)local_malloc(sizeof(struct a_rdata));
 	if (a==0){
 		LOG(L_ERR, "ERROR: dns_a_parser: out of memory\n");
@@ -246,7 +349,7 @@ struct aaaa_rdata* dns_aaaa_parser(unsigned char* rdata, unsigned char* end)
 {
 	struct aaaa_rdata* aaaa;
 	
-	if (rdata+16>=end) goto error;
+	if (rdata+16>end) goto error;
 	aaaa=(struct aaaa_rdata*)local_malloc(sizeof(struct aaaa_rdata));
 	if (aaaa==0){
 		LOG(L_ERR, "ERROR: dns_aaaa_parser: out of memory\n");
@@ -281,17 +384,17 @@ void free_rdata_list(struct rdata* head)
  * returns a dyn. alloc'ed struct rdata linked list with the parsed responses
  * or 0 on error
  * see rfc1035 for the query/response format */
-struct rdata* get_record(char* name, int type)
+struct rdata* get_record(char* name, int type, int flags)
 {
 	int size;
+	int skip;
 	int qno, answers_no;
 	int r;
-	int ans_len;
 	static union dns_query buff;
 	unsigned char* p;
-	unsigned char* t;
 	unsigned char* end;
-	static unsigned char answer[ANS_SIZE];
+	static char rec_name[MAX_DNS_NAME]; /* placeholder for the record name */
+	int rec_name_len;
 	unsigned short rtype, class, rdlength;
 	unsigned int ttl;
 	struct rdata* head;
@@ -326,26 +429,35 @@ struct rdata* get_record(char* name, int type)
 		for (;(p<end && (*p)); p++);
 		p+=1+2+2; /* skip the ending  '\0, QCODE and QCLASS */
 	#endif
-		if (p>=end) {
+		if (p>end) {
 			LOG(L_ERR, "ERROR: get_record: p>=end\n");
 			goto error;
 		}
 	};
 	answers_no=ntohs((unsigned short)buff.hdr.ancount);
-	ans_len=ANS_SIZE;
-	t=answer;
+again:
 	for (r=0; (r<answers_no) && (p<end); r++){
+#if 0
 		/*  ignore it the default domain name */
 		if ((p=dns_skipname(p, end))==0) {
 			LOG(L_ERR, "ERROR: get_record: skip_name=0 (#2)\n");
 			goto error;
 		}
-		/*
-		skip=dn_expand(buff.buff, end, p, t, ans_len);
+#else
+		if ((skip=dn_expand(buff.buff, end, p, rec_name, MAX_DNS_NAME-1))==-1){
+			LOG(L_ERR, "ERROR: get_record: dn_expand(rec_name) failed\n");
+			goto error;
+		}
+#endif
 		p+=skip;
-		*/
+		rec_name_len=strlen(rec_name);
+		if (rec_name_len>255){
+			LOG(L_ERR, "ERROR: get_record: dn_expand(rec_name): name too"
+					" long  (%d)\n", rec_name_len);
+			goto error;
+		}
 		/* check if enough space is left for type, class, ttl & size */
-		if ((p+2+2+4+2)>=end) goto error_boundary;
+		if ((p+2+2+4+2)>end) goto error_boundary;
 		/* get type */
 		memcpy((void*) &rtype, (void*)p, 2);
 		rtype=ntohs(rtype);
@@ -362,18 +474,14 @@ struct rdata* get_record(char* name, int type)
 		memcpy((void*)&rdlength, (void*)p, 2);
 		rdlength=ntohs(rdlength);
 		p+=2;
-		/* check for type */
-		/*
-		if (rtype!=type){
-			LOG(L_ERR, "WARNING: get_record: wrong type in answer (%d!=%d)\n",
-					rtype, type);
+		if ((flags & RES_ONLY_TYPE) && (rtype!=type)){
+			/* skip */
 			p+=rdlength;
 			continue;
 		}
-		*/
 		/* expand the "type" record  (rdata)*/
 		
-		rd=(struct rdata*) local_malloc(sizeof(struct rdata));
+		rd=(struct rdata*) local_malloc(sizeof(struct rdata)+rec_name_len+1-1);
 		if (rd==0){
 			LOG(L_ERR, "ERROR: get_record: out of memory\n");
 			goto error;
@@ -382,6 +490,9 @@ struct rdata* get_record(char* name, int type)
 		rd->class=class;
 		rd->ttl=ttl;
 		rd->next=0;
+		memcpy(rd->name, rec_name, rec_name_len);
+		rd->name[rec_name_len]=0;
+		rd->name_len=rec_name_len;
 		switch(rtype){
 			case T_SRV:
 				srv_rd= dns_srv_parser(buff.buff, end, p);
@@ -390,6 +501,8 @@ struct rdata* get_record(char* name, int type)
 				
 				/* insert sorted into the list */
 				for (crt=&head; *crt; crt= &((*crt)->next)){
+					if ((*crt)->type!=T_SRV)
+						continue;
 					crt_srv=(struct srv_rdata*)(*crt)->rdata;
 					if ((srv_rd->priority <  crt_srv->priority) ||
 					   ( (srv_rd->priority == crt_srv->priority) && 
@@ -439,13 +552,36 @@ struct rdata* get_record(char* name, int type)
 		p+=rdlength;
 		
 	}
+	if (flags & RES_AR){
+		flags&=~RES_AR;
+		answers_no=ntohs((unsigned short)buff.hdr.nscount);
+		DBG("get_record: skipping %d NS (p=%p, end=%p)\n", answers_no, p, end);
+		for (r=0; (r<answers_no) && (p<end); r++){
+			/* skip over the ns records */
+			if ((p=dns_skipname(p, end))==0) {
+				LOG(L_ERR, "ERROR: get_record: skip_name=0 (#3)\n");
+				goto error;
+			}
+			/* check if enough space is left for type, class, ttl & size */
+			if ((p+2+2+4+2)>end) goto error_boundary;
+			memcpy((void*)&rdlength, (void*)p+2+2+4, 2);
+			p+=2+2+4+2+ntohs(rdlength);
+		}
+		answers_no=ntohs((unsigned short)buff.hdr.arcount);
+		DBG("get_record: parsing %d ARs (p=%p, end=%p)\n", answers_no, p, end);
+		goto again; /* add also the additional records */
+	}
+			
 	return head;
 error_boundary:
 		LOG(L_ERR, "ERROR: get_record: end of query buff reached\n");
-		 if (head) free_rdata_list(head);
+		if (head) free_rdata_list(head);
 		return 0;
 error_parse:
-		LOG(L_ERR, "ERROR: get_record: rdata parse error \n");
+		LOG(L_ERR, "ERROR: get_record: rdata parse error (%s, %d), %p-%p"
+						" rtype=%d, class=%d, ttl=%d, rdlength=%d \n",
+				name, type,
+				p, end, rtype, class, ttl, rdlength);
 		if (rd) local_free(rd); /* rd->rdata=0 & rd is not linked yet into
 								   the list */
 error:
@@ -457,6 +593,8 @@ not_found:
 
 
 
+
+
 /* resolves a host name trying SRV lookup if *port==0 or normal A/AAAA lookup
  * if *port!=0.
  * when performing SRV lookup (*port==0) it will use proto to look for
@@ -464,14 +602,14 @@ not_found:
  * returns: hostent struct & *port filled with the port from the SRV record;
  *  0 on error
  */
-struct hostent* sip_resolvehost(str* name, unsigned short* port, int proto)
+struct hostent* _sip_resolvehost(str* name, unsigned short* port, int proto)
 {
 	struct hostent* he;
+	struct ip_addr* ip;
+	static char tmp[MAX_DNS_NAME]; /* tmp. buff. for SRV lookups */
 	struct rdata* head;
 	struct rdata* l;
 	struct srv_rdata* srv;
-	struct ip_addr* ip;
-	static char tmp[MAX_DNS_NAME]; /* tmp. buff. for SRV lookups */
 
 	/* try SRV if no port specified (draft-ietf-sip-srv-06) */
 	if ((port)&&(*port==0)){
@@ -514,8 +652,7 @@ struct hostent* sip_resolvehost(str* name, unsigned short* port, int proto)
 							proto);
 					return 0;
 			}
-
-			head=get_record(tmp, T_SRV);
+			head=get_record(tmp, T_SRV, RES_ONLY_TYPE);
 			for(l=head; l; l=l->next){
 				if (l->type!=T_SRV) continue; /*should never happen*/
 				srv=(struct srv_rdata*) l->rdata;
@@ -549,3 +686,31 @@ skip_srv:
 	he=resolvehost(tmp);
 	return he;
 }
+
+
+
+/* resolve host, port, proto using sip rules (e.g. use SRV if port=0 a.s.o)
+ *  and write the result in the sockaddr_union to
+ *  returns -1 on error (resolve failed), 0 on success */
+int sip_hostport2su(union sockaddr_union* su, str* name, unsigned short port,
+						int proto)
+{
+	struct hostent* he;
+	
+	
+	he=sip_resolvehost(name, &port, proto);
+	if (he==0){
+		ser_error=E_BAD_ADDRESS;
+		LOG(L_ERR, "ERROR: sip_hostport2su: could not resolve hostname:"
+					" \"%.*s\"\n", name->len, name->s);
+		goto error;
+	}
+	/* port filled by sip_resolvehost if empty*/
+	if (hostent2su(su, he, 0, port)<0){
+		ser_error=E_BAD_ADDRESS;
+		goto error;
+	}
+	return 0;
+error:
+	return -1;
+}
