@@ -1,4 +1,4 @@
-/* $Id: socket_info.c,v 1.12 2004/12/01 16:51:17 andrei Exp $
+/* $Id: socket_info.c,v 1.14.2.1 2008/04/17 21:33:50 calrissian Exp $
  *
  * find & manage listen addresses 
  *
@@ -278,6 +278,48 @@ found:
 	return si;
 }
 
+/* checks if the proto:port is one of the ports we listen on
+ * and returns the corresponding socket_info structure.
+ * if proto==0 (PROTO_NONE) the protocol is ignored
+ * returns  0 if not found
+ */
+struct socket_info* grep_sock_info_by_port(unsigned short port, 
+											unsigned short proto)
+{
+	struct socket_info* si;
+	struct socket_info** list;
+	unsigned short c_proto;
+
+	if (!port) {
+		goto not_found;
+	}
+	c_proto=proto?proto:PROTO_UDP;
+	do{
+		/* get the proper sock_list */
+		if (c_proto==PROTO_NONE)
+			list=&udp_listen;
+		else
+			list=get_sock_info_list(c_proto);
+	
+		if (list==0){
+			LOG(L_WARN, "WARNING: grep_sock_info_by_port: "
+						"unknown proto %d\n", c_proto);
+			goto not_found; /* false */
+		}
+		for (si=*list; si; si=si->next){
+			DBG("grep_sock_info_by_port - checking if port %d matches port %d\n", 
+					si->port_no, port);
+			if (si->port_no==port) {
+				goto found;
+			}
+		}
+	}while( (proto==0) && (c_proto=next_proto(c_proto)) );
+not_found:
+	return 0;
+found:
+	return si;
+}
+
 
 
 /* checks if the proto: ip:port is one of the address we listen on
@@ -369,6 +411,12 @@ int add_listen_iface(char* name, unsigned short port, unsigned short proto,
 #endif
 				port_no;
 		}
+#ifdef USE_TLS
+		else if ((c_proto==PROTO_TLS) && (proto==0)){
+			/* -l  ip:port => on udp:ip:port; tcp:ip:port and tls:ip:port+1? */
+			port++;
+		}
+#endif
 		if (new_sock2list(name, port, c_proto, flags, list)<0){
 			LOG(L_ERR, "ERROR: add_listen_iface: new_sock2list failed\n");
 			goto error;
@@ -511,8 +559,9 @@ error:
 
 
 /* fixes a socket list => resolve addresses, 
- * interface names, fills missing members, remove duplicates */
-static int fix_socket_list(struct socket_info **list)
+ * interface names, fills missing members, remove duplicates
+ * fills type_flags if not null with SOCKET_T_IPV4 and/or SOCKET_T_IPV6*/
+static int fix_socket_list(struct socket_info **list, int* type_flags)
 {
 	struct socket_info* si;
 	struct socket_info* l;
@@ -522,9 +571,10 @@ static int fix_socket_list(struct socket_info **list)
 	struct hostent* he;
 	char** h;
 	
+	if (type_flags)
+		*type_flags=0;
 	/* try to change all the interface names into addresses
 	 *  --ugly hack */
-	
 	for (si=*list;si;){
 		next=si->next;
 		if (add_interfaces(si->name.s, AF_INET, si->port_no,
@@ -592,6 +642,9 @@ static int fix_socket_list(struct socket_info **list)
 			}
 		hostent2ip_addr(&si->address, he, 0); /*convert to ip_addr 
 														 format*/
+		if (type_flags){
+			*type_flags|=(si->address.af==AF_INET)?SOCKET_T_IPV4:SOCKET_T_IPV6;
+		}
 		if ((tmp=ip_addr2a(&si->address))==0) goto error;
 		si->address_str.s=(char*)pkg_malloc(strlen(tmp)+1);
 		if (si->address_str.s==0){
@@ -701,11 +754,15 @@ error:
 
 
 
-/* fix all 3 socket lists
+/* fix all 3 socket lists, fills socket_types if non-null
  * return 0 on success, -1 on error */
-int fix_all_socket_lists()
+int fix_all_socket_lists(int* socket_types)
 {
 	struct utsname myname;
+	int flags;
+	
+	if (socket_types)
+		*socket_types=0;
 	
 	if ((udp_listen==0)
 #ifdef USE_TCP
@@ -746,23 +803,32 @@ int fix_all_socket_lists()
 			}
 		}
 	}
-	if (fix_socket_list(&udp_listen)!=0){
+	if (fix_socket_list(&udp_listen, &flags)!=0){
 		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
 				" udp failed\n");
 		goto error;
 	}
+	if (flags && socket_types){
+		*socket_types|=flags|SOCKET_T_UDP;
+	}
 #ifdef USE_TCP
-	if (!tcp_disable && (fix_socket_list(&tcp_listen)!=0)){
+	if (!tcp_disable && (fix_socket_list(&tcp_listen, &flags)!=0)){
 		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
 				" tcp failed\n");
 		goto error;
 	}
+	if (flags && socket_types){
+		*socket_types|=flags|SOCKET_T_TCP;
+	}
 #ifdef USE_TLS
-	if (!tls_disable && (fix_socket_list(&tls_listen)!=0)){
+	if (!tls_disable && (fix_socket_list(&tls_listen, &flags)!=0)){
 		LOG(L_ERR, "ERROR: fix_all_socket_lists: fix_socket_list"
 				" tls failed\n");
 		goto error;
 	}
+	if (flags && socket_types){
+		*socket_types|=flags|SOCKET_T_TLS;
+	}
 #endif
 #endif
 	if ((udp_listen==0)
