@@ -1,5 +1,5 @@
 /*
- * $Id: tcp_read.c,v 1.27 2004/08/24 08:45:10 janakj Exp $
+ * $Id: tcp_read.c,v 1.35.2.2 2007/02/21 01:51:45 andrei Exp $
  *
  * Copyright (C) 2001-2003 FhG Fokus
  *
@@ -33,6 +33,10 @@
  * 2003-07-01  tcp_read & friends take no a single tcp_connection 
  *              parameter & they set c->state to S_CONN_EOF on eof (andrei)
  * 2003-07-04  fixed tcp EOF handling (possible infinite loop) (andrei)
+ * 2005-07-05  migrated to the new io_wait code (andrei)
+ * 2006-02-03  use tsend_stream instead of send_all (andrei)
+ * 2006-10-13  added STUN support - state machine for TCP (vlada)
+ * 2007-02-20  fixed timeout calc. bug (andrei)
  */
 
 #ifdef USE_TCP
@@ -58,10 +62,31 @@
 #include "receive.h"
 #include "timer.h"
 #include "ut.h"
-#ifdef USE_TLS
+#ifdef CORE_TLS
 #include "tls/tls_server.h"
+#else
+#include "tls_hooks.h"
 #endif
 
+#define HANDLE_IO_INLINE
+#include "io_wait.h"
+#include <fcntl.h> /* must be included after io_wait.h if SIGIO_RT is used */
+#include "tsend.h"
+
+#ifdef USE_STUN
+#include "ser_stun.h"
+
+int is_msg_complete(struct tcp_req* r);
+
+#endif /* USE_STUN */
+
+/* types used in io_wait* */
+enum fd_types { F_NONE, F_TCPMAIN, F_TCPCONN };
+
+/* list of tcp connections handled by this process */
+static struct tcp_connection* tcp_conn_lst=0;
+static io_wait_h io_w; /* io_wait handler*/
+static int tcpmain_sock=-1;
 
 
 /* reads next available bytes
@@ -124,6 +149,11 @@ int tcp_read_headers(struct tcp_connection *c)
 	char *p;
 	struct tcp_req* r;
 	
+#ifdef USE_STUN
+	unsigned int mc;   /* magic cookie */
+	unsigned short body_len;
+#endif
+	
 	#define crlf_default_skip_case \
 					case '\n': \
 						r->state=H_LF; \
@@ -282,11 +312,98 @@ int tcp_read_headers(struct tcp_connection *c)
 						r->start=p;
 						break;
 					default:
+#ifdef USE_STUN
+						/* STUN support can be switched off even if it's compiled */
+						/* stun test */						
+						if (stun_allow_stun && (unsigned char)*p == 0x00) {
+							r->state=H_STUN_MSG;
+						/* body will used as pointer to the last used byte */
+							r->body=p;
+							body_len = 0;
+							DBG("stun msg detected\n");
+						}else
+#endif
 						r->state=H_SKIP;
 						r->start=p;
 				};
 				p++;
 				break;
+#ifdef USE_STUN
+			case H_STUN_MSG:
+				if ((r->pos - r->body) >= sizeof(struct stun_hdr)) {
+					r->content_len = 0;
+					/* copy second short from buffer where should be body 
+					 * length 
+					 */
+					memcpy(&body_len, &r->start[sizeof(unsigned short)], 
+						sizeof(unsigned short));
+					
+					body_len = ntohs(r->content_len);
+					
+					/* check if there is valid magic cookie */
+					memcpy(&mc, &r->start[sizeof(unsigned int)], 
+						sizeof(unsigned int));
+					mc = ntohl(mc);
+					/* using has_content_len as a flag if there should be
+					 * fingerprint or no
+					 */
+					r->has_content_len = (mc == MAGIC_COOKIE) ? 1 : 0;
+					
+					r->body += sizeof(struct stun_hdr);
+					p = r->body; 
+					
+					if (body_len > 0) {
+						r->state = H_STUN_READ_BODY;
+					}
+					else {
+						if (is_msg_complete(r) != 0) {
+							goto skip;
+						}
+						else {
+							/* set content_len to length of fingerprint */
+							body_len = sizeof(struct stun_attr) + 
+									   SHA_DIGEST_LENGTH;
+						}
+					}
+				}
+				else {
+					p = r->pos; 
+				}
+				break;
+				
+			case H_STUN_READ_BODY:
+				/* check if the whole body was read */
+				if ((r->pos - r->body) >= body_len) {
+					r->body += body_len;
+					p = r->body;
+					if (is_msg_complete(r) != 0) {
+						goto skip;
+					}
+					else {
+						/* set content_len to length of fingerprint */
+						body_len = sizeof(struct stun_attr)+SHA_DIGEST_LENGTH;
+					}
+				}
+				else {
+					p = r->pos;
+				}
+				break;
+				
+			case H_STUN_FP:
+				/* content_len contains length of fingerprint in this place! */
+				if ((r->pos - r->body) >= body_len) {
+					r->body += body_len;
+					p = r->body;
+					r->state = H_STUN_END;
+					r->complete = 1;
+					r->has_content_len = 1; /* hack to avoid error check */
+					goto skip;
+				}
+				else {
+					p = r->pos;
+				}
+				break;
+#endif /* USE_STUN */
 			change_state_case(H_CONT_LEN1,  'O', 'o', H_CONT_LEN2);
 			change_state_case(H_CONT_LEN2,  'N', 'n', H_CONT_LEN3);
 			change_state_case(H_CONT_LEN3,  'T', 't', H_CONT_LEN4);
@@ -385,15 +502,19 @@ skip:
 
 
 
-int tcp_read_req(struct tcp_connection* con)
+int tcp_read_req(struct tcp_connection* con, int* bytes_read)
 {
 	int bytes;
+	int total_bytes;
 	int resp;
 	long size;
 	struct tcp_req* req;
 	int s;
 	char c;
+	int ret;
 		
+		bytes=-1;
+		total_bytes=0;
 		resp=CONN_RELEASE;
 		s=con->fd;
 		req=&con->req;
@@ -415,7 +536,7 @@ again:
 			DBG("read= %d bytes, parsed=%d, state=%d, error=%d\n",
 					bytes, (int)(req->parsed-req->start), req->state,
 					req->error );
-			DBG("tcp_read_req: last char=%X, parsed msg=\n%.*s\n",
+			DBG("tcp_read_req: last char=0x%02X, parsed msg=\n%.*s\n",
 					*(req->parsed-1), (int)(req->parsed-req->start),
 					req->start);
 #endif
@@ -424,6 +545,7 @@ again:
 				resp=CONN_ERROR;
 				goto end_req;
 			}
+			total_bytes+=bytes;
 			/* eof check:
 			 * is EOF if eof on fd and req.  not complete yet,
 			 * if req. is complete we might have a second unparsed
@@ -487,7 +609,16 @@ again:
 							   previous char, req->parsed should be ok
 							   because we always alloc BUF_SIZE+1 */
 			*req->parsed=0;
-			if (receive_msg(req->start, req->parsed-req->start, &con->rcv)<0){
+#ifdef USE_STUN
+			if (req->state==H_STUN_END){
+				/* stun request */
+				ret = stun_process_msg(req->start, req->parsed-req->start,
+									 &con->rcv);
+			}else
+#endif
+				ret = receive_msg(req->start, req->parsed-req->start, &con->rcv);
+				
+			if (ret < 0) {
 				*req->parsed=c;
 				resp=CONN_ERROR;
 				goto end_req;
@@ -520,7 +651,7 @@ again:
 		
 		
 	end_req:
-		
+		if (bytes_read) *bytes_read=total_bytes;
 		return resp;
 }
 
@@ -538,12 +669,14 @@ void release_tcpconn(struct tcp_connection* c, long state, int unix_sock)
 		/* errno==EINTR, EWOULDBLOCK a.s.o todo */
 		response[0]=(long)c;
 		response[1]=state;
-		if (send_all(unix_sock, response, sizeof(response))<=0)
-			LOG(L_ERR, "ERROR: release_tcpconn: send_all failed\n");
+		
+		if (tsend_stream(unix_sock, (char*)response, sizeof(response), -1)<=0)
+			LOG(L_ERR, "ERROR: release_tcpconn: tsend_stream failed\n");
 }
 
 
-
+#ifdef DEBUG_TCP_RECEIVE
+/* old code known to work, kept arround for debuging */
 void tcp_receive_loop(int unix_sock)
 {
 	struct tcp_connection* list; /* list with connections in use */
@@ -557,7 +690,7 @@ void tcp_receive_loop(int unix_sock)
 	fd_set sel_set;
 	int maxfd;
 	struct timeval timeout;
-	int ticks;
+	ticks_t ticks;
 	
 	
 	/* init */
@@ -588,7 +721,7 @@ void tcp_receive_loop(int unix_sock)
 			if (FD_ISSET(unix_sock, &sel_set)){
 				nfds--;
 				/* a new conn from "main" */
-				n=receive_fd(unix_sock, &con, sizeof(con), &s);
+				n=receive_fd(unix_sock, &con, sizeof(con), &s, 0);
 				if (n<0){
 					if (errno == EWOULDBLOCK || errno == EAGAIN ||
 							errno == EINTR){
@@ -617,14 +750,14 @@ void tcp_receive_loop(int unix_sock)
 					release_tcpconn(con, resp, unix_sock);
 					goto skip;
 				}
-				con->timeout=get_ticks()+TCP_CHILD_TIMEOUT;
+				con->timeout=get_ticks_raw()+S_TO_TICKS(TCP_CHILD_TIMEOUT);
 				FD_SET(s, &master_set);
 				if (maxfd<s) maxfd=s;
 				if (con==list){
 					LOG(L_CRIT, "BUG: tcp_receive_loop: duplicate"
 							" connection received: %p, id %d, fd %d, refcnt %d"
 							" state %d (n=%d)\n", con, con->id, con->fd,
-							con->refcnt, con->state, n);
+							atomic_get(&con->refcnt), con->state, n);
 					resp=CONN_ERROR;
 					release_tcpconn(con, resp, unix_sock);
 					goto skip; /* try to recover */
@@ -632,12 +765,13 @@ void tcp_receive_loop(int unix_sock)
 				tcpconn_listadd(list, con, c_next, c_prev);
 			}
 skip:
-			ticks=get_ticks();
+			ticks=get_ticks_raw();
 			for (con=list; con ; con=c_next){
 				c_next=con->c_next; /* safe for removing*/
 #ifdef EXTRA_DEBUG
 				DBG("tcp receive: list fd=%d, id=%d, timeout=%d, refcnt=%d\n",
-						con->fd, con->id, con->timeout, con->refcnt);
+						con->fd, con->id, con->timeout,
+						atomic_get(&con->refcnt));
 #endif
 				if (con->state<0){
 					/* S_CONN_BAD or S_CONN_ERROR, remove it */
@@ -666,7 +800,7 @@ skip:
 					}
 				}else{
 					/* timeout */
-					if (con->timeout<=ticks){
+					if ((s_ticks_t)(ticks-con->timeout)>=0){
 						/* expired, return to "tcp main" */
 						DBG("tcp_receive_loop: %p expired (%d, %d)\n",
 								con, con->timeout, ticks);
@@ -680,15 +814,246 @@ skip:
 		
 	}
 }
+#else /* DEBUG_TCP_RECEIVE */
 
 
-#if 0
-int main(int argv, char** argc )
-{
-	printf("starting tests\n");
-	tcp_receive_loop();
+
+/* handle io routine, based on the fd_map type
+ * (it will be called from io_wait_loop* )
+ * params:  fm  - pointer to a fd hash entry
+ *          idx - index in the fd_array (or -1 if not known)
+ * return: -1 on error, or when we are not interested any more on reads
+ *            from this fd (e.g.: we are closing it )
+ *          0 on EAGAIN or when by some other way it is known that no more 
+ *            io events are queued on the fd (the receive buffer is empty).
+ *            Usefull to detect when there are no more io events queued for
+ *            sigio_rt, epoll_et, kqueue.
+ *         >0 on successfull read from the fd (when there might be more io
+ *            queued -- the receive buffer might still be non-empty)
+ */
+inline static int handle_io(struct fd_map* fm, int idx)
+{	
+	int ret;
+	int n;
+	struct tcp_connection* con;
+	int s;
+	long resp;
+	
+	switch(fm->type){
+		case F_TCPMAIN:
+again:
+			ret=n=receive_fd(fm->fd, &con, sizeof(con), &s, 0);
+			DBG("received n=%d con=%p, fd=%d\n", n, con, s);
+			if (n<0){
+				if (errno == EWOULDBLOCK || errno == EAGAIN){
+					ret=0;
+					break;
+				}else if (errno == EINTR) goto again;
+				else{
+					LOG(L_CRIT,"BUG: tcp_receive: handle_io: read_fd: %s \n",
+							strerror(errno));
+						abort(); /* big error*/
+				}
+			}
+			if (n==0){
+				LOG(L_ERR, "WARNING: tcp_receive: handle_io: 0 bytes read\n");
+				goto error;
+			}
+			if (con==0){
+					LOG(L_CRIT, "BUG: tcp_receive: handle_io null pointer\n");
+					goto error;
+			}
+			con->fd=s;
+			if (s==-1) {
+				LOG(L_ERR, "ERROR: tcp_receive: handle_io: read_fd:"
+									"no fd read\n");
+				goto con_error;
+			}
+			if (con==tcp_conn_lst){
+				LOG(L_CRIT, "BUG: tcp_receive: handle_io: duplicate"
+							" connection received: %p, id %d, fd %d, refcnt %d"
+							" state %d (n=%d)\n", con, con->id, con->fd,
+							atomic_get(&con->refcnt), con->state, n);
+				release_tcpconn(con, CONN_ERROR, tcpmain_sock);
+				break; /* try to recover */
+			}
+			/* must be before io_watch_add, io_watch_add might catch some
+			 * already existing events => might call handle_io and
+			 * handle_io might decide to del. the new connection =>
+			 * must be in the list */
+			tcpconn_listadd(tcp_conn_lst, con, c_next, c_prev);
+			con->timeout=get_ticks_raw()+S_TO_TICKS(TCP_CHILD_TIMEOUT);
+			if (io_watch_add(&io_w, s, F_TCPCONN, con)<0){
+				LOG(L_CRIT, "ERROR: tcp_receive: handle_io: failed to add"
+						" new socket to the fd list\n");
+				tcpconn_listrm(tcp_conn_lst, con, c_next, c_prev);
+				goto con_error;
+			}
+			break;
+		case F_TCPCONN:
+			con=(struct tcp_connection*)fm->data;
+			resp=tcp_read_req(con, &ret);
+			if (resp<0){
+				ret=-1; /* some error occured */
+				io_watch_del(&io_w, con->fd, idx, IO_FD_CLOSING);
+				tcpconn_listrm(tcp_conn_lst, con, c_next, c_prev);
+				con->state=S_CONN_BAD;
+				release_tcpconn(con, resp, tcpmain_sock);
+			}else{
+				/* update timeout */
+				con->timeout=get_ticks_raw()+S_TO_TICKS(TCP_CHILD_TIMEOUT);
+			}
+			break;
+		case F_NONE:
+			LOG(L_CRIT, "BUG: handle_io: empty fd map %p (%d): "
+						"{%d, %d, %p}\n", fm, (int)(fm-io_w.fd_hash),
+						fm->fd, fm->type, fm->data);
+			goto error;
+		default:
+			LOG(L_CRIT, "BUG: handle_io: uknown fd type %d\n", fm->type); 
+			goto error;
+	}
+	
+	return ret;
+con_error:
+	con->state=S_CONN_BAD;
+	release_tcpconn(con, CONN_ERROR, fm->fd);
+	return ret;
+error:
+	return -1;
 }
 
+
+
+/* releases expired connections and cleans up bad ones (state<0) */
+static inline void tcp_receive_timeout()
+{
+	struct tcp_connection* con;
+	struct tcp_connection* next;
+	ticks_t ticks;
+	
+	ticks=get_ticks_raw();
+	for (con=tcp_conn_lst; con; con=next){
+		next=con->c_next; /* safe for removing */
+		if (con->state<0){   /* kill bad connections */ 
+			/* S_CONN_BAD or S_CONN_ERROR, remove it */
+			/* fd will be closed in release_tcpconn */
+			io_watch_del(&io_w, con->fd, -1, IO_FD_CLOSING);
+			tcpconn_listrm(tcp_conn_lst, con, c_next, c_prev);
+			con->state=S_CONN_BAD;
+			release_tcpconn(con, CONN_ERROR, tcpmain_sock);
+			continue;
+		}
+		if ((s_ticks_t)(ticks-con->timeout)>=0){
+			/* expired, return to "tcp main" */
+			DBG("tcp_receive_loop: %p expired (%d, %d)\n",
+					con, con->timeout, ticks);
+			/* fd will be closed in release_tcpconn */
+			io_watch_del(&io_w, con->fd, -1, IO_FD_CLOSING);
+			tcpconn_listrm(tcp_conn_lst, con, c_next, c_prev);
+			release_tcpconn(con, CONN_RELEASE, tcpmain_sock);
+		}
+	}
+}
+
+
+
+void tcp_receive_loop(int unix_sock)
+{
+	
+	/* init */
+	tcpmain_sock=unix_sock; /* init com. socket */
+	if (init_io_wait(&io_w, tcp_max_fd_no, tcp_poll_method)<0)
+		goto error;
+	/* add the unix socket */
+	if (io_watch_add(&io_w, tcpmain_sock, F_TCPMAIN, 0)<0){
+		LOG(L_CRIT, "ERROR: tcp_receive_loop: init: failed to add socket "
+							" to the fd list\n");
+		goto error;
+	}
+	/* main loop */
+	switch(io_w.poll_method){
+		case POLL_POLL:
+				while(1){
+					io_wait_loop_poll(&io_w, TCP_CHILD_SELECT_TIMEOUT, 0);
+					tcp_receive_timeout();
+				}
+				break;
+#ifdef HAVE_SELECT
+		case POLL_SELECT:
+			while(1){
+				io_wait_loop_select(&io_w, TCP_CHILD_SELECT_TIMEOUT, 0);
+				tcp_receive_timeout();
+			}
+			break;
+#endif
+#ifdef HAVE_SIGIO_RT
+		case POLL_SIGIO_RT:
+			while(1){
+				io_wait_loop_sigio_rt(&io_w, TCP_CHILD_SELECT_TIMEOUT);
+				tcp_receive_timeout();
+			}
+			break;
+#endif
+#ifdef HAVE_EPOLL
+		case POLL_EPOLL_LT:
+			while(1){
+				io_wait_loop_epoll(&io_w, TCP_CHILD_SELECT_TIMEOUT, 0);
+				tcp_receive_timeout();
+			}
+			break;
+		case POLL_EPOLL_ET:
+			while(1){
+				io_wait_loop_epoll(&io_w, TCP_CHILD_SELECT_TIMEOUT, 1);
+				tcp_receive_timeout();
+			}
+			break;
+#endif
+#ifdef HAVE_KQUEUE
+		case POLL_KQUEUE:
+			while(1){
+				io_wait_loop_kqueue(&io_w, TCP_CHILD_SELECT_TIMEOUT, 0);
+				tcp_receive_timeout();
+			}
+			break;
+#endif
+#ifdef HAVE_DEVPOLL
+		case POLL_DEVPOLL:
+			while(1){
+				io_wait_loop_devpoll(&io_w, TCP_CHILD_SELECT_TIMEOUT, 0);
+				tcp_receive_timeout();
+			}
+			break;
+#endif
+		default:
+			LOG(L_CRIT, "BUG: tcp_receive_loop: no support for poll method "
+					" %s (%d)\n", 
+					poll_method_name(io_w.poll_method), io_w.poll_method);
+			goto error;
+	}
+error:
+	destroy_io_wait(&io_w);
+	LOG(L_CRIT, "ERROR: tcp_receive_loop: exiting...");
+	exit(-1);
+}
+
+#endif /* DEBUG_TCP_RECEIVE */
+
+#ifdef USE_STUN
+int is_msg_complete(struct tcp_req* r)
+{
+	if (r->has_content_len == 1) {
+		r->state = H_STUN_FP;
+		return 0;
+	}
+	else {
+		/* STUN message is complete */
+		r->state = H_STUN_END;
+		r->complete = 1;
+		r->has_content_len = 1; /* hack to avoid error check */
+		return 1;
+	}
+}
 #endif
 
-#endif
+#endif /* USE_TCP */
